/**
* @file GT2004BehaviorControl.h
*
* File modified by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file GT2004BehaviorControl.h
* 
* Implementation of class GT2004BehaviorControl.
*
* @author Martin Ltzsch
* @author Matthias Jngel
*/

#ifndef __GT2004BehaviorControl_h_
#define __GT2004BehaviorControl_h_

#include "../Xabsl2BehaviorControl.h"

#include "GT2004BasicBehaviors/GT2004SimpleBasicBehaviors.h"
#include "GT2004BasicBehaviors/GT2004EvolutionBasicBehaviors.h"
#include "GT2004BasicBehaviors/GT2004PotentialFieldBasicBehaviors.h"
#include "../CommonXabsl2BasicBehaviors/CommonBasicBehaviors.h"

#include "../CommonXabsl2Symbols/CalibrationSymbols.h"
#include "../CommonXabsl2Symbols/AngleSymbols.h"
#include "../CommonXabsl2Symbols/BallSymbols.h"
#include "../CommonXabsl2Symbols/HeadAndTailSymbols.h"
#include "../CommonXabsl2Symbols/JoystickSymbols.h"
#include "../CommonXabsl2Symbols/LEDAndSoundSymbols.h"
#include "../CommonXabsl2Symbols/KickSelectionSymbols.h"
#include "../CommonXabsl2Symbols/MathFunctions.h"
#include "../CommonXabsl2Symbols/MotionRequestSymbols.h"
#include "../CommonXabsl2Symbols/ObstaclesSymbols.h"
#include "../CommonXabsl2Symbols/RoboCupGameManagerSymbols.h"
#include "../CommonXabsl2Symbols/RobotPoseSymbols.h"
#include "../CommonXabsl2Symbols/RobotStateSymbols.h"
#include "../CommonXabsl2Symbols/SpecialVisionSymbols.h"
#include "../CommonXabsl2Symbols/ChallengeSymbols.h"
#include "GT2004Symbols/GT2004StrategySymbols.h"
#include "GT2004Symbols/GT2004ConfigurationSymbols.h"
#include "GT2004Symbols/EvolutionSymbols.h"
#include "GT2004Symbols/OpenChallengeSymbols.h"
#include "GT2004Symbols/TaskVisionSymbols.h"
#include "GT2004Symbols/SLAMSymbols.h"

#include "../KickLogger.h"


// Includes for DynamicTeamTactic
#include "Tools/DynamicTeamTactic/CollectedBeliefs.h"
#include "Tools/DynamicTeamTactic/OptionRating.h"
#include "Tools/DynamicTeamTactic/TacticChooser.h"
#include "Tools/DynamicTeamTactic/RateableOptions.h"
#include "GT2004Symbols/GT2004DTTSymbols.h"
#include "Tools/DynamicTeamTactic/ChooserRaterInterfaces.h"
#include "GT2004DTT/DefaultOptionRating.h"
#include "GT2004DTT/DefaultTacticChooser.h"


/**
* @class GT2004BehaviorControl
*
* A Behavior based on the Xabsl2Engine that is used by GermanTeam
* for the RoboCup 2004.
*
* @author Martin Ltzsch
* @author Matthias Jngel
*/ 
class GT2004BehaviorControl : public Xabsl2BehaviorControl
{
public:
  /** 
  * Constructor.
  * @param interfaces The paramters of the BehaviorControl module.
  */
  GT2004BehaviorControl(BehaviorControlInterfaces& interfaces);

  /** destructor */
  ~GT2004BehaviorControl();

  /** Registers symbols and basic behaviors at the engine */
  virtual void registerSymbolsAndBasicBehaviors();
  
  /** Sets some varibles for leds and sound before the execution of the engine */
  void preExecute();

  /** Executes the module */
  virtual void execute();

  /** Sets some variables for head motion and sound after the execution of the engine */
  void postExecute();

  /** 
  * Is called for every incoming debug message.
  * @param message An interface to read the message from the queue
  * @return if the messag was read
  */
  virtual bool handleMessage(InMessage& message);

private:
  //!@name Symbols
  //!@{
  CalibrationSymbols calibrationSymbols;
  AngleSymbols angleSymbols;
  BallSymbols ballSymbols;
  GT2004ConfigurationSymbols configurationSymbols;
  HeadAndTailSymbols headAndTailSymbols;
  JoystickSymbols joystickSymbols;
  LEDAndSoundSymbols ledAndSoundSymbols;
  KickSelectionSymbols kickSelectionSymbols;
  MathFunctions mathFunctions;
  MotionRequestSymbols motionRequestSymbols;
  ObstaclesSymbols obstaclesSymbols;
  RoboCupGameManagerSymbols roboCupGameManagerSymbols;
  RobotPoseSymbols robotPoseSymbols;
  RobotStateSymbols robotStateSymbols;
  SpecialVisionSymbols specialVisionSymbols;
  TaskVisionSymbols taskVisionSymbols;
  GT2004StrategySymbols strategySymbols;
  EvolutionSymbols evolutionSymbols;
  ChallengeSymbols challengeSymbols;
  OpenChallengeSymbols openchallengeSymbols;
	GT2004DTTSymbols dttSymbols;
	SLAMSymbols slamSymbols;
  //!@}

  //!@name Basic Behaviors
  //!@{
  GT2004SimpleBasicBehaviors simpleBasicBehaviors;
  GT2004EvolutionBasicBehaviors evolutionBasicBehaviors;
  GT2004PotentialFieldBasicBehaviors potentialFieldBasicBehaviors;
  CommonBasicBehaviors commonBasicBehaviors;
  //!@}

  //!@name GT2004 Evolution stuff
  //!@{
  GT2004ParametersSet gt2004ParametersSet;
  GT2004Parameters gt2004ExtraParameters;
  Pose2D gt2004ParametersCalibration[128];
  Pose2D measurementRequest;
  int evolutionMode;
  int gt2004CurrentIndex;
  //!@}

  /*
  * Runs in parallel to the behavior control and logs the preconditions
  * and results of each kick.
  */
  KickLogger kickLogger;

  /** If quickStop is true, the robot just stands */
  bool quickStop;

  /** Indicates which HeadControlMode is set, when the robot is demobilized (quick stop)*/
  HeadControlMode debugHeadControlMode;

	// DynamicTeamTactic Section
  ChooserRaterInterfaces chooserRaterInterfaces;
	
	// CollectedBeliefs Structure
	CollectedBeliefs collectedBeliefs;

	// OptionRating engine(s)
  OptionRating* optionRating[RateableOptions::numOfOptionRatings];
	// The actual choosen OptionRating
	OptionRating* actualOptionRating;

	// TacticChooser engine(s)
  TacticChooser* tacticChooser[RateableOptions::numOfTacticChoosers];
	// The actual choosen OptionRating
	TacticChooser* actualTacticChooser;

};

#endif// __GT2004BehaviorControl_h_

/* * Change log :
 * 
 * $Log: GT2004BehaviorControl.h,v $
 * Revision 1.14  2004/07/22 22:38:19  kerdels
 * added DTT used by Open Challenge, RIP and Xabsl-Options will follow
 *
 * Revision 1.13  2004/07/14 20:34:17  spranger
 * removed continous-basic-behaviors
 *
 * Revision 1.12  2004/07/10 00:13:59  spranger
 * renaming for coderelease and preparations for gt2005
 *
 * Revision 1.11  2004/06/22 18:48:56  juengel
 * kickAngles clean up
 *
 * Revision 1.10  2004/06/16 14:43:51  risler
 * added obstacles symbols
 * added symbol obstacles.opponent-close-to-ball
 *
 * Revision 1.9  2004/06/08 18:36:12  juengel
 * Added  "debugHeadControlMode".
 *
 * Revision 1.8  2004/06/02 10:56:25  juengel
 * quickStop added
 *
 * Revision 1.7  2004/05/29 18:18:19  dueffert
 * walk parameter evolution, measurement and calibration stuff ported to GT2004_WM
 *
 * Revision 1.6  2004/05/27 16:48:08  loetzsch
 * added the kick logger
 *
 * Revision 1.5  2004/05/25 12:58:17  tim
 * added missing potential field files and example behavior
 *
 * Revision 1.4  2004/05/24 16:35:20  spranger
 * removed doProfiling
 *
 * Revision 1.3  2004/05/22 21:33:33  loetzsch
 * adopted kick selection table to GT2004
 *
 * Revision 1.2  2004/05/22 21:31:04  loetzsch
 * adopted kick selection table to GT2004
 *
 * Revision 1.1.1.1  2004/05/22 17:17:59  cvsadm
 * created new repository GT2004_WM
 *
 * Revision 1.3  2004/05/08 16:18:13  hamerla
 * Open Challenge
 *
 * Revision 1.2  2004/05/04 10:48:58  loetzsch
 * replaced all enums
 * xxxBehaviorControlTimeAfterWhichCommunicatedBallsAreAccepted
 * by
 * behaviorControlTimeAfterWhichCommunicatedBallsAreAccepted
 * (this mechanism was neither fully implemented nor used)
 *
 * Revision 1.1  2004/05/02 13:26:42  juengel
 * Added GT2004BehaviorControl.
 *
 */
