/**
* @file GT2004ConfigurationSymbols.cpp
*
* File modified by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/** 
* @file GT2004ConfigurationSymbols.cpp
*
* Implementation of class GT2004ConfigurationSymbols.
*
* @author Max Risler
*/

#include "GT2004ConfigurationSymbols.h"
#include "Tools/FieldDimensions.h"


GT2004ConfigurationSymbols::GT2004ConfigurationSymbols(
                           BehaviorControlInterfaces& interfaces)
: BehaviorControlInterfaces(interfaces)
{
  currentKickoff = 0;

	// kickoff positions adapted to new rules
	goalie.x = xPosOwnGoalpost +150;
	goalie.y = yPosCenterGoal;

	// start position for defensive supporter (both kickoffs)
	defensiveSupporter.x = xPosOwnGoalpost/2-100;//behind own mid line
	defensiveSupporter.y = yPosLeftGroundline/2; 

	// start position for offensive supporter (opponent kickoffs)
	offensiveSupporterOpponentKickoff.x = xPosOwnGoalpost/2-100;//behind own mid line
	offensiveSupporterOpponentKickoff.y = yPosRightGroundline/2; 

	// start position for striker (opponent kickoffs)
	strikerOpponentKickoff.x = xPosOwnGoalpost/2-100; //behind own mid line
	strikerOpponentKickoff.y = 0;

	// even kickoffs
	offensiveSupporterOwnKickoff[0].x = xPosHalfWayLine-250;
	offensiveSupporterOwnKickoff[0].y = yPosRightGroundline/2;
	offensiveSupporterOwnKickoffAngle[0] = 30.0;

	strikerOwnKickoff[0].x = xPosHalfWayLine-centerCircleRadius-150;
	strikerOwnKickoff[0].y = yPosCenterGoal;
	strikerOwnKickoffAngle[0] = 0;

	// long kickoff to the left
	offensiveSupporterOwnKickoff[1].x = xPosHalfWayLine-250;
	offensiveSupporterOwnKickoff[1].y = yPosRightGroundline/2;
	offensiveSupporterOwnKickoffAngle[1] = 30.0;
	strikerOwnKickoff[1].x = xPosHalfWayLine-centerCircleRadius-150;
	strikerOwnKickoff[1].y = yPosCenterGoal;
	strikerOwnKickoffAngle[1] = 0.0;

}

void GT2004ConfigurationSymbols::registerSymbols(Xabsl2Engine& engine)
{
  engine.registerDecimalInputSymbol("current-kickoff-variant",this,
    (double (Xabsl2FunctionProvider::*)())&GT2004ConfigurationSymbols::getCurrentKickoff);

  engine.registerDecimalInputSymbol("own-kickoff.angle",this,
    (double (Xabsl2FunctionProvider::*)())&GT2004ConfigurationSymbols::getOwnKickoffAngle);
  engine.registerDecimalInputSymbol("own-kickoff.x",this,
    (double (Xabsl2FunctionProvider::*)())&GT2004ConfigurationSymbols::getOwnKickoffX);
  engine.registerDecimalInputSymbol("own-kickoff.y",this,
    (double (Xabsl2FunctionProvider::*)())&GT2004ConfigurationSymbols::getOwnKickoffY);

  engine.registerDecimalInputSymbol("opponent-kickoff.angle",this,
    (double (Xabsl2FunctionProvider::*)())&GT2004ConfigurationSymbols::getOpponentKickoffAngle);
  engine.registerDecimalInputSymbol("opponent-kickoff.x",this,
    (double (Xabsl2FunctionProvider::*)())&GT2004ConfigurationSymbols::getOpponentKickoffX);
  engine.registerDecimalInputSymbol("opponent-kickoff.y",this,
    (double (Xabsl2FunctionProvider::*)())&GT2004ConfigurationSymbols::getOpponentKickoffY);
}

void GT2004ConfigurationSymbols::update()
{
  currentKickoff = gameControlData.data.opponentScore % numOfKickoff;

  // calculate kickoff position of striker based on obstacle-model
  double radius = 250.0;

  double leftAbs = fromDegrees(10.0);
  double rightAbs = fromDegrees(55.0);
  double freeWidth = fromDegrees(15.0);

  double step = fromDegrees(5.0);
  double bestAngle = 0.0;
  double bestDist = 0.0;
  
  double start = 0.0, end = 0.0;
  enum { left, right } direction = left;

  switch (currentKickoff % 2)
  {
    case 0: // right
      start = -1 * leftAbs;
      end = -1 * rightAbs;
      step *= -1;
      direction = right;
      break;
    case 1: // left
      start = leftAbs;
      end = rightAbs;
      step = step;
      direction = left;
      break;
  }

  for(
    double ang = start;
    (direction == left && ang <= end) || (direction == right && ang >= end);
    ang += step
  )
  {
    double currDist = obstaclesModel.getDistanceInCorridor(ang, freeWidth);
    double angAbs = ang - robotPose.getAngle();
    currDist = obstaclesModel.getDistanceInDirection(angAbs, freeWidth);
    if(currDist > bestDist)
    {
      bestDist = currDist;
      bestAngle = ang;
    }
  }

  double posX = - cos ( bestAngle ) * radius;
  double posY = - sin ( bestAngle ) * radius;

  obstacleKickoff.translation.x = posX;
  obstacleKickoff.translation.y = posY;
  obstacleKickoff.fromAngle(toDegrees(bestAngle));
}

double GT2004ConfigurationSymbols::getCurrentKickoff()
{
  return currentKickoff;
}

double GT2004ConfigurationSymbols::getOwnKickoffAngle()
{
  switch (getPlayer().getPlayerNumber())
  {
    case Player::one: // robot number 1
		return 0;
    case Player::two: // robot number 2
    default:
      return -20;
    case Player::three: // robot number 3
      return offensiveSupporterOwnKickoffAngle[currentKickoff];
    case Player::four: // robot number 4
      //return getOwnKickoffAngleByObstacles();
      return strikerOwnKickoffAngle[currentKickoff];
  }
}

double GT2004ConfigurationSymbols::getOwnKickoffX()
{
  switch (getPlayer().getPlayerNumber())
  {
    case Player::one: // robot number 1
    default:
      return goalie.x;
    case Player::two: // robot number 2
      return defensiveSupporter.x;
    case Player::three: // robot number 3
      return offensiveSupporterOwnKickoff[currentKickoff].x;
    case Player::four: // robot number 4
      //return getOwnKickoffXByObstacles();
      return strikerOwnKickoff[currentKickoff].x;
  }
}

double GT2004ConfigurationSymbols::getOwnKickoffY()
{
  switch (getPlayer().getPlayerNumber())
  {
    case Player::one: // robot number 1
    default:
      return goalie.y;
    case Player::two: // robot number 2
      return defensiveSupporter.y;
    case Player::three: // robot number 3
      return offensiveSupporterOwnKickoff[currentKickoff].y;
    case Player::four: // robot number 4
      //return getOwnKickoffYByObstacles();
      return strikerOwnKickoff[currentKickoff].y;
  }
}

double GT2004ConfigurationSymbols::getOwnKickoffXByObstacles()
{
  return obstacleKickoff.translation.x;
}

double GT2004ConfigurationSymbols::getOwnKickoffYByObstacles()
{
  return obstacleKickoff.translation.y;
}

double GT2004ConfigurationSymbols::getOwnKickoffAngleByObstacles()
{
  return obstacleKickoff.getAngle();
}

double GT2004ConfigurationSymbols::getOpponentKickoffAngle()
{
  switch (getPlayer().getPlayerNumber())
  {
    case Player::one: // robot number 1
		return 0;
    case Player::two: // robot number 2
		return -20;
    case Player::three: // robot number 3
		return 20;
    case Player::four: // robot number 4
    default:
      return 0;
  }
}

double GT2004ConfigurationSymbols::getOpponentKickoffX()
{
  switch (getPlayer().getPlayerNumber())
  {
    case Player::one: // robot number 1
    default:
      return goalie.x;
    case Player::two: // robot number 2
      return defensiveSupporter.x;
    case Player::three: // robot number 3
      return offensiveSupporterOpponentKickoff.x;
    case Player::four: // robot number 4
      return strikerOpponentKickoff.x;
  }
}

double GT2004ConfigurationSymbols::getOpponentKickoffY()
{
  switch (getPlayer().getPlayerNumber())
  {
    case Player::one: // robot number 1
    default:
      return goalie.y;
    case Player::two: // robot number 2
      return defensiveSupporter.y;
    case Player::three: // robot number 3
      return offensiveSupporterOpponentKickoff.y;
    case Player::four: // robot number 4
      return strikerOpponentKickoff.y;
  }
}

/*
* Change Log
* 
* $Log: GT2004ConfigurationSymbols.cpp,v $
* Revision 1.3  2004/09/08 12:41:43  wachter
* - Fixed some warnings
* - Fixed some documentation in network-code
*
* Revision 1.2  2004/07/02 16:52:55  thomas
* modified kickoff
* - striker positioning to angle based on obstacle-model
* - supporter not so near to side-border and a bit more away from middle-line
*
* Revision 1.1.1.1  2004/05/22 17:18:02  cvsadm
* created new repository GT2004_WM
*
* Revision 1.1  2004/05/02 13:23:45  juengel
* Added GT2004BehaviorControl.
*
*/

