/**
* @file ColorClustering.cpp
* 
* 
* @author Jurgen Sturm
*/

#include "ColorClustering.h"
#include "Tools/Debugging/Debugging.h"
#include "Tools/Debugging/Stopwatch.h"

inline unsigned char colorDistance(Vector3<unsigned char> &color1,Vector3<unsigned char> &color2) {
	int ax = color1.x,
		ay = color1.y,
		az = color1.z,
		bx = color2.x,
		by = color2.y,
		bz = color2.z;

	int distance = 
						abs(ax-bx) +
						abs(ay-by) +
						abs(az-bz) +
						abs(abs(ay-az) - abs(by-bz));

	if(distance>255) return 255;
	return distance;
}

// **************************************************************************** ColorRecordScanParam
ColorRecordScanParam::ColorRecordScanParam() {
	threshold = 256;	// never reach
}

void ColorRecordScanParam::reset() {
	count = 0;
	for(int i=0;i<256;i++) histogram[i]=0;
}

void ColorRecordScanParam::recomputeThreshold() {
	// compute cutoff
	int N = (int)(count * COLOR_EDGE_ABOVE);
	int sum =0;
	threshold = 0;
	while(threshold<255 && sum<N) sum += histogram[threshold++];
//	OUTPUT(idText,text,"ColorRecordScanParam.threshold="<<threshold);
}

// **************************************************************************** ColorCollection
ColorCollection::ColorCollection() {
	length = virtualLength = 0;
}

void ColorCollection::insertColor(Vector3<unsigned char>aColor) {
	if(length<COLLECT_COLORS) {
		color[length++] = aColor;
		virtualLength++;
	} else {
		int position = rand() % (virtualLength+1);	// guess a position in virtual collection
		if(position<COLLECT_COLORS) {
			color[position]=aColor;
		}
		virtualLength++;
	}
}

void ColorCollection::showCollectedColors(Image &debugImage) {
	WINDEBUG(

		for(int iy=0;iy<cameraResolutionHeight_ERS7;iy++) 
			for(int ix=0;ix<cameraResolutionWidth_ERS7;ix++) {
				debugImage.image[iy][0][ix] = 128;
				debugImage.image[iy][1][ix] = 128;
				debugImage.image[iy][2][ix] = 128;
				}
		//memset(debugImage.image,0,sizeof(debugImage.image));

		// get the scaling right
		Vector3<unsigned char> low(255,255,255);
		Vector3<unsigned char> high(0,0,0);
		int i;
		for(i=0;i<length;i++) {
			Vector3<unsigned char> &c = color[i];
			low.x = min(low.x,c.x);
			low.y = min(low.y,c.y);
			low.z = min(low.z,c.z);
			high.x = max(high.x,c.x);
			high.y = max(high.y,c.y);
			high.z = max(high.z,c.z);
		}
		high.x = max(high.x,low.x+10);
		high.y = max(high.y,low.y+10);
		high.z = max(high.z,low.z+10);

		int cx;
		int cy;
		int cz;
		int x;
		int y;
		for(i=0;i<length;i++) {
			Vector3<unsigned char> c = color[i];
			cx = ((int)c.x-(high.x+low.x)/2)*cameraResolutionWidth_ERS7/(high.x-low.x);
			cy = ((int)c.y-(high.y+low.y)/2)*cameraResolutionWidth_ERS7/(high.y-low.y);
			cz = ((int)c.z-(high.z+low.z)/2)*cameraResolutionHeight_ERS7/(high.z-low.z);


			//x = cx;
			//y = cameraResolutionHeight_ERS7*2/3 + 
	//			abs(cy+cz)*((cy>cz)?(-(cy-cz)/2) : ((cz-cy)/2))/(cameraResolutionHeight_ERS7/2);
	//
			x = cameraResolutionWidth_ERS7/2+cy*2/3+cx/4;
			y = cameraResolutionHeight_ERS7/2+cz/2-cx/2;

			x= max(min(x,cameraResolutionWidth_ERS7-1),0);
			y= max(min(y,cameraResolutionHeight_ERS7-1),0);

			int it=0;
			while(debugImage.image[y][0][x] != 128 && it<10) {
				x= max(min(x+rand() % 3 - 1,cameraResolutionWidth_ERS7-1),0);
				y= max(min(y+rand() % 3 - 1,cameraResolutionHeight_ERS7-1),0);
				it++;
			}
		
			// normalize color coords
			//c.x = ((int)c.x-low.x)*255/(high.x-low.x);
			//c.y = ((int)c.y-low.y)*255/(high.y-low.y);
			//c.z = ((int)c.z-low.z)*255/(high.z-low.z);

			debugImage.image[y][0][x] = c.x;
			debugImage.image[y][1][x] = c.y;
			debugImage.image[y][2][x] = c.z;
		}
	)
}

// **************************************************************************** ColorHistogram
ColorHistogram::ColorHistogram(ColorCollection &colorCollection) {
	for(int x=0;x<COLOR_CUBES;x++) 
	for(int y=0;y<COLOR_CUBES;y++) 
	for(int z=0;z<COLOR_CUBES;z++) 
		count[x][y][z] = 0;

	for(int i=0;i<colorCollection.length;i++) {
		Vector3<unsigned char> &c = colorCollection.color[i];
		if(insertColor(c))
			strongColorsRemoved.insertColor(c);
	}

//	OUTPUT(idText,text,"ColorHistogram raw colors="<<colorCollection.length<<", removing strong colors "<<strongColorsRemoved.length);

}

bool ColorHistogram::insertColor(Vector3<unsigned char>color) {
	if(count[color.x*COLOR_CUBES/256][color.y*COLOR_CUBES/256][color.z*COLOR_CUBES/256]==255) 
		return false;
	else 
		return (rand() % ++count[color.x*COLOR_CUBES/256][color.y*COLOR_CUBES/256][color.z*COLOR_CUBES/256]) == 0;
}

// **************************************************************************** ColorClustering
ColorClustering::ColorClustering(const Image &image,const CameraMatrix &cameraMatrix,const CameraMatrix &prevCameraMatrix,Image &debugImage)
:ScanningGrid(image,cameraMatrix,prevCameraMatrix),debugImage(debugImage),mixture(false,false)
{
	//OUTPUT(idText,text,"sizeof(ColorClustering)="<<sizeof(ColorClustering));
	//OUTPUT(idText,text,"sizeof(ColorCollection)="<<sizeof(ColorCollection));
	//OUTPUT(idText,text,"sizeof(ColorHistogram)="<<sizeof(ColorHistogram));
	//OUTPUT(idText,text,"sizeof(EMGaussianMixture)="<<sizeof(EMGaussianMixture));

	scanningDistanceY = 2;
	scanningDistanceX = 2;

	colorCollection = NULL;
	colorHistogram = NULL;
}
  

// reserves memory for colorCollection
void ColorClustering::recordPrepare() {
	if(colorCollection != NULL) 
		delete colorCollection;
	colorCollection = new ColorCollection();
//	OUTPUT(idText,text,"ColorClustering::recordPrepare()" );
}

// using the scanning grid, extracts interesting colors
void ColorClustering::recordExecute() {
	WINDEBUG(
		debugImage = image;
	)

	recordScanParam.reset();

//	STOPWATCH("SCAN_IMAGE ",SCAN_IMAGE(false,false,true,scanPixel,recordScanParam);	)
	SCAN_IMAGE(false,false,true,scanPixel,recordScanParam);

	recordScanParam.recomputeThreshold();

	//OUTPUT(idText,text,"ColorClustering::recordExecute(), virtualLength="<<colorCollection->virtualLength );
}

// prepares colorHistogram and colorMatrix, finally destroys colorCollection and colorHistogram
void ColorClustering::clusteringPrepare() {
	colorHistogram = new ColorHistogram(*colorCollection);
	colorHistogram->strongColorsRemoved.showCollectedColors(debugImage);
	mixture.initRandom();

	Vector3<double> *training;
	training = (Vector3<double> *)calloc(colorHistogram->strongColorsRemoved.length,sizeof(Vector3<double>));

	//unsigned char c1,c2,c3;
	for(int i=0;i<colorHistogram->strongColorsRemoved.length;i++) {
		training[i] = Vector3<double>(
										colorHistogram->strongColorsRemoved.color[i].x,
										colorHistogram->strongColorsRemoved.color[i].y,
										colorHistogram->strongColorsRemoved.color[i].z
									);
	}
	mixture.setTrainingSet(&training[0],colorHistogram->strongColorsRemoved.length);
	mixture.initMeansOnTrainingSet();

}

// merge close clusters
void ColorClustering::clusteringExecute() {
	mixture.startEM(1);
}

// fills mean, weight, lookupTable, finally destroys ColorMatrix
void ColorClustering::clusteringFinish() {
	free(mixture.trainingData);
	mixture.setTrainingSet(NULL,0);
	delete colorHistogram;
	colorHistogram = NULL;
	delete colorCollection;
	colorCollection = NULL;
}

inline bool ColorClustering::scanPixel(
		int &is,int &ix,int &iy,
		int &x,int &y,
		Vector3<unsigned char> &color,
		double &radScanline,
		ColorRecordScanParam &param) {

	int dist = colorDistance(color,param.lastColor);
	param.histogram[dist]++;
	param.count++;


	if(iy!=0 && dist>=param.threshold) {
		colorCollection->insertColor(color);
		colorCollection->insertColor(param.lastColor);

		WINDEBUG(
			debugImage.image[y][0][x] = 255;
		)
	}

	param.lastColor = color;
	return(false);
}

unsigned char ColorClustering::assignCluster(Vector3<unsigned char> &color) {
	return(mixture.lookupTable[color.x>>2][color.y>>2][color.z>>2]);
}

void ColorClustering::saveLookupTable(const char* filename) {
	mixture.saveLookupTable(filename);
}

bool ColorClustering::loadLookupTable(const char* filename) {
	return(mixture.loadLookupTable(filename));
}

void ColorClustering::clusterImage() {
	WINDEBUG(
		debugImage = image;
		ColorCorrector::correct(debugImage);
		mixture.clusterImage(debugImage);
		mixture.visualizeClusters(debugImage);
	)
}

/*
*
*/
