/**
* @file FastPanoramaProcessor.cpp
* 
* 
* @author Jurgen Sturm
*/

#include "FastPanoramaProcessor.h"
#include "Platform/SystemCall.h"
#include "Tools/RobotConfiguration.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Tools/Debugging/Debugging.h"
#include "Tools/Debugging/Stopwatch.h"
#include "Modules/ImageProcessor/ImageProcessorTools/ColorCorrector.h"

FastPanoramaProcessor::FastPanoramaProcessor(const ImageProcessorInterfaces& interfaces)
: ImageProcessor(interfaces),

	cluster((const ColorClustering&) ColorClustering(
		interfaces.image,
		interfaces.cameraMatrix,
		prevCameraMatrix,
		clusterDebug)),

	signatureGenerator((const SignatureGenerator&) SignatureGenerator(
		interfaces.image,interfaces.
		cameraMatrix,prevCameraMatrix,
		cluster,landmarksPercept,
		robotPose,
		signatureDebug)),

	horizonEstimator((const HorizonEstimator&) HorizonEstimator(
		interfaces.image,interfaces.
		cameraMatrix,prevCameraMatrix,
		cluster,landmarksPercept,
		robotPose,
		signatureDebug))

{
	TEXTLOG(idText,text,"FastPanoramaProcessor: "<<CLUSTERS<<" clusters, "<<(PLAIN_COLORS_ONLY?"plain colors only":"color transitions"))  ;
	statEmptyFrames = statFrameCount = 0;

	if(cluster.loadLookupTable("cluster.tab")) {
		TEXTLOG(idText,text,"SUCCESS: loaded lookup table");
	}
}


void FastPanoramaProcessor::execute()
{ 
//	STOPWATCH("FastPanoramaProcessor::framePrepare() ",framePrepare();)
//	STOPWATCH("FastPanoramaProcessor::frameExecute() ",frameExecute();)
//	STOPWATCH("FastPanoramaProcessor::frameFinish() ",frameFinish();)
	framePrepare();
	frameExecute();
	frameFinish();
}

void FastPanoramaProcessor::framePrepare() {
  static bool prevCameraMatrixUninitialized = true;
  if (prevCameraMatrixUninitialized && cameraMatrix.frameNumber != 0)
  {
    prevCameraMatrix = cameraMatrix;
		prevCameraMatrixUninitialized = false;
		Geometry::setupRadialCorrection(image.cameraInfo);
		ColorCorrector::load();
  }

/*
  if(image.frameNumber != 0
	&&
	abs(image.frameNumber-robotPose.frameNumber)>200) {
  	  OUTPUT(idText,text,"frameNumber = "<<image.frameNumber);
  }
*/

  landmarksPercept.frameNumber = image.frameNumber;


}

void FastPanoramaProcessor::frameExecute() {
	if(slamData.currentAction==SLAMData::offline) return;

	switch(slamData.currentAction) {
		case SLAMData::resetImageProcessor: 
			//OUTPUT(idText,text,"imageprocessor.reset");
			statEmptyFrames = statFrameCount = 0;
			slamData.actionCompleted(true);
			break;

		case SLAMData::resetSelfLocator:
			break; 

		case SLAMData::standby: 
			break;

		case SLAMData::loadClusters: 
			//slamData.actionCompleted(false);			// loading deactivated

			if(cluster.loadLookupTable("cluster.tab")) {
				TEXTLOG(idText,text,"using pre-computed lookup table")  ;
				for(int c=0;c<CLUSTERS;c++)
					slamData.clusterColor[c] = cluster.mixture.clusterColor[c];
				slamData.actionCompleted(true);
			} else {
				TEXTLOG(idText,text,"no lookup table found. starting calibration")  ;
				slamData.actionCompleted(false);
			}
			break;

		case SLAMData::recordColors:
			switch(slamData.actionIteration) {
				case 0:
					cluster.recordPrepare();
					slamData.processingAndBusy();
					break;
				default:
					cluster.recordExecute();
					slamData.processingAndBusy();
					break;
			}

#if defined(_WIN32)
			if(cluster.colorCollection->virtualLength > 10000) {
				slamData.actionCompleted(true);
			}
#else
			if(cluster.colorCollection->virtualLength > 100000) {
				slamData.actionCompleted(true);
			}
#endif

			break;

		case SLAMData::clusterColors:
			switch(slamData.actionIteration) {
				case 0:
					TEXTLOG(idText,text,"starting clustering process on "<<cluster.colorCollection->virtualLength<<" colors");
			//		STOPWATCH("FastPanoramaProcessor::cluster.clusteringPrepare() ",cluster.clusteringPrepare();)
					cluster.clusteringPrepare();
					slamData.processingAndBusy();
					break;
				case 1:
				case 2:
				case 3:
				case 4:
				case 5:
				case 6:
				case 7:
				case 8:
				case 9:
				case 10:
			//		STOPWATCH("FastPanoramaProcessor::cluster.clusteringExecute() ",cluster.clusteringExecute();)
					cluster.clusteringExecute();
					slamData.processingAndBusy();
					break;
				case 11:
				case 12:
				case 13:
				case 14:
				case 15:
				case 16:
				case 17:
				case 18:
					//STOPWATCH("FastPanoramaProcessor::cluster.mixture.updateLookupTable()",
						cluster.mixture.updateLookupTable(slamData.actionIteration-11);
						//cluster.mixture.updateLookupTable();
					//)
					slamData.processingAndBusy();
					break;
				case 19:
					//STOPWATCH("FastPanoramaProcessor::cluster.clusteringFinish() ",cluster.clusteringFinish();)
					cluster.clusteringFinish();
					for(int c=0;c<CLUSTERS;c++)
						slamData.clusterColor[c] = cluster.mixture.clusterColor[c];
					slamData.actionCompleted(true);
					break;
				}
			break;

		case SLAMData::saveClusters: 
			cluster.saveLookupTable("cluster.tab");
			cluster.clusterImage();
			
			slamData.actionCompleted(true);
			
			break;

		default:
			if(slamData.currentAction >= SLAMData::clustersAvailable) {
				cluster.clusterImage();
				signatureGenerator.execute();

				int count=0;
				for(int s=0;s<SECTORS;s++) {
					if(!landmarksPercept.panoramaHistogram.sector[s].empty)
						count++;
				}

				if(count==0) statEmptyFrames++;
				statFrameCount++;

				if(slamData.actionIteration % 30 == 0) {
					if(count==0)
						OUTPUT(idText,text,
							" frames localized "<< statFrameCount <<"  ("<<statEmptyFrames<<" empty)"
							);

					/*
					if(count==0) {
					OUTPUT(idText,text,
						image.image[10][0][10]<<"-"<<image.image[10][1][10]<<"-"<<image.image[10][2][10]<<"   "<<
						image.image[20][0][50]<<"-"<<image.image[20][1][50]<<"-"<<image.image[20][2][50]<<"   "<<
						image.image[30][0][100]<<"-"<<image.image[30][1][100]<<"-"<<image.image[30][2][100]<<"   "<<
						image.image[40][0][100]<<"-"<<image.image[40][1][100]<<"-"<<image.image[40][2][100]<<"   "
						);
						
					}
					*/
				}
				//horizonEstimator.execute();
			} else 
				OUTPUT(idText,text,"FastPanoramaProcessor::execute() unknown SLAM action: "<<slamData.getActionName(slamData.currentAction))  ;
			

	}

}

void FastPanoramaProcessor::frameFinish() {

	INIT_DEBUG_IMAGE(imageProcessorBall,clusterDebug);
	INIT_DEBUG_IMAGE(imageProcessorGoals,signatureDebug);

	SEND_DEBUG_IMAGE(imageProcessorBall);
	SEND_DEBUG_IMAGE(imageProcessorGoals);

//	DEBUG_DRAWING_FINISHED(selfLocator); 
//	DEBUG_DRAWING_FINISHED(selfLocatorField);  

  prevCameraMatrix = cameraMatrix;
}

/*
*
*/
