/**
* @file SignatureGenerator.cpp
* 
* 
* @author Jurgen Sturm
*/

#include "HorizonEstimator.h"
#include "Tools/Debugging/Debugging.h"

HorizonScanData::HorizonScanData(PanoramaHistogram &panoramaHistogram)
	:	panoramaHistogram(panoramaHistogram) 
{
	resetDistribution();

}

void inline HorizonScanData::resetDistribution() {
	for(int i=0;i<2;i++)	
	for(int cluster=0;cluster<CLUSTERS;cluster++)
		distribution[i][cluster] = 0;

	horizonHeightSum = horizonHeightCount = 1;
}

void inline HorizonScanData::resetSectorData() {
	sector = -1;
	seenTop = false;
	seenBottom = false;
	for(int i=0;i<HORIZON_SIZE;i++)	
	for(int cluster=0;cluster<CLUSTERS;cluster++)
		freq[i][cluster] = 0;
}

int inline HorizonScanData::estimateMLHorizonHeight() {
	double likelihoodAbove[HORIZON_SIZE];//likelihood that pixel comes from above horizon
	double likelihoodBelow[HORIZON_SIZE];
	int i,diff;

	for(i=0;i<HORIZON_SIZE;i++)	{
		likelihoodAbove[i] = 0;//normalize later
		likelihoodBelow[i] = 0;
		diff = 0;
		for(int cluster=0;cluster<CLUSTERS;cluster++) {
			if(distribution[0][cluster] + distribution[1][cluster]==0) continue;
			if(freq[i][cluster]==0) continue;

			likelihoodAbove[i] += 
				(distribution[0][cluster]) / (double)(distribution[0][cluster] + distribution[1][cluster])
				* freq[i][cluster];

			likelihoodBelow[i] += 
				(distribution[1][cluster]) / (double)(distribution[0][cluster] + distribution[1][cluster])
				* freq[i][cluster];
			
		}
		//likelihood[i] = -log(1+exp(likelihoodBelow - likelihoodAbove));
		// floating point trouble
	}

	double sumAbove[HORIZON_SIZE];
	double sumBelow[HORIZON_SIZE];

	sumAbove[0] = likelihoodAbove[0];
	for(i=1;i<HORIZON_SIZE;i++)	
		sumAbove[i] = sumAbove[i-1] + likelihoodAbove[i];

	sumBelow[HORIZON_SIZE-1] = likelihoodBelow[HORIZON_SIZE-1];
	for(i=HORIZON_SIZE-1;i>0;i--)	
		sumBelow[i] = sumBelow[i+1] + (likelihoodBelow[i]);

	// now find max
	int bestHeight = -1;
	double bestSum = -1;
	for(i=10;i<HORIZON_SIZE-10;i++)	
		if(sumAbove[i] - sumAbove[i-10] + sumBelow[i] - sumBelow[i+10]> bestSum || bestHeight<0) {
			bestHeight = i;
			bestSum = sumAbove[i] - sumAbove[i-10] + sumBelow[i] - sumBelow[i+10];
		}

	int height = (bestHeight-HORIZON_SIZE/2+1);

	horizonHeightSum += height;
	horizonHeightCount++;

	return height;
}

HorizonEstimator::HorizonEstimator(
			const Image &image,
			const CameraMatrix &cameraMatrix,
			const CameraMatrix &prevCameraMatrix, 
			ColorClustering &cluster, 
			LandmarksPercept &landmarksPercept,
			const RobotPose &robotPose,
			Image &debugImage)
: 
	ScanningGrid(image,cameraMatrix,prevCameraMatrix),
	cluster(cluster),
	debugImage(debugImage),
	robotPose(robotPose),
	landmarksPercept(landmarksPercept),
	horizonScanData(landmarksPercept.panoramaHistogram) 
{
//	OUTPUT(idText,text,"sizeof(HorizonEstimator)="<<sizeof(HorizonEstimator));

	scanningDistanceY = 1;
	scanningDistanceX = 1;
	maxVerticalScanPoints = HORIZON_SIZE/2;
}

void HorizonEstimator::execute() {

	WINDEBUG(
		int x;
		int y;
		for(y=0;y<cameraResolutionHeight_ERS7;y++) {
			for(x=0;x<cameraResolutionWidth_ERS7;x++) {
				debugImage.image[y][0][x] = image.image[y][0][x];
				debugImage.image[y][1][x] = image.image[y][1][x];
				debugImage.image[y][2][x] = image.image[y][2][x];
			}
		}
	)

	horizonScanData.resetSectorData();
	SCAN_IMAGE(true,false,true,scanPixel,horizonScanData);

}

inline bool HorizonEstimator::scanPixel(
		int &is,int &ix,int &iy,
		int &x,int &y,
		Vector3<unsigned char> &color,
		double &radScanline,
		HorizonScanData &horizonScanData) {

	unsigned char currentCluster = cluster.mixture.lookupTable[color.x>>2][color.y>>2][color.z>>2];

	if(iy==0 && is==0) {
		int currentSector = (int)((-radScanline/(2*PI) + 0.5)*SECTORS);// same formula as in PanoramaImageProcessor

		if(currentSector != horizonScanData.sector) {
			if(horizonScanData.seenTop && horizonScanData.seenBottom) {
				// analyze previous sector
				int height = horizonScanData.estimateMLHorizonHeight();

				debugImage.image[y+height][0][x] = 255;
				debugImage.image[y+height][1][x] = 128;
				debugImage.image[y+height][2][x] = 128;

				debugImage.image[y+horizonScanData.horizonHeightSum/horizonScanData.horizonHeightCount][0][x] = 255;
				debugImage.image[y+horizonScanData.horizonHeightSum/horizonScanData.horizonHeightCount][1][x] = 0;
				debugImage.image[y+horizonScanData.horizonHeightSum/horizonScanData.horizonHeightCount][2][x] = 128;

				debugImage.image[y][0][x] = 255;
				debugImage.image[y][1][x] = 128;
				debugImage.image[y][2][x] = 0;

			}
			horizonScanData.resetSectorData();
			horizonScanData.sector = currentSector;
		}

		/*
		WINDEBUG(
			if(horizonScanData.sector%2 == 0) {
				debugImage.image[y][0][x] = 255;
				debugImage.image[y][1][x] = 128;
				debugImage.image[y][2][x] = 0;
			} else {
				debugImage.image[y][0][x] = 255;
				debugImage.image[y][1][x] = 128;
				debugImage.image[y][2][x] = 128;
			}
		)
		*/


		return(false);	// skip this point once (because horizon is seen twice: in the above-horizon-half and vice versa.
	}

	if(iy==HORIZON_SIZE/2-1) {
		if(is==0) 
			horizonScanData.seenTop = true; 
		else 
			horizonScanData.seenBottom = true;
	}

	// learn the histograms of both sides
	horizonScanData.distribution[is][currentCluster]++;

	// store seen cluster at this height
	horizonScanData.freq[is?(iy+HORIZON_SIZE/2):(HORIZON_SIZE/2-iy-1)][currentCluster]++;

	return(false);
}


/*
*
*/
