/**
* @file ScanningGrid.cpp
* 
* 
* @author Jurgen Sturm
*/

#include "ScanningGrid.h"
#include "Tools/Debugging/Debugging.h"
#include "Modules/ImageProcessor/ImageProcessorTools/BresenhamLineScan.h"

ScanningGrid::ScanningGrid(const Image &image,const CameraMatrix &cameraMatrix,const CameraMatrix &prevCameraMatrix):
	image(image), cameraMatrix(cameraMatrix),prevCameraMatrix(prevCameraMatrix),imageInfo()
{
//	OUTPUT(idText,text,"sizeof(ScanningGrid)="<<sizeof(ScanningGrid))  ;
	scanningDistanceX = 4;
	scanningDistanceY = 4;
	maxVerticalScanPoints = DIAGONAL;
}


void ScanningGrid::prepare() {
	prepareImageInfo();
	prepareHorizontalScanline();
	prepareVerticalScanline();
}

void ScanningGrid::prepareImageInfo() {
	// Compute additional information about the image
	imageInfo.maxImageCoordinates.x = image.cameraInfo.resolutionWidth - 1;
	imageInfo.maxImageCoordinates.y = image.cameraInfo.resolutionHeight - 1;
	static CameraMatrix oldMatrix;
	imageInfo.horizon = Geometry::calculateHorizon(cameraMatrix, image.cameraInfo);
	imageInfo.horizonInImage = Geometry::getIntersectionPointsOfLineAndRectangle(
	Vector2<int>(0,0), imageInfo.maxImageCoordinates, imageInfo.horizon, 
					imageInfo.horizonStart, imageInfo.horizonEnd);
	imageInfo.vertLine = imageInfo.horizon;
	imageInfo.vertLine.direction.x = -imageInfo.vertLine.direction.y;
	imageInfo.vertLine.direction.y = imageInfo.horizon.direction.x;
	imageInfo.vertLine.normalizeDirection();

	if(cameraMatrix.frameNumber != prevCameraMatrix.frameNumber) {

		Vector2<int> centerHorizon((int)imageInfo.horizon.base.x,(int)imageInfo.horizon.base.y);
		Vector2<double> angles;	// to center-horizon
		Vector2<int> prevCenterHorizon;

		Geometry::calculateAnglesForPoint(centerHorizon,cameraMatrix,image.cameraInfo,angles);
		Geometry::calculatePointByAngles(angles,prevCameraMatrix,image.cameraInfo,prevCenterHorizon);

    const RobotDimensions& r = getRobotConfiguration().getRobotDimensions();
    double timeDiff = (cameraMatrix.frameNumber - prevCameraMatrix.frameNumber) * r.motionCycleTime; // in seconds
    double imageDiff = timeDiff * r.imagesPerSecond; // how many images are between the old and the current one (should be 1)?
		int diff = (int)(1000/(imageDiff*0.9));// interpolate, 10% less
    
		prevImageOffset = (prevCenterHorizon - centerHorizon) * diff; 

		//prevImageBaseOffset = centerHorizon;
		//correctForHeadMovement(prevImageBaseOffset);

		prevImageBaseOffset = - prevImageOffset 
			* (cameraResolutionHeight_ERS7 - centerHorizon.y) / (cameraResolutionHeight_ERS7*1000);
	} else {
		prevImageOffset = Vector2<int>(0,0);
	}
}

void ScanningGrid::prepareHorizontalScanline() {
	// horizon correction
	Vector2<double> correction = (imageInfo.vertLine.direction * HORIZON_CORRECTION);

	// correct manual horizon shift in vertical direction
	imageInfo.horizon.base += correction;

	// correct horizon transpose by camera movement
	imageInfo.horizon.base += Vector2<double>(prevImageBaseOffset.x,prevImageBaseOffset.y);

	imageInfo.horizonInImage = Geometry::getIntersectionPointsOfLineAndRectangle(
	Vector2<int>(0,0), imageInfo.maxImageCoordinates, imageInfo.horizon, 
					imageInfo.horizonStart, imageInfo.horizonEnd);

	Vector2<int> pos = imageInfo.horizonStart;

	BresenhamLineScan scan(imageInfo.horizonStart,imageInfo.horizonEnd);
	scan.init();

	horizontalScanPoints = 0;
	int pixels=0;



	while((pixels < scan.numberOfPixels) && (horizontalScanPoints < DIAGONAL)) {
		scan.getNext(pos);
		pixels++;
		horizontalScanPoints = (int)(pixels/scanningDistanceX);
		horizontal[horizontalScanPoints] = pos;
	}


	Vector2<double> angles;

	Geometry::calculateAnglesForPoint(imageInfo.horizonStart,cameraMatrix,prevCameraMatrix,image.cameraInfo,angles);
	radHorizonLeft = angles.x;

	Geometry::calculateAnglesForPoint(imageInfo.horizonEnd,cameraMatrix,prevCameraMatrix,image.cameraInfo,angles);
	radHorizonRight = angles.x;

	radPerHorizontalScanPoint = (radHorizonRight-radHorizonLeft)/horizontalScanPoints;

}

void ScanningGrid::prepareVerticalScanline() {
	Vector2<int> src(0,0);
	Vector2<int> dest(
					(int)(imageInfo.vertLine.direction.x*DIAGONAL),
					(int)(imageInfo.vertLine.direction.y*DIAGONAL));

	// correct vertical line for head movement
	correctForHeadMovement(src);
	correctForHeadMovement(dest);

	BresenhamLineScan scan(src,dest);

	scan.init();

	verticalScanPoints = 0;
	Vector2<int> pos;
	int pixels=0;

	vertical[0][0] = 
		vertical[1][0] = 
		verticalDetailled[0][0] = 
		verticalDetailled[1][0] = Vector2<int>();
	verticalIndex[0]=0;
	while(
		   (pixels < scan.numberOfPixels) 
		&& (verticalScanPoints < DIAGONAL-2) 
		&& (pixels<DIAGONAL-1) 
		&& (verticalScanPoints<maxVerticalScanPoints)) 
	{
		scan.getNext(pos);
		pixels++;
		verticalScanPoints = (int)(pixels/scanningDistanceY)+1;
		vertical[0][verticalScanPoints] = -pos;
		vertical[1][verticalScanPoints] = pos;
		verticalDetailled[0][pixels] = -pos;
		verticalDetailled[1][pixels] = pos;
		verticalIndex[verticalScanPoints]=pixels;
	}
	
}

void ScanningGrid::correctForHeadMovement(Vector2<int> &point) {
	point += prevImageOffset * (cameraResolutionHeight_ERS7 - point.y) / (cameraResolutionHeight_ERS7*1000);
}




/*
*
*/
