/**
* @file ScanningGrid.h
* 
* Definition of class ScanningGrid
*
* @author Jurgen Sturm
*/


#ifndef __ScanningGrid_h_
#define __ScanningGrid_h_

#include "Tools/Math/Geometry.h"
#include "Representations/Perception/CameraInfo.h"
#include "Representations/Perception/CameraMatrix.h"
#include "Representations/Perception/Image.h"
#include "Modules/ImageProcessor/ImageProcessorTools/ColorCorrector.h"
#include "Modules/ImageProcessor/GT2004ImageProcessor/GT2004ImageProcessorTools.h"
#include "Modules/ImageProcessor/ImageProcessor.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Tools/Debugging/DebugImages.h"
#include "Representations/Perception/PanoramaParameters.h"


#define SCAN_IMAGE(horizon,radialDist,colCorr,scanPixel,scanPixelParam) \
	SCAN_IMAGE_DETAIL(horizon,radialDist,colCorr,scanPixel,scanPixelParam,false,detailScanDoNothing,NULL);

#define SCAN_IMAGE_DETAIL(horizon,radialDist,colCorr,scanPixel,scanPixelParam,detailEnabled,detailPixel,detailParam) \
{ \
	prepare();\
	int is,ix,iy,id,x,y,bx,by;\
	Vector3<unsigned char> color;\
	double radScanline = radHorizonLeft;\
	int vertSides = (horizon) + 1,horPoints,vertPoints;\
	horPoints = horizontalScanPoints;\
	for(ix=0;ix<horPoints ;ix++)\
	{\
		bx = horizontal[ix].x;\
		by = horizontal[ix].y;\
		\
		for(is=0;is<vertSides;is++) {\
		\
			vertPoints = verticalScanPoints;\
  \
			for(iy=0;iy<vertPoints;iy++) {\
				x = bx + vertical[is][iy].x;\
				y = by + vertical[is][iy].y;\
				if(\
					x>=0 && \
					y>=0 && \
					x < cameraResolutionWidth_ERS7 && \
					y < cameraResolutionHeight_ERS7\
				) {\
  \
					if(radialDist)\
						Geometry::radialDistortionCorrectionFast(x,y,x,y);\
  \
					color.x = image.image[y][0][x];\
					color.y = image.image[y][1][x];\
					color.z = image.image[y][2][x];\
  \
					if(colCorr) \
						ColorCorrector::correct(x,y,color.x,color.y,color.z);\
  \
					if(scanPixel(is,ix,iy,x,y,color,radScanline,scanPixelParam)) {\
						if(detailEnabled) {\
							if(iy!=0) {	\
								int id2=verticalIndex[iy];\
								for(id=verticalIndex[iy-1];id<=id2;id++) {\
									x = bx + verticalDetailled[is][id].x;\
									y = by + verticalDetailled[is][id].y;\
	\
									if(radialDist)\
										Geometry::radialDistortionCorrectionFast(x,y,x,y);\
	\
									color.x = image.image[y][0][x];\
									color.y = image.image[y][1][x];\
									color.z = image.image[y][2][x];\
	\
									if(colCorr) \
										ColorCorrector::correct(x,y,color.x,color.y,color.z);\
	\
									if(detailPixel(is,ix,iy,id,x,y,color,radScanline,detailParam)) break;\
								}\
							}\
						}\
					}\
				}\
				else \
					break;\
			}\
		}\
		radScanline += radPerHorizontalScanPoint;\
	}\
}

class ScanningGrid
{
public:
	const Image &image;
	const CameraMatrix &cameraMatrix;
	const CameraMatrix &prevCameraMatrix;
	ImageInfo imageInfo;
	Vector2<int> prevImageOffset;
	Vector2<int> prevImageBaseOffset;
	
	double scanningDistanceX;
	double scanningDistanceY;

	Vector2<int> horizontal[DIAGONAL];
	Vector2<int> vertical[2][DIAGONAL];	// above or below horizon
	Vector2<int> verticalDetailled[2][DIAGONAL];	// above or below horizon
	int verticalIndex[DIAGONAL];
	int horizontalScanPoints,verticalScanPoints;
	int maxVerticalScanPoints;
	double radHorizonLeft, radHorizonRight, radPerHorizontalScanPoint;	// angles to left and right end of horizon

  ColorCorrector colorCorrector; /**< The color correction tool. */

	ScanningGrid(const Image &image,const CameraMatrix &cameraMatrix,const CameraMatrix &prevCameraMatrix);


	// precompute horizon and vertical scan lines
	void prepare();
	void prepareImageInfo();
	void prepareHorizontalScanline();
	void prepareVerticalScanline();

	void correctForHeadMovement(Vector2<int> &point);
};


inline bool detailScanDoNothing(		
		int &is,int &ix,int &iy,int &id,
		int &x,int &y,
		Vector3<unsigned char> &color,double &radScanline, void *something) { return(true); }

#endif// __ScanningGrid_h_

/*
*
*
*/
