/**
* @file ColorTableAuto.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/** 
* @file ColorTableAuto.h
* Declaration of class ColorTableAuto.
*
* @author <A href=mailto:juengel@informatik.hu-berlin.de>Matthias Jngel</A>
*/

#ifndef _ColorTableAuto_h_
#define _ColorTableAuto_h_

#include "Representations/Perception/Image.h"
#include "Tools/ColorClasses.h"

/**
* @class ColorTableAuto
*
* @author <A href=mailto:juengel@informatik.hu-berlin.de>Matthias Jngel</A>
*/
class ColorTableAuto : public ColorTable
{
public:
  /** Constructor */
  ColorTableAuto();

  /** Destructor */
  ~ColorTableAuto();

  /** 
  * Calculates the color classes of a point in the yuv color space.
  * @param y the y value of the pixel
  * @param u the u value of the pixel
  * @param v the v value of the pixel
  */
  virtual colorClass getColorClass (
    const unsigned char y, 
    const unsigned char u, 
    const unsigned char v
    ) const;

  /** 
  * Calculates the color classes of a point in the yuv color space.
  * @param y the y value of the pixel
  * @param u the u value of the pixel
  * @param v the v value of the pixel
  */
  virtual colorClass getColorClassAuto (
    const unsigned char y, 
    const unsigned char u, 
    const unsigned char v
    ) const;


  /**
  * Segments an image to an color class image.
  * 
  * This doesn't need to used in the image processor, but is needed for visualisation
  * of color tables.
  * @param image A reference to the image to be segmented
  * @param colorClassImage A reference to the color class image to be created
  * @return A reference to the created color class image.
  */
  virtual void generateColorClassImage(
    const Image& image, 
    ColorClassImage& colorClassImage) const;

  virtual void generateColorClassImageAuto(
    const Image& image, 
    ColorClassImage& colorClassImage) const;


  int getAverageGreen(int channel) const;
  int getGreenMin(int channel) const;
  int getGreenMax(int channel) const;

  void setThresholds(
    colorClass color,
    int minY, int minU, int minV, int maxY, int maxU, int maxV
    );

  int getThresholdMin(colorClass color, int channel) const;
  int getThresholdMax(colorClass color, int channel) const;


private:
  void setGreen();

  int thresholdMin[numOfColors][3];
  int thresholdMax[numOfColors][3];
  unsigned long int lastTimeWhenThresholdWasSet[numOfColors];

  int averageGreen[3];

  int greenFieldAutoMin[3];
  int greenFieldAutoMax[3];

  int skyblueAutoMin[3];
  int skyblueAutoMax[3];

  int yellowAutoMin[3];
  int yellowAutoMax[3];

  int pinkAutoMin[3];
  int pinkAutoMax[3];

  int whiteAutoMin[3];
  int whiteAutoMax[3];
  
  int whiteFixedMin[3];
  int whiteFixedMax[3];
};

/**
 * Streaming operator that reads ColorTableAuto from a stream.
 * @param stream The stream from which is read.
 * @param colorTableAuto The ColorTableAuto object.
 * @return The stream.
 */ 
In& operator>>(In& stream, ColorTableAuto& colorTableAuto);
 

Out& operator<<(Out& stream, const ColorTableAuto& colorTableAuto);



#endif   //  _ColorTableAuto_h_

/*
 * Change log :
 * 
 * $Log: ColorTableAuto.h,v $
 * Revision 1.3  2004/07/08 19:09:15  floris
 * started cleaning code, removed Gridimage
 *
 * Revision 1.2  2004/02/27 11:31:13  wcaarls
 * Import of Utrecht changes as of 26-02-2004.
 *
 * Revision 1.1.1.1  2004/01/15 12:30:27  bottens
 *
 *
 * Revision 1.5  2003/09/26 15:27:49  juengel
 * Renamed DataTypes to representations.
 *
 * Revision 1.4  2003/08/29 11:59:38  roefer
 * Comment corrected
 *
 * Revision 1.3  2003/08/18 12:06:01  juengel
 * Added streaming operators.
 *
 * Revision 1.2  2003/07/29 12:44:26  juengel
 * Optimization
 *
 * Revision 1.1  2003/07/21 13:40:24  juengel
 * Moved ColorTableAuto and GoalRecognizer to ImageProcessorTools.
 *
 * Revision 1.2  2003/07/19 19:31:04  juengel
 * Improved AutoCalibration
 *
 * Revision 1.1.1.1  2003/07/02 09:40:23  cvsadm
 * created new repository for the competitions in Padova from the 
 * tamara CVS (Tuesday 2:00 pm)
 *
 * removed unused solutions
 *
 * Revision 1.4  2003/05/05 12:27:27  juengel
 * Added method setGreen.
 *
 * Revision 1.3  2003/04/23 16:17:55  juengel
 * ColorTableAuto::getColorClassForFlagsAndGoals returns a color class now.
 *
 * Revision 1.2  2003/04/02 13:21:11  juengel
 * Improved flag colors.
 *
 * Revision 1.1  2003/04/01 13:59:34  juengel
 * Added ColorTableAuto
 *
 *
 */
