/* 
 * wrapper class for the anigauss-library of Jan-Mark Geusebroek
 */
#include "AnisotropicFilterLibrary.h"
#include "anigauss.h"

// create an empty anisotropic image wrap
AnisotropicImage::AnisotropicImage() {
	width = 0;
	height = 0;
	for(int c=0;c<3;c++)
		for(int y=0;y<cameraResolutionHeight_ERS7;y++)
			for(int x=0;x<cameraResolutionWidth_ERS7;x++)
				image[c][y][x] = 0;
}

// create an image wrap from existing aibo image
AnisotropicImage::AnisotropicImage(const Image &source) {
	copyFromImage(source);
}

// create an image from an existing wrapper by applying a filter
AnisotropicImage::AnisotropicImage(
		AnisotropicImage &source, 
		double sigmav, double sigmau, 
		double phi, int orderv, int orderu) 
{
	width = source.width;
	height = source.height;
	anigauss((double*)&source.image[0], (double*)&image[0], width, height, sigmav, sigmau, phi, orderv, orderu);
	anigauss((double*)&source.image[1], (double*)&image[1], width, height, sigmav, sigmau, phi, orderv, orderu);
	anigauss((double*)&source.image[2], (double*)&image[2], width, height, sigmav, sigmau, phi, orderv, orderu);
}

void AnisotropicImage::copyFromImage(const Image &source) {
	width = source.cameraInfo.resolutionWidth;
	height = source.cameraInfo.resolutionHeight;

	for(int c=0;c<3;c++)
		for(int y=0;y<cameraResolutionHeight_ERS7;y++)
			for(int x=0;x<cameraResolutionWidth_ERS7;x++)
				image[c][y][x] = source.image[y][c][x];
}

// basic filtering
void AnisotropicImage::AnisotropicGauss(
		double sigmav, double sigmau, 
		double phi, int orderv, int orderu) 
{
	anigauss((double*)&image[0], (double*)&image[0], width, height, sigmav, sigmau, phi, orderv, orderu);
	anigauss((double*)&image[1], (double*)&image[1], width, height, sigmav, sigmau, phi, orderv, orderu);
	anigauss((double*)&image[2], (double*)&image[2], width, height, sigmav, sigmau, phi, orderv, orderu);
}

// write image back to an aibo image
void AnisotropicImage::copyToImage(Image &destination) {
	copyToImage(destination,direct);
}

char minmax256(int a) {
	if(a<=0) return((char)0);
	if(a>=255) return((char)255);
	return((char)a);
}

// write image back to an aibo image
void AnisotropicImage::copyToImage(Image &destination,CopyMode copyMode) {
	int c,x,y;
	destination.cameraInfo.resolutionWidth = width;
	destination.cameraInfo.resolutionHeight = height;

	switch(copyMode) {
		case direct:
			for(c=0;c<3;c++)
				for(y=0;y<height;y++)
					for(x=0;x<width;x++)
						destination.image[y][c][x] = (int)image[c][y][x];
			break;
		case add128:
			for(c=0;c<3;c++)
				for(y=0;y<height;y++)
					for(x=0;x<width;x++)
						destination.image[y][c][x] = minmax256((int)image[c][y][x]+128);
			break;
		case add128div2:
			for(c=0;c<3;c++)
				for(y=0;y<height;y++)
					for(x=0;x<width;x++)
						destination.image[y][c][x] = minmax256((int)image[c][y][x]/2+128);
			break;
		case blackwhite:
			int b;
			for(y=0;y<height;y++)
				for(x=0;x<width;x++) {
					b = minmax256(
						(int)sqrt(
							sqr(image[0][y][x])+
						    sqr(image[1][y][x])+
							sqr(image[0][y][x])
						));

					destination.image[y][0][x] = b;
					destination.image[y][1][x] = b;
					destination.image[y][2][x] = b;

				}
			break;
		case autoscale:
			int b2;
			double brightness,bottom=255,top=0;
			for(y=0;y<height;y++)
				for(x=0;x<width;x++) {
					brightness = 
						sqrt(
							sqr(image[0][y][x])+
						    sqr(image[1][y][x])+
							sqr(image[2][y][x])
						);
					if(brightness<bottom) bottom=brightness;
					if(brightness>top) top=brightness;
				}
			if(bottom>=top) {
				bottom=0;
				top=255;
			}
			for(y=0;y<height;y++)
				for(x=0;x<width;x++) {
					b2 = 
						(int)((sqrt(
							sqr(image[0][y][x])+
						    sqr(image[1][y][x])+
							sqr(image[2][y][x])
						)-bottom)/(top-bottom)*255);
					//b2 = (int)((image[0][y][x]-bottom)/(top-bottom)*255);
					destination.image[y][2][x] = b2;
					destination.image[y][0][x] = b2;
					destination.image[y][1][x] = b2;

				}
			break;
	}
}

void AnisotropicImage::copyToMagnitudeMap(ChannelImage &destination) {
	destination.width = width;
	destination.height = height;
	int b2,x,y;
	double brightness,bottom=255,top=0;
	for(y=0;y<height;y++)
		for(x=0;x<width;x++) {
			brightness = 
				sqrt(
					sqr(image[0][y][x])+
					sqr(image[1][y][x])+
					sqr(image[1][y][x]-image[2][y][x])+		// misschien +
					sqr(image[2][y][x])
				);
			if(brightness<bottom) bottom=brightness;
			if(brightness>top) top=brightness;
		}
	if(bottom>=top) {
		bottom=0;
		top=255;
	}
	for(y=0;y<height;y++)
		for(x=0;x<width;x++) {
			b2 = 
				(unsigned char)((sqrt(
					sqr(image[0][y][x])+
					sqr(image[1][y][x])+
					sqr(image[1][y][x]-image[2][y][x])+
					sqr(image[2][y][x])
				)-bottom)/(top-bottom)*255);
			//b2 = (int)((image[0][y][x]-bottom)/(top-bottom)*255);
			destination.image[y][x] = b2;
		}
}
