/* 
 * Implementation of EM algorithm for gaussian mixture models
 * by jurgen sturm
 * following the explanation in the phd thesis of Jakob Jozef Verbeek
 */

#include "EMGaussianMixtureLibrary.h"
#include <stdlib.h>
#include <string.h>
#include "Tools/Math/Common.h"
#include "Representations/Perception/PanoramaParameters.h"

//#include "Platform/Win32/GTAssert.h"
// reduce found clusters until CLUSTERS remain

void saturate(unsigned char y,unsigned char u,unsigned char v,unsigned char &Y,unsigned char &U,unsigned char &V) {
	Y=y;
	U=u;
	V=v;
//	Y=255;
	if(u==128) {
		U = 128;
		V = v<128?0:255;
	} else 
	if(v==128) {
		U = u<128?0:255;
		V = 128;
	} else
	if(abs(u-128)<abs(v-128)) {
		U = (unsigned char) (((int)u-128)*abs(128/((int)v-128))+128);
		V = v<128?0:255;
	} else {
		V = (unsigned char) (((int)v-128)*abs(128/((int)u-128))+128);
		U = u<128?0:255;
	}
}


EMGaussianMixture::EMGaussianMixture(bool useFixedWeight,bool useFixedSigma) {
	fixedWeight = useFixedWeight;
	fixedSigma = useFixedSigma;

	trainingDataLength = 0;
}

EMGaussianMixture::~EMGaussianMixture() { //destroy q_ns buffer
	setTrainingSet(NULL,0);
}

void EMGaussianMixture::initRandom() {
	int cluster;

	for(cluster=0;cluster<CLUSTERS;cluster++) {

		// init weights; normalize later
		if(fixedWeight) 
			weight[cluster] = 1;
		else
			weight[cluster] = random();

		// init means, somewhere in the color cube
		mean[cluster].x = random()*255;
		mean[cluster].y = random()*255;
		mean[cluster].z = random()*255;

		// init sigmas, only diagonal
		sigma[cluster] *= 30;	

		sigmaInvValid[cluster] = false;
	}


	// normalize weights; sum=1
	double sumWeight = 0;
	for(cluster=0;cluster<CLUSTERS;cluster++) sumWeight += weight[cluster];
	for(cluster=0;cluster<CLUSTERS;cluster++) weight[cluster] /= sumWeight;
}

void EMGaussianMixture::initMeansOnTrainingSet() {
	for(int cluster=0;cluster<CLUSTERS;cluster++) 
		mean[cluster] = trainingData[rand() % trainingDataLength];
}


void EMGaussianMixture::setTrainingSet(Vector3<double> * newTrainingData, int newTrainingDataLength) {
	int cluster;

	if(trainingDataLength != 0) 
		for(cluster=0;cluster<CLUSTERS;cluster++) 
			free(q[cluster]);

	trainingData = newTrainingData;
	trainingDataLength = newTrainingDataLength;

	if(trainingDataLength != 0) 
		for(cluster=0;cluster<CLUSTERS;cluster++) 
			q[cluster] = (double *) calloc(trainingDataLength,sizeof(double));
}

/* first step: assign to clusters: Expectation */
void EMGaussianMixture::stepExpectation() {
	int cluster;
	// sjaak, inline formula 3.2, way faster than posteriorProbability()

	for(int i=0;i<trainingDataLength;i++) {
		// get the sample
		Vector3<double> &sample=trainingData[i];

		// compute probability
		double sum=0;
		for(cluster=0;cluster<CLUSTERS;cluster++)
			sum += (q[cluster][i] = gaussianProbability(sample,cluster));

		if(sum<1e-250) {
			for(cluster=0;cluster<CLUSTERS;cluster++)
				q[cluster][i] = 1.0/CLUSTERS;
		} else {
			// normalize
			for(cluster=0;cluster<CLUSTERS;cluster++)
				q[cluster][i] /= sum;
			}
		}
}

int EMGaussianMixture::assignToCluster(Vector3<double> &sample) {
	int bestCluster=-1;
	double bestProb = -1;

	for(int cluster=0;cluster<CLUSTERS;cluster++) {
		if(weight[cluster]>0) {
			double p = gaussianProbability(sample,cluster)/(weight[cluster]);// why not, make smaller clusters bigger
			if(p>=bestProb) {
				bestProb = p;
				bestCluster = cluster;
			}
		}
	}

	return(bestCluster);
}

double EMGaussianMixture::gaussianProbability(Vector3<double> &sample,int cluster) {
	if(!sigmaInvValid[cluster]) {
		sigmaInv[cluster] = sigma[cluster].invert();
		sigmaInvValid[cluster] = true;
	}

	Vector3<double> diff = (sample - mean[cluster]);
	Vector3<double> diff2 = (sigmaInv[cluster] 
									* (sample - mean[cluster]));
//	double res = (sample - mean[cluster]) 
//									* (sigmaInv[cluster] 
//									* (sample - mean[cluster]));

	// sjaak, formula 3.8
	return 
		(			pow(2*pi,-1.5) 
				* exp(
						-0.5
							*(
										(sample - mean[cluster]) 
									* (sigmaInv[cluster] 
									* (sample - mean[cluster])))));

}

double EMGaussianMixture::posteriorProbability(Vector3<double> &sample,int cluster) {

	// sjaak, formula 3.2
	int c;
	double sum=0;
	for(c=0;c<CLUSTERS;c++)
		sum += weight[c] * gaussianProbability(sample,c);

	return((weight[cluster] * gaussianProbability(sample,cluster))/sum);
}

/* second step: optimize parameters: Maximization */
void EMGaussianMixture::stepMaximization() {
	for(int cluster = 0; cluster<CLUSTERS; cluster++) {
		reestimateWeight(cluster);
		reestimateMean(cluster);
		if(!fixedSigma) reestimateSigma(cluster);
		if(fixedWeight) weight[cluster] = 1.0/CLUSTERS;
	}
}

void EMGaussianMixture::reestimateWeight(int cluster) {

	// sjaak, formula 3.9
	double sum=0;
	for(int i=0;i<trainingDataLength;i++) {
		sum += q[cluster][i];
	}

	weight[cluster] = sum/trainingDataLength;
}

void EMGaussianMixture::reestimateMean(int cluster) {

	// sjaak, formula 3.10

	Vector3<double> sum;
	for(int i=0;i<trainingDataLength;i++) {
		// get the sample
		Vector3<double> &sample=trainingData[i];

		sum += (sample * q[cluster][i]);
	}

	mean[cluster] = sum/(trainingDataLength * weight[cluster]);

}

void EMGaussianMixture::reestimateSigma(int cluster) {
	// sjaak, formula 3.11

	Matrix3x3<double> sum;
	for(int i=0;i<trainingDataLength;i++) {
		// get the sample
		Vector3<double> &sample=trainingData[i];

		sum = sum + Matrix3x3<double>(sample - mean[cluster],sample - mean[cluster]);// cross-product
	}

	sigma[cluster] = sum/(trainingDataLength * weight[cluster]);
}

void EMGaussianMixture::clusterImage(Image &clustered) {
	Vector3<double> pixel;
	int cluster;

	int x,y;
  for(y=0;y<clustered.cameraInfo.resolutionHeight;y++) {
		for(x=0;x<clustered.cameraInfo.resolutionWidth;x++) {
			/* hsv experiment
			unsigned char c1,c2,c3;
			Image::convertFromYCbCrToHSI(
				clustered.image[y][0][x],
				clustered.image[y][1][x],
				clustered.image[y][2][x],
				c1,c2,c3);
			pixel = Vector3<double>(
											c1,c2,c3
										);
			*/
			/*
			pixel = Vector3<double>(
											clustered.image[y][0][x],
											clustered.image[y][1][x],
											clustered.image[y][2][x]
										);
			cluster = assignToCluster(pixel);
			*/

			cluster = lookupTable
										[clustered.image[y][0][x] >> 2]
										[clustered.image[y][1][x] >> 2]
										[clustered.image[y][2][x] >> 2];

			clustered.image[y][0][x] = clusterColor[cluster].x;
			clustered.image[y][1][x] = clusterColor[cluster].y;
			clustered.image[y][2][x] = clusterColor[cluster].z;
		}
	}
}

void EMGaussianMixture::visualizeClusters(Image &overview) {
	int SCALE = 1;
	Vector3<double> pixel;
	int cluster,x,y,sx,sy,sy2;

	/*
	for(cluster = 0; cluster<CLUSTERS; cluster++) {
		for(y=(sy=overview.cameraInfo.resolutionHeight*cluster/CLUSTERS);
			  y<(sy2=overview.cameraInfo.resolutionHeight*(cluster+1)/CLUSTERS);y++) {
			sx=(int)(weight[cluster]*overview.cameraInfo.resolutionWidth*SCALE)+5;
			for(x=0;x<sx;x++) {
				if((x==sx-1)||(x==0)||(y==sy)||(y==sy2-1)) {
					if(equals[cluster]==cluster) {
						overview.image[y][0][x] = 0;
						overview.image[y][1][x] = 128;
						overview.image[y][2][x] = 128;
					} else {
						// alias cluster
						overview.image[y][0][x] = clusterColor[equals[cluster]].x;
						overview.image[y][1][x] = clusterColor[equals[cluster]].y;
						overview.image[y][2][x] = clusterColor[equals[cluster]].z;
					}
				} else {
						overview.image[y][0][x] = clusterColor[cluster].x;
						overview.image[y][1][x] = clusterColor[cluster].y;
						overview.image[y][2][x] = clusterColor[cluster].z;
				}
			}
		}
	}
	*/


	int usedClusters = 0;
	for(cluster = 0; cluster<CLUSTERS; cluster++) 
		if(equals[cluster]==cluster) usedClusters++;
	int index =0;
	for(cluster = 0; cluster<usedClusters; cluster++) {
		for(y=(sy=overview.cameraInfo.resolutionHeight*cluster/usedClusters);
			  y<(sy2=overview.cameraInfo.resolutionHeight*(cluster+1)/usedClusters);y++) {
			sx=(int)(weight[index]*overview.cameraInfo.resolutionWidth*0.5)+10;
			for(x=1;x<sx;x++) {
				if((x==sx-1)||(x==1)||(y==sy)||(y==sy2-1)) {
					overview.image[y][0][overview.cameraInfo.resolutionWidth-x] = 0;
					overview.image[y][1][overview.cameraInfo.resolutionWidth-x] = 128;
					overview.image[y][2][overview.cameraInfo.resolutionWidth-x] = 128;
				} else {
					overview.image[y][0][overview.cameraInfo.resolutionWidth-x] = clusterColor[index].x;
					overview.image[y][1][overview.cameraInfo.resolutionWidth-x] = clusterColor[index].y;
					overview.image[y][2][overview.cameraInfo.resolutionWidth-x] = clusterColor[index].z;
				}
			}
		}

		do index++; while(index<CLUSTERS && equals[index]!=index) ;
	}


}

void EMGaussianMixture::startEM(int maxSteps) {
	for(int i=0;i<maxSteps;i++) {
		stepExpectation();
		stepMaximization();
	}
}

void EMGaussianMixture::sortClusters() {
	// using bubble sort; not effective but simple.
	int i,j,k,c1,c2;
	for(i=0;i<CLUSTERS;i++) 
		for(j=0;j<CLUSTERS-1;j++) {
			c1 = j;
			c2 = j+1;
			if(weight[c1]<weight[c2]) {
				// swap c1 and c2
				double tempWeight = weight[c1];
				Vector3<double> tempMean = mean[c1];
				Matrix3x3<double> tempSigma = sigma[c1];
				Matrix3x3<double> tempSigmaInv = sigmaInv[c1];
				bool tempSigmaInvValid = sigmaInvValid[c1];
				int tempEquals = equals[c1];

				weight[c1] = weight[c2];
				mean[c1] = mean[c2];
				sigma[c1] = sigma[c2];
				sigmaInv[c1] = sigmaInv[c2];
				sigmaInvValid[c1] = sigmaInvValid[c2];
				equals[c1] = equals[c2];

				weight[c2] = tempWeight;
				mean[c2] = tempMean;
				sigma[c2] = tempSigma;
				sigmaInv[c2] = tempSigmaInv;
				sigmaInvValid[c2] = tempSigmaInvValid;
				equals[c2] = tempEquals;

				// now exchange everywhere in equals[] c1 with c2
				for(k=0;k<CLUSTERS;k++) {
					if(equals[k]==c1) 
						equals[k]=c2;
					else
						if(equals[k]==c2) equals[k]=c1;
				}
			}
		}
}

void EMGaussianMixture::updateLookupTable() {
	sortClusters();

	for(int c=0;c<CLUSTERS;c++)
		equals[c]=c;

	double dist, minDist;
	int minCluster,minClusterReplacement;

	for(int removeCluster=CLUSTERS;removeCluster<CLUSTERS;removeCluster++) {
		minCluster = -1;
		minClusterReplacement = -1;
		minDist = 10000000;
		for(int c1=0;c1<CLUSTERS;c1++) {
			if(equals[c1]==c1) {
				for(int c2=c1+1;c2<CLUSTERS-1;c2++) {
					if(equals[c2]==c2) {
						//double dist = (mean[c1] - mean[c2])*(mean[c1] - mean[c2]);
						Vector3<double> delta = mean[c1] - mean[c2];
						dist = min(//extrem dark and extrem bright colors are equal too.. (kind of dimension reduction)
							sqr(delta.y/mean[c1].x) + sqr(delta.z/mean[c1].x),
							//sqr(delta.y/(255-mean[c1].x)) + sqr(delta.z/(255-mean[c1].x)),//orange==pink, we don't want that
							255
							);
						if(dist<minDist) {
							minDist = dist;
							minCluster = c2;
							minClusterReplacement = c1;
						}
					}
				}
			}
		}

		equals[minCluster]=minClusterReplacement;
		weight[minClusterReplacement] += weight[minCluster];		// parent cluster now heavier!
	}
	for(int c1=0;c1<CLUSTERS;c1++) {
		while(equals[equals[c1]] != equals[c1]) equals[c1] = equals[equals[c1]];// find the big boss
	}

	sortClusters();

	int i,j,k;

	Vector3<double> col;
	for(i=0;i<64;i++)
		for(j=0;j<64;j++)
			for(k=0;k<64;k++) 
				lookupTable[i][j][k] = 255;
				

	computeRecursively(6,0,0,0);

	/*
	for(i=0;i<64;i++)
		for(j=0;j<64;j++)
			for(k=0;k<64;k++) {
				if(lookupTable[i][j][k] ==-1)
					OUTPUT(idText,text," -1 at ax=" <<i<<" ay=" <<j<<" az=" <<k);
				}
*/
	/*
	for(i=0;i<64;i++)
		for(j=0;j<64;j++)
			for(k=0;k<64;k++) {
				col.x=i<<2;
				col.y=j<<2;
				col.z=k<<2;
				if(lookupTable[i][j][k] != equals[assignToCluster(col)]) {
					OUTPUT(idText,text," difference at ax=" <<i<<" ay=" <<j<<" az=" <<k<<" rec="<<lookupTable[i][j][k] <<" normal="<<equals[assignToCluster(col)]);
				}
			}
*/
	for(int cluster=0;cluster<CLUSTERS;cluster++) {
		clusterColor[cluster].x = (int)mean[cluster].x;
		clusterColor[cluster].y = (int)mean[cluster].y;
		clusterColor[cluster].z = (int)mean[cluster].z;
	}
}

void EMGaussianMixture::updateLookupTable(int phase) {
	if(phase==0) {
		sortClusters();

		for(int c=0;c<CLUSTERS;c++)
			equals[c]=c;

		double dist, minDist;
		int minCluster,minClusterReplacement;

		for(int removeCluster=CLUSTERS;removeCluster<CLUSTERS;removeCluster++) {
			minCluster = -1;
			minClusterReplacement = -1;
			minDist = 10000000;
			for(int c1=0;c1<CLUSTERS;c1++) {
				if(equals[c1]==c1) {
					for(int c2=c1+1;c2<CLUSTERS-1;c2++) {
						if(equals[c2]==c2) {
							//double dist = (mean[c1] - mean[c2])*(mean[c1] - mean[c2]);
							Vector3<double> delta = mean[c1] - mean[c2];
							dist = min(//extrem dark and extrem bright colors are equal too.. (kind of dimension reduction)
								sqr(delta.y/mean[c1].x) + sqr(delta.z/mean[c1].x),
								//sqr(delta.y/(255-mean[c1].x)) + sqr(delta.z/(255-mean[c1].x)),//orange==pink, we don't want that
								255
								);
							if(dist<minDist) {
								minDist = dist;
								minCluster = c2;
								minClusterReplacement = c1;
							}
						}
					}
				}
			}

			equals[minCluster]=minClusterReplacement;
			weight[minClusterReplacement] += weight[minCluster];		// parent cluster now heavier!
		}
		for(int c1=0;c1<CLUSTERS;c1++) {
			while(equals[equals[c1]] != equals[c1]) equals[c1] = equals[equals[c1]];// find the big boss
		}

		sortClusters();

		int i,j,k;

		Vector3<double> col;
		for(i=0;i<64;i++)
			for(j=0;j<64;j++)
				for(k=0;k<64;k++) 
					lookupTable[i][j][k] = 255;
					

		//computeRecursively(6,0,0,0);
	}

	computeRecursively(5,(phase%2)*32,((phase/2)%2)*32,((phase/4)%2)*32);

	if(phase==7) {
		for(int cluster=0;cluster<CLUSTERS;cluster++) {
			clusterColor[cluster].x = (int)mean[cluster].x;
			clusterColor[cluster].y = (int)mean[cluster].y;
			clusterColor[cluster].z = (int)mean[cluster].z;
		}
	}
}

void EMGaussianMixture::computeRecursively(int level,int x,int y, int z) {
//	if(level>=5) OUTPUT(idText,text," rec: level "<<level<<" x=" <<x<<" y=" <<y<<" z=" <<z);

	Vector3<double> col;
	// compute 8 corners
	int ax,ay,az;
	int cluster,firstCluster=-1;
	bool descend = false;
	for(int ix=0;ix<2;ix++)
	for(int iy=0;iy<2;iy++)
	for(int iz=0;iz<2;iz++) {
		ax=(x + ix*((1<<level) - 1));
		ay=(y + iy*((1<<level) - 1));
		az=(z + iz*((1<<level) - 1));

		if((cluster=lookupTable[ax][ay][az])==255) {
			col.x = ax<<2;
			col.y = ay<<2;
			col.z = az<<2;

			cluster = equals[assignToCluster(col)];
	//		OUTPUT(idText,text," computing at level "<<level<<" ax=" <<ax<<" ay=" <<ay<<" az=" <<az<<" cluster="<<cluster);

			lookupTable[ax][ay][az] = cluster;
		}
		if(firstCluster==-1) 
			firstCluster = cluster;
		else if(firstCluster != cluster) 
			descend = true;

	}
	if(level==1) return;// done

	if(!descend && level<5) {//fill the rest, same cluster
//		OUTPUT(idText,text,"level "<<level<<" starting to fill ");
		for(int ix=x;ix<(x + ((1<<level) ));ix++)
		for(int iy=y;iy<(y + ((1<<level) ));iy++)
		for(int iz=z;iz<(z + ((1<<level) ));iz++) 
			lookupTable[ix][iy][iz] = cluster;
	} else {//divide and conquer
//		OUTPUT(idText,text,"level "<<level<<" divide and conquer ");
		for(int ix=0;ix<2;ix++)
		for(int iy=0;iy<2;iy++)
		for(int iz=0;iz<2;iz++) {
			ax=(x + ix*((1<<(level-1))));
			ay=(y + iy*((1<<(level-1))));
			az=(z + iz*((1<<(level-1))));
			computeRecursively(level-1,ax,ay,az);
		}

	}
}

void EMGaussianMixture::lookupColor(const Vector3<unsigned char> &fullColor, Vector3<unsigned char> &result) {
	int cluster = lookupTable[fullColor.x>>2][fullColor.y>>2][fullColor.z>>2];

	result = clusterColor[cluster];

	if(0){//colorfy
		result.x = (cluster*101 % 3)*85;
		result.y = (cluster*101/3 % 3)*85;
		result.z = (cluster*101/9 % 3)*85;
	}

}

int EMGaussianMixture::lookupColorNumber(const Vector3<unsigned char> &fullColor) {
	int cluster = lookupTable[fullColor.x>>2][fullColor.y>>2][fullColor.z>>2];
	return(cluster);
}

void EMGaussianMixture::saveLookupTable(const char* filename) {
	OutBinaryFile stream(filename);

	stream << (*this); // row row row the boat gently down the stream
}

bool EMGaussianMixture::loadLookupTable(const char* filename) {
	InBinaryFile stream(filename);

	if(!stream.exists()) return(false);
	
	EMGaussianMixture * stored = (EMGaussianMixture *) calloc(1,sizeof(EMGaussianMixture));
	stream >> (*stored);

	for(int c=0;c<CLUSTERS;c++) {
		clusterColor[c] = stored->clusterColor[c];
		equals[c] = stored->equals[c];
		weight[c] = stored->weight[c];
	}

	memcpy(lookupTable,stored->lookupTable,64*64*64);

	free(stored);
/*
	trainingData = NULL;
	trainingDataLength = 0;
	for(int i=0;i<CLUSTERS;i++)
		q[i] = NULL;

*/
	/*
	for(int cluster=0;cluster<CLUSTERS;cluster++) {
		clusterColor[cluster].x = (int)mean[equals[equals[cluster]]].x;
		clusterColor[cluster].y = (int)mean[equals[equals[cluster]]].y;
		clusterColor[cluster].z = (int)mean[equals[equals[cluster]]].z;
	}
	*/
	/*
	// print cluster means
	for(int c=0;c<CLUSTERS;c++) {
		if(equals[c]==c) {
			OUTPUT(idText,text,c<<" cluster mean= "<<mean[c].x<<" "<<mean[c].y<<" "<<mean[c].z)  ;
		}
	}
	*/

	return(true);
}


In& operator>>(In& stream,EMGaussianMixture& EMGaussianMixture)
{
  stream.read(&EMGaussianMixture,sizeof(EMGaussianMixture));
	EMGaussianMixture.trainingDataLength = 0;
	EMGaussianMixture.trainingData = NULL;
  return stream;
}

Out& operator<<(Out& stream, const EMGaussianMixture& EMGaussianMixture)
{
  stream.write(&EMGaussianMixture,sizeof(EMGaussianMixture));
  return stream;
}
  
