/* 
 * Implementation of EM algorithm for gaussian mixture models
 * by jurgen sturm
 * following the explanation in the phd thesis of Jakob Jozef Verbeek
 */

#ifndef __EMGaussianMixtureLibrary_h_
#define __EMGaussianMixtureLibrary_h_

#include "Tools/Math/Matrix.h"
#include "Tools/Math/Vector3.h"
#include "Representations/Perception/Image.h"
#include "Representations/Perception/PanoramaParameters.h"


void saturate(unsigned char y,unsigned char u,unsigned char v,unsigned char &Y,unsigned char &U,unsigned char &V);

class EMGaussianMixture {
public:
	double weight[MAX_CLUSTERS];
	Vector3<double> mean[MAX_CLUSTERS];
	Matrix3x3<double> sigma[MAX_CLUSTERS];

	Matrix3x3<double> sigmaInv[MAX_CLUSTERS];
	bool sigmaInvValid[MAX_CLUSTERS];

	bool fixedWeight;
	bool fixedSigma;

	Vector3<double> * trainingData;
	int trainingDataLength;
	double * q[MAX_CLUSTERS];

	unsigned char lookupTable[64][64][64];//260kb
	Vector3<unsigned char> clusterColor[MAX_CLUSTERS];
	int equals[MAX_CLUSTERS];

	EMGaussianMixture(bool useFixedWeight,bool useFixedSigma);
	~EMGaussianMixture();//destroy q_ns buffer

	void initRandom();
	void initMeansOnTrainingSet();

	void setTrainingSet(Vector3<double> * newTrainingData, int newTrainingDataLength);
	void startEM(int maxSteps);
	void sortClusters();
	//void unionCloseClusters(double distance);

	/* first step: assign to clusters: Expectation */
	void stepExpectation();
	int assignToCluster(Vector3<double> &sample);
	double gaussianProbability(Vector3<double> &sample,int cluster);
	double posteriorProbability(Vector3<double> &sample,int cluster);

	/* second step: optimize parameters: Maximization */
	void stepMaximization();
	void reestimateWeight(int cluster);
	void reestimateMean(int cluster);
	void reestimateSigma(int cluster);

	/* debug functions */
	void updateLookupTable();
	void updateLookupTable(int phase);
	void lookupColor(const Vector3<unsigned char> &fullColor, Vector3<unsigned char> &clusterColor);
	int lookupColorNumber(const Vector3<unsigned char> &fullColor);
	void saveLookupTable(const char* filename);
	bool loadLookupTable(const char* filename);

	void clusterImage(Image &clustered);
	void visualizeClusters(Image &overview);

	void computeRecursively(int level,int x,int y, int z);
};

In& operator>>(In& stream,EMGaussianMixture& EMGaussianMixture);
Out& operator<<(Out& stream, const EMGaussianMixture& EMGaussianMixture);

#endif
