/**
* @file PanoramaImageProcessor.cpp
* 
* 
* @author Jurgen Sturm
*/

#include "PanoramaImageProcessor.h"
#include "Platform/SystemCall.h"
#include "Tools/RobotConfiguration.h"
#include "AnisotropicFilterLibrary.h"
#include "EMGaussianMixtureLibrary.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Tools/Debugging/Debugging.h"
#include "Tools/Debugging/Stopwatch.h"

PanoramaImageProcessor::PanoramaImageProcessor(const ImageProcessorInterfaces& interfaces)
: ImageProcessor(interfaces), mixture(false,false)
{
	colorBufferVirtualCount = colorBufferCount = 0;


//	OUTPUT(idText,text,"sizeof(PanoramaImageProcessor)="<<sizeof(PanoramaImageProcessor))  ;
//	OUTPUT(idText,text,"sizeof(ColorCorrector)="<<sizeof(ColorCorrector))  ;
//	OUTPUT(idText,text,"sizeof(CameraMatrix)="<<sizeof(CameraMatrix))  ;
//	OUTPUT(idText,text,"sizeof(ImageInfo)="<<sizeof(ImageInfo))  ;
//	OUTPUT(idText,text,"sizeof(Image)="<<sizeof(Image))  ;
//	OUTPUT(idText,text,"sizeof(ChannelImage)="<<sizeof(ChannelImage))  ;
//	OUTPUT(idText,text,"sizeof(AnisotropicImage)="<<sizeof(AnisotropicImage))  ;
//	OUTPUT(idText,text,"sizeof(HistogramPanorama)="<<sizeof(HistogramPanorama))  ;
//	OUTPUT(idText,text,"sizeof(colorBuffer)="<<sizeof(colorBuffer))  ;

	slamData.requestAction(SLAMData::loadClusters);

}


void PanoramaImageProcessor::initialize() {

  ColorCorrector::load();

  static bool prevCameraMatrixUninitialized = true;
  if (prevCameraMatrixUninitialized && cameraMatrix.frameNumber != 0)
  {
    prevCameraMatrix = cameraMatrix;
		prevCameraMatrixUninitialized = false;
		Geometry::setupRadialCorrection(image.cameraInfo);
  }

  // Compute additional information about the image
  imageInfo.maxImageCoordinates.x = image.cameraInfo.resolutionWidth - 1;
  imageInfo.maxImageCoordinates.y = image.cameraInfo.resolutionHeight - 1;
  imageInfo.horizon = Geometry::calculateHorizon(cameraMatrix, image.cameraInfo);
  imageInfo.horizonInImage = Geometry::getIntersectionPointsOfLineAndRectangle(
  Vector2<int>(0,0), imageInfo.maxImageCoordinates, imageInfo.horizon, 
          imageInfo.horizonStart, imageInfo.horizonEnd);
  imageInfo.vertLine = imageInfo.horizon;
  imageInfo.vertLine.direction.x = -imageInfo.vertLine.direction.y;
  imageInfo.vertLine.direction.y = imageInfo.horizon.direction.x;
  imageInfo.vertLine.normalizeDirection();

	currentHistogramPanorama.flush();

	landmarksPercept.reset(image.frameNumber);
}



void PanoramaImageProcessor::applyImageFilters() {

  // 1. Start with raw image
	STOPWATCH("memcpy/image() ",correctedImage = image;)
  

  // 2. Correct blue filter
  STOPWATCH("ColorCorrector::correct() ",ColorCorrector::correct(correctedImage);)
  GENERATE_DEBUG_IMAGE(imageProcessorGeneral,
    INIT_DEBUG_IMAGE(imageProcessorGeneral, correctedImage);
  )
  SEND_DEBUG_IMAGE(imageProcessorGeneral);

  // 3. Convert to anisotropic
  STOPWATCH("covertToDouble() ",filtered.copyFromImage(correctedImage);)

  // 4. Simple gauss to remove noise, store that one
  STOPWATCH("AnisotropicSmoothing() ",filtered.AnisotropicGauss(1,1,0,0,0);)
  STOPWATCH("covertToUChar() ",filtered.copyToImage(smoothed);)

  // 5. Find horizontal edges, use horizon
  STOPWATCH("convertToDouble() ",filtered.copyFromImage(correctedImage);)
  STOPWATCH("AnisotropicDeriv() ",filtered.AnisotropicGauss(2,2,-imageInfo.horizon.direction.angle()/PI*180,1,0);)
//  img.copyToImage(edges,AnisotropicImage::autoscale);
	STOPWATCH("convertToMagnitudeMap() ",filtered.copyToMagnitudeMap(magnitudes);)

	// create blue edges RGB debug image
	int x,y,brightness;
	for(y=0;y<cameraResolutionHeight_ERS7;y++)
		for(x=0;x<cameraResolutionWidth_ERS7;x++) {
				brightness =  //Y = 0.3*R + 0.59*G + 0.11*B
					image.image[y][0][x];// argh already in YCbCr
//					((int)image.image[y][0][x]*30+
//					(int)image.image[y][1][x]*59+
//					(int)image.image[y][2][x]*11)/100;
				//brightness=0;
				edges.image[y][0][x]= brightness;
				edges.image[y][1][x]= brightness;
				edges.image[y][2][x]= brightness/2+128;
		}


}


void PanoramaImageProcessor::scanForTransitions() 
{
	// 6. Find transitions
  // 6.1 Find local maxima in vertical direction

  // 6.1.1 create vertical scanlet

  Vector2<int> scanlet[SCANLET_LENGTH];
//  double dx=0,dy=0;
	int i;
  for(i=0;i<SCANLET_LENGTH;i++) {
    scanlet[i].x = (int)(imageInfo.vertLine.direction.x*i);
    scanlet[i].y = (int)(imageInfo.vertLine.direction.y*i);
  }


	// 6.1.2 scan image for points above magic threshold value, find maxima, find hue of source, mark as done
  // 6.2 correct for radial distortions
  // 6.3 project them back onto 360deg plane
	bool done[cameraResolutionHeight_ERS7][cameraResolutionWidth_ERS7];
	memset(&done,0,cameraResolutionHeight_ERS7*cameraResolutionWidth_ERS7);
	int x,y,ix,iy;
	// start "inside" the image (5pix to the border)
 	for(y=5;y<magnitudes.height-5;y++) {
		for(x=5;x<magnitudes.width-5;x++) {
			if(!done[y][x] && (magnitudes.image[y][x]>MAGNITUDE_THRESHOLD_HIGH) ) {
				// search along scanlet for maxima
				int bestValue=-1,bestIndex=-1,dropIndexPos=-SCANLET_LENGTH,dropIndexNeg=-SCANLET_LENGTH;
				for(i=0;i<SCANLET_LENGTH;i++) {
					ix=x+scanlet[i].x;
					iy=y+scanlet[i].y;
					if((ix>=0)&&(iy>=0)&&(ix<magnitudes.width)&&(iy<magnitudes.height)) {
						// on screen
						//edges.image[iy][1][ix]=255;
						if(magnitudes.image[iy][ix]>bestValue) {
							bestValue = magnitudes.image[iy][ix];
							bestIndex = i;
						}
						if((magnitudes.image[iy][ix]<MAGNITUDE_THRESHOLD_LOW)) {
							dropIndexPos=i;
							break;
						}
  					//edges.image[iy][1][ix]=200;
						done[iy][ix]=true;
					}
				}
				
				for(i=0;i>-SCANLET_LENGTH;i--) {
					ix=x-scanlet[-i].x;
					iy=y-scanlet[-i].y;
					if((ix>=0)&&(iy>=0)&&(ix<magnitudes.width)&&(iy<magnitudes.height)) {
						// on screen
						//edges.image[iy][1][ix]=255;
						if(magnitudes.image[iy][ix]>bestValue) {
							bestValue = magnitudes.image[iy][ix];
							bestIndex = i;
						}
						if((magnitudes.image[iy][ix]<MAGNITUDE_THRESHOLD_LOW)) {
							dropIndexNeg=i;
							break;
						}
  					//edges.image[iy][1][ix]=200;
						done[iy][ix]=true;
					}
				}

				Vector3<unsigned char> firstColor,secondColor;
				
				if(bestIndex!=-SCANLET_LENGTH) {
					// maximum detected

					unsigned char r,g,b;

					if(dropIndexPos!=-SCANLET_LENGTH) {
						// pos source color detected

						ix=x+scanlet[dropIndexPos].x;
						iy=y+scanlet[dropIndexPos].y;

						firstColor=Vector3<unsigned char>(
																			smoothed.image[iy][0][ix],
																			smoothed.image[iy][1][ix],
																			smoothed.image[iy][2][ix]);

//						saturate(
//							smoothed.image[iy][0][ix],
//							smoothed.image[iy][1][ix],
//							smoothed.image[iy][2][ix],
//							Y,U,V);
//						Image::convertFromYCbCrToRGB(Y,U,V,r,g,b);

						if(slamData.currentAction >= SLAMData::clustersAvailable) 
							mixture.lookupColor(firstColor,firstColor);

						Image::convertFromYCbCrToRGB(
							firstColor.x,
							firstColor.y,
							firstColor.z,
							r,g,b);
						for(i=1;i<=SCANLET_DISPLAY_LENGTH;i++) {
							ix=x+scanlet[i].x;
							iy=y+scanlet[i].y;	
							if((ix>=0)&&(iy>=0)&&(ix<magnitudes.width)&&(iy<magnitudes.height)) {
								edges.image[iy][0][ix]=r;
								edges.image[iy][1][ix]=g;
								edges.image[iy][2][ix]=b;
							}
						}
					}

					if(dropIndexNeg!=-SCANLET_LENGTH) {
						// neg source color detected

						ix=x-scanlet[-dropIndexNeg].x;
						iy=y-scanlet[-dropIndexNeg].y;	

						secondColor=Vector3<unsigned char>(
																			smoothed.image[iy][0][ix],
																			smoothed.image[iy][1][ix],
																			smoothed.image[iy][2][ix]);

						if(slamData.currentAction >= SLAMData::clustersAvailable) 
							mixture.lookupColor(secondColor,secondColor);

						Image::convertFromYCbCrToRGB(
							secondColor.x,
							secondColor.y,
							secondColor.z,
							r,g,b);
						
						for(i=-1;i>=-SCANLET_DISPLAY_LENGTH;i--) {
							ix=x-scanlet[-i].x;
							iy=y-scanlet[-i].y;	
							if((ix>=0)&&(iy>=0)&&(ix<magnitudes.width)&&(iy<magnitudes.height)) {
								edges.image[iy][0][ix]=r;
								edges.image[iy][1][ix]=g;
								edges.image[iy][2][ix]=b;
							}
						}
						
					}
					// mark maximum WHITE
					if(bestIndex>=0) {
						ix=x+scanlet[bestIndex].x;
						iy=y+scanlet[bestIndex].y;	
					} else {
						ix=x-scanlet[-bestIndex].x;
						iy=y-scanlet[-bestIndex].y;	
					}

					r=g=b=128;

					if((dropIndexPos!=-SCANLET_LENGTH) && (dropIndexNeg!=-SCANLET_LENGTH)) {
						// valid feature vector established
						
						// correction for radial distortion
						Vector2<int>featurePosition;
						Geometry::radialDistortionCorrectionFast(ix,iy,featurePosition.x,featurePosition.y);

						// determine view angle
						Vector2<double> featureAngles;
						Geometry::calculateAnglesForPoint(
							featurePosition,cameraMatrix,prevCameraMatrix,image.cameraInfo,featureAngles);

						int sector = getSectorOfAngle(featureAngles);
						if(featureAngles.y>HORIZON_ABOVE) {
							// above horizon
							//r=g=b=255;
							r=g=b=0;
							
							featureDetected(
								0, 
								featureAngles,
								firstColor,
								secondColor);

							if((sector%2)==0)
								r = 255;//red
							else 
								g=r = 255;//or yellow
						}
						else {
							// below horizon
							r=g=b=64; 
						}
					  //OUTPUT(idText,text," ax="<<featureAngles.x<<" ay="<<featureAngles.y)  ;


					}

					edges.image[iy][0][ix]=r;
					edges.image[iy][1][ix]=g;
					edges.image[iy][2][ix]=b;
				}
			}//if not done
			done[y][x]=true;
		}//for
	}//for

	//send image
  GENERATE_DEBUG_IMAGE(imageProcessorScanLines,
    INIT_DEBUG_IMAGE(imageProcessorScanLines, edges);
  )
  SEND_DEBUG_IMAGE(imageProcessorScanLines);

}

int PanoramaImageProcessor::getSectorOfAngle(Vector2<double> &angles) {
	// TODO: correct distortion depending on learning-pose
	int sector = (int)((-angles.x/(2*PI) + 0.5)*SECTORS);
	return(sector);

}

void PanoramaImageProcessor::featureDetected(
				int pose, Vector2<double> &angles,
				Vector3<unsigned char> &firstColor,
				Vector3<unsigned char> &secondColor) 
{
	if(slamData.currentAction == SLAMData::recordColors) { 
		storeColor(firstColor);
		storeColor(secondColor);
	}

	if(slamData.currentAction >= SLAMData::clustersAvailable) {

		int sector = getSectorOfAngle(angles);

		int firstCluster = mixture.lookupColorNumber(firstColor);
		int secondCluster = mixture.lookupColorNumber(secondColor);

		//OUTPUT(idText,text,"storing transition in sector "<<sector)  ;

		Transition transition(firstCluster,secondCluster);
		transition.transitionCount = (int)(1/(mixture.weight[firstCluster]*mixture.weight[secondCluster]));

		currentHistogramPanorama.histogram[sector].add(transition,angles.y);
	}

}

void PanoramaImageProcessor::storeColor(Vector3<unsigned char> &color) {
	if(color.x==0) {
		OUTPUT(idText,text,"null color detected!")  ;
	}
	if(colorBufferCount<COLOR_BUFFER_SIZE) {
		colorBuffer[colorBufferCount++] = color;
	} else {	// in a imaginary buffer of correct (total) size, guess a position and when small enough store color there
		if((rand()%colorBufferVirtualCount) < COLOR_BUFFER_SIZE)		
			colorBuffer[rand() % COLOR_BUFFER_SIZE] = color;
	}
	colorBufferVirtualCount++;
}


void PanoramaImageProcessor::computeClusters() {
	mixture.initRandom();

	Vector3<double> training[COLOR_BUFFER_SIZE];

	//unsigned char c1,c2,c3;
	for(int i=0;i<colorBufferCount;i++) {
		training[i] = Vector3<double>(
										colorBuffer[i].x,
										colorBuffer[i].y,
										colorBuffer[i].z
									);
	}
	mixture.setTrainingSet(&training[0],colorBufferCount);
	mixture.initMeansOnTrainingSet();
	mixture.startEM(10);
	//mixture.unionCloseClusters(200);// closer than square of distance
	mixture.updateLookupTable();

}

void PanoramaImageProcessor::showDebugClusters() {
	Image clustered(smoothed);
	mixture.clusterImage(clustered);
	mixture.visualizeClusters(clustered);
	GENERATE_DEBUG_IMAGE(imageProcessorBall,
//			clustered.convertFromYCbCrToHSI(clustered);
		INIT_DEBUG_IMAGE(imageProcessorBall, clustered);
	)
	SEND_DEBUG_IMAGE(imageProcessorBall);
}


void PanoramaImageProcessor::execute()
{ 
	STOPWATCH("initialize() ",
		initialize();
	)

	if(slamData.currentAction==SLAMData::offline) return;

	switch(slamData.currentAction) {
		case SLAMData::resetImageProcessor: 
			colorBufferCount = 0;
			slamData.actionCompleted(true);
			break;

		case SLAMData::standby: 
			break;

		case SLAMData::loadClusters: 
			if(mixture.loadLookupTable("cluster.tab")) {
				OUTPUT(idText,text,"using pre-computed lookup table")  ;
				slamData.actionCompleted(true);
			} else {
				OUTPUT(idText,text,"no lookup table found. starting calibration")  ;
				slamData.actionCompleted(false);
			}
			break;

		case SLAMData::recordColors:
			STOPWATCH("applyImageFilters() ",
				// start image filtering
				applyImageFilters();
			)

			STOPWATCH("scanForTransitions() ",
				// scan for transitions
				scanForTransitions();
			)
			OUTPUT(idText,text,"color buffer count: "<<colorBufferCount<<" ("<<colorBufferVirtualCount <<")"<<" of "<<COLOR_BUFFER_SIZE)  ;
			if(colorBufferVirtualCount>20000 ) 
				slamData.actionCompleted(true);

			break;

		case SLAMData::clusterColors:
			STOPWATCH("computeClusters() ",
				computeClusters();
			)
			slamData.actionCompleted(true);
			break;

		case SLAMData::saveClusters: 
			mixture.saveLookupTable("cluster.tab");
			slamData.actionCompleted(true);

			break;

		default:
			if(slamData.currentAction >= SLAMData::clustersAvailable) {
				STOPWATCH("applyImageFilters() ",
					// start image filtering
					applyImageFilters();
				)

				STOPWATCH("scanForTransitions() ",
					// scan for transitions
					scanForTransitions();
				)

				STOPWATCH("generateSignatures() ",
					landmarksPercept.local = PanoramaView(currentHistogramPanorama);
				)

				//OUTPUT(idText,text,
				//		   "displaying derived sector signatures ("
				//		<< local.filledSectors << " filled, "
				//		<< local.invalidSectors<<" invalid)" );
				for(int sector=0;sector<SECTORS;sector++) {
					if(!landmarksPercept.local.sector[sector].empty) {
				//		OUTPUT(idText,text,"sector "<<sector<<" has "<<local.sector[sector].signatures<<" signatures");
						for(int signature=0;signature<landmarksPercept.local.sector[sector].signatures;signature++) {
				//			OUTPUT(idText,text,"    sector "<<sector<<", signature "<< signature<<": "<<local.sector[sector].signature[signature].transitions<<" transitions");
							for(int transition=0;transition < landmarksPercept.local.sector[sector].signature[signature].transitions;transition++) {
				//				OUTPUT(idText,text,"        "<<transition<<": "<<
				//					local.sector[sector].signature[signature].transition[transition].firstColor<<"<->"<<
				//					local.sector[sector].signature[signature].transition[transition].secondColor<<" ("<<
				//					local.sector[sector].signature[signature].transition[transition].transitionCount<<")"
				//				);
								drawTransition(
									sector,				
									transition,
									landmarksPercept.local.sector[sector].signature[signature].transition[transition]
								);
							}
						}
						
					}
				}
			}
	}


	DEBUG_DRAWING_FINISHED(selfLocator); 
	DEBUG_DRAWING_FINISHED(selfLocatorField);  

  prevCameraMatrix = cameraMatrix;
}

void PanoramaImageProcessor::drawTransition(int sector, int transition, Transition color) {
	double rad1 = ((double)sector)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad2 = ((double)sector+0.33)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad3 = ((double)sector+0.66)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad4 = ((double)sector+1)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double size = 100;
	double dist1 = 1000+transition*size;
	double dist2 = 1000+(transition+0.4)*size;
	double dist3 = 1000+(transition+0.8)*size;

	Vector3<unsigned char> c1 = 	mixture.clusterColor[color.firstColor],first;
	Vector3<unsigned char> c2 = 	mixture.clusterColor[color.secondColor],second;
	Image::convertFromYCbCrToRGB(c1.x,c1.y,c1.z,first.x,first.y,first.z);
	Image::convertFromYCbCrToRGB(c2.x,c2.y,c2.z,second.x,second.y,second.z);
//	second.x=255;
//	second.y=0;
//	second.z=0;


	OCTANGLE_RGB(selfLocatorField,
		sin(rad1)*dist1+robotPose.translation.x,cos(rad1)*dist1+robotPose.translation.y,
		sin(rad2)*dist1+robotPose.translation.x,cos(rad2)*dist1+robotPose.translation.y,
		sin(rad3)*dist1+robotPose.translation.x,cos(rad3)*dist1+robotPose.translation.y,
		sin(rad4)*dist1+robotPose.translation.x,cos(rad4)*dist1+robotPose.translation.y,
		sin(rad4)*dist2+robotPose.translation.x,cos(rad4)*dist2+robotPose.translation.y,
		sin(rad3)*dist2+robotPose.translation.x,cos(rad3)*dist2+robotPose.translation.y,
		sin(rad2)*dist2+robotPose.translation.x,cos(rad2)*dist2+robotPose.translation.y,
		sin(rad1)*dist2+robotPose.translation.x,cos(rad1)*dist2+robotPose.translation.y,
		first.x,
		first.y,
		first.z);
	OCTANGLE_RGB(selfLocatorField,
		sin(rad1)*dist2+robotPose.translation.x,cos(rad1)*dist2+robotPose.translation.y,
		sin(rad2)*dist2+robotPose.translation.x,cos(rad2)*dist2+robotPose.translation.y,
		sin(rad3)*dist2+robotPose.translation.x,cos(rad3)*dist2+robotPose.translation.y,
		sin(rad4)*dist2+robotPose.translation.x,cos(rad4)*dist2+robotPose.translation.y,
		sin(rad4)*dist3+robotPose.translation.x,cos(rad4)*dist3+robotPose.translation.y,
		sin(rad3)*dist3+robotPose.translation.x,cos(rad3)*dist3+robotPose.translation.y,
		sin(rad2)*dist3+robotPose.translation.x,cos(rad2)*dist3+robotPose.translation.y,
		sin(rad1)*dist3+robotPose.translation.x,cos(rad1)*dist3+robotPose.translation.y,
		second.x,
		second.y,
		second.z);
}

/*
*
*/
