/**
* @file PanoramaImageProcessor.h
* 
* Definition of class PanoramaImageProcessor
*
* @author Jurgen Sturm
*/


#ifndef __PanoramaImageProcessor_h_
#define __PanoramaImageProcessor_h_

#include "Modules/ImageProcessor/ImageProcessor.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Tools/Math/Geometry.h"
#include "Tools/Debugging/DebugImages.h"
#include "Modules/ImageProcessor/ImageProcessorTools/ColorCorrector.h"
#include "Modules/ImageProcessor/GT2004ImageProcessor/GT2004ImageProcessorTools.h"
#include "Representations/Perception/Image.h"
#include "AnisotropicFilterLibrary.h"
#include "EMGaussianMixtureLibrary.h"
#include "Representations/Perception/PanoramaLibrary.h"
#include "Representations/Perception/PanoramaParameters.h"

using namespace Panorama;

class PanoramaImageProcessor : public ImageProcessor
{
public:
  PanoramaImageProcessor(const ImageProcessorInterfaces& interfaces);
  
  /** Executes the module */
  virtual void execute();

	void initialize();
	void applyImageFilters();
	void scanForTransitions();
	int getSectorOfAngle(Vector2<double> &angles);
	void featureDetected(int pose, Vector2<double> &angles, Vector3<unsigned char> &firstColor,Vector3<unsigned char> &secondColor);
	void storeColor(Vector3<unsigned char> &color);
	void computeClusters();
  void showDebugClusters();
	void drawTransition(int sector, int transition, Transition color);
private:
  ColorCorrector colorCorrector; /**< The color correction tool. */
	CameraMatrix prevCameraMatrix;
  ImageInfo imageInfo; /**< Additional information about the current image */
	Image correctedImage;
	Image smoothed;
	Image edges; 
	ChannelImage magnitudes;
	AnisotropicImage filtered;
	EMGaussianMixture mixture;
	HistogramPanorama currentHistogramPanorama;

	int colorBufferCount;
	int colorBufferVirtualCount;
	Vector3<unsigned char> colorBuffer[COLOR_BUFFER_SIZE];

  DECLARE_DEBUG_COLOR_CLASS_IMAGE(segmentedImage1);
  DECLARE_DEBUG_IMAGE(imageProcessorGeneral);
  DECLARE_DEBUG_IMAGE(imageProcessorScanLines);
  DECLARE_DEBUG_IMAGE(imageProcessorBall);
  DECLARE_DEBUG_IMAGE(imageProcessorGoals);
};


#endif// __PanoramaImageProcessor_h_

/*
* $Log: CheckerboardDetector.h,v $
* Revision 1.1.1.1  2004/05/22 17:19:26  cvsadm
* created new repository GT2004_WM
*
* Revision 1.4  2004/03/08 01:38:56  roefer
* Interfaces should be const
*
* Revision 1.3  2004/02/29 13:46:48  dueffert
* localization in critical cases improved
*
* Revision 1.2  2003/12/15 10:57:24  dueffert
* calculation bug fixed
*
* Revision 1.1  2003/10/06 14:10:13  cvsadm
* Created GT2004 (M.J.)
*
* Revision 1.3  2003/09/01 10:20:11  juengel
* DebugDrawings clean-up 2
* DebugImages clean-up
* MessageIDs clean-up
* Stopwatch clean-up
*
* Revision 1.2  2003/08/08 11:24:51  dueffert
* doxygen docu corrected
*
* Revision 1.1  2003/07/30 14:50:02  dueffert
* CheckerboardDetector added
*
* Revision 1.5  2003/02/21 14:16:12  dueffert
* recognition of our checkerboard continued, see logs/special_localisator2.log
*
* Revision 1.4  2003/02/20 15:43:27  dueffert
* bug fixed
*
* Revision 1.3  2003/02/20 15:32:17  dueffert
* work continued
*
* Revision 1.2  2003/02/19 15:57:34  dueffert
* some checkerboard detection work
*
* Revision 1.1  2003/01/22 15:00:03  dueffert
* checkerboard stuff added
*
*
*/
