/**
* @file RBallSpecialist2.h
* 
* This file contains the definition of class RBallSpecialist2.
*
* @author <a href="mailto:sadprofessor@web.de">Bernd Schmidt</a>
*/
#ifndef RBALLSPECIALIST2_H
#define RBALLSPECIALIST2_H




#include "RasterSpecialist.h"
#include "Tools/Math/Geometry.h"
#include <vector>
#include <list>
#include "Tools/Math/Vector2.h"
#include "RasterStrategy.h"
#include "Modules/ImageProcessor/RasterImageProcessor/REdgeDetection.h"


/**	
  * @class RBallSpecialist2	
  * Implements a detection algorithm for balls.
  *				
  */
	
class RBallSpecialist2 :public RasterSpecialist  
{
	/**	*/
	enum{MAX_EDGE_POINTS = 30,MAX_CIRCLE_DIST = 2,EDGE_THR = 30,SCAN_THR = 30};
public:
	
	/**	Constructor
	*
	*		@param processor	The image processor interfaces.
	*		@param strategy	The strategy.
	*/
	RBallSpecialist2(RasterImageProcessor &processor,RasterStrategy &strategy);
	/**	Destructor*/
	virtual ~RBallSpecialist2();
	/** Invokes the specialist in the second scan stage at position (x,y) 
		*	(this member should be replaced later).
		*		@param x The x coordinate.
		*		@param y The y coordinate.	
		*/
	void invokeOnPostScan(int x, int y);
	/** Executes the detection algorithm. */
	void executePostProcessing();
	/** Invokes the specialist in the first scan stage at position (x,y)
		*	(this member should be replaced later).
		*		@param x The x coordinate.
		*		@param y The y coordinate.*/
	void invokeOnPreScan(int x,int y);
	/**	Getter for the type.
	*		@return The type of the specialist.
	*/
	virtual int getType();
	/**	Initializes the specialist.*/
	virtual void init();
	/**	Validates the circle, by comparing it to a color pattern.
	*		
	*		@param circle The circle.
	*		@return The validity.
	*/
	double validateCircle(Geometry::Circle &circle);
	/**	Validates the circle, by calculating the percantage of 
	*		edge points near the circle.
	*
	*		@param circle The circle.
	*		@return	The validity of the circle.
	*/
	double validateEdgePoints(Geometry::Circle &circle);
	/**	Calculates the coordinates of a vector that looks in direction angle.
	*		Only a helper.
	*
	*		@param angle input
	*		@param x output variable
	*		@param y output variable
	*/
	inline void getCoordinatesByAngle(double angle,double& x,double& y){
		x = cos(angle);
		y = sin(angle);
	}

	/**	A temporary variable.*/
	Vector2<int> temp;
	/**	The runs to analyze.*/
	std::list<LinePair> rows;

private:
	/**	The strategy.*/
	RasterStrategy *strategy;

	//SUSANEdgeDetectionLite edgeDetector;
	
	/**	The edge detector.*/
	REdgeDetection edgeScanner;
	/**	The normal threshold.*/
	int threshold;
	/**	The threshold for large balls.*/
	int minEdgeThreshold;

	/**	Adds the Ball to the BallPercept. 
	*		Makes some small tests befor generating a percept.
	*
	*		@param circle The circle detected as a ball.
	*		@param validity	The color validity for this circle.
	*/
	void addBallPercept(Geometry::Circle &circle,double validity);
	/**	Approximates a circle for small regions.
	*
	*		@param circle	The circle.
	*		@return Validity of the circle.
	*/
	double calculateSmallCircle(Geometry::Circle& circle);
	/**	Approximates the circle with a randomized algorithm.
	*		@param circle The result.
	*		@return	The validity of the result.
	*/
	double calculateCircleByEdges(Geometry::Circle& circle);
	/**	Approximates the circle with a randomized algorithm.
	*		Edge points are filtered with a star scan, which has its
	*		origin in the center of the box.
	*
	*		@param input The bounding box of the region seems to belong to a ball.
	*		@param circle The result.
	*		@return	The validity of the result.
	*/
	double calculateLargeCircle(const Box& input,Geometry::Circle& circle);
	/**	Filters the edge points with a contrast filter.
	*		
	*		@param segment The region, which is the input data.
	*		@return	True, if selected more than 3 points.
	*/
	bool filterBallPoints(std::list<LinePair>& segment);
	/**	Builds a bounding box for a segment.	
	*		
	*		@param segment The input data.
	*		@param box The result.
	*		@return	True, if building was succesful.
	*/
	bool createBox(std::list<LinePair>& segment,Box& box);
	/**	Calculates the average distance of the edge points to the circle.
	*
	*		@param circle The circle.
	*		@return	The average distance of all edge points.
	*/
	double middleEdgePointDist(Geometry::Circle &circle);
	
	/**	A buffer for the edge points of a region.*/
	std::vector<Vector2<int> > points;
	/**	The number of edge points.*/
	int numberOfEdgePoints;
	/**	The roundness of the last detected circle.*/
	double roundness;
	
	/**	The function defines a filter used in filterBallPoints().
	*
	*		@param x The x-coordinate.
	*		@param y The y-coordinate.
	*		@return True if the point lies on a edge.
	*/
	inline bool isEdge(int x,int y){
	//	return edgeDetector.isEdgePoint(rip->image,x,y,
	//		SUSANEdgeDetectionLite::componentA);
		return edgeScanner.isHorizontalEdge(x,y) || edgeScanner.isVerticalEdge(x,y);
	}
	/**	Setter for the edge filter threshold.
	*
	*		@param threshold A threshold.
	*/
	void setThreshold(int threshold);
};

#endif

/*
* Change log :
* 
* $Log: RBallSpecialist2.h,v $
* Revision 1.6  2004/09/09 11:08:04  spranger
* renamed GT2004EdgeDetection to REdgeDetection for consistency
*
* Revision 1.5  2004/09/08 14:39:02  wachter
* - Fixed some doxygen-errors
*
* Revision 1.4  2004/09/06 12:02:26  schmidtb
* commented almost all members, removed warnings in documentation

* did further code clean up
*
* Revision 1.3  2004/09/02 07:59:29  schmidtb
* Added RasterImageProcessor to repository, because we used it for the OpenChallenge.
*
* Revision 1.3  2004/05/25 13:27:34  schmidtb
* modified version of rip for open-challenge
*
* Revision 1.5  2004/04/22 16:57:26  pg_besc
* new version of RBallSpecialist2, now omidirectional scans for detection
*
* Revision 1.4  2004/04/20 08:12:38  pg_besc
* fixed bug
*
* Revision 1.3  2004/04/20 07:50:27  pg_besc
* new version of pre scan
*
* Revision 1.2  2004/03/25 15:26:10  pg_besc
* made some changes
*
*/
