/**
* @file RasterStrategy.h
* 
* This file contains the definition of class RasterStrategy.
*
* @author <a href="mailto:sadprofessor@web.de">Bernd Schmidt</a>
*/
#ifndef RASTERSTRATEGY_H
#define RASTERSTRATEGY_H

class RasterStrategy;

#include "RasterImageProcessor.h"
#include "Representations/Perception/ColorTableTSL.h"
#include <vector>

const int	Y_VALUE = 0, /**< Relative offset of Y component. */
			U_VALUE = cameraResolutionWidth_ERS7, /**< Relative offset of U component. */
			V_VALUE = 2 * cameraResolutionWidth_ERS7; /**< Relative offset of V component. */	

/** @class RasterStrategy 
*		Base-class for RasterStrategies.
*	
*		A RasterStrategy is a object that delegates a collection of specialists.
*		And decides what pixels are scanned on the image.
*		First the image has to be scanned, than the several post-processings
*		of the specialists have to be executed.
*		@author <a href="mailto:sadprofessor@web.de">Bernd Schmidt</a>
*/
class RasterStrategy  
{
public:

	/** Keys for the several activities can be made by strategies. */
	enum ActivationKeys{
		preScan = 0,
		postScan,
		ball,
		flag,
		goal,
		lines,
		foes,
		numberOfActivationKeys
	};

	typedef const unsigned char* I_Pin;

	/** True if the strategy scans over a ball.*/
	bool insideBall;
	/** True if the strategy scans over a box.*/
	bool insideBox;
	/** True if the strategy scans over a bridge beacon.*/
	bool insideBridge;
	/** True if the strategy scans over a player.*/
	bool insidePlayer;
	/** True if the strategy scans over the field.*/
	bool insideField;
	/** True if the strategy scans over a landmark.*/
	bool insideLM;

	/**	Constructor
	*
	*			@param iProcessor The image processor interfaces.
	*/
	RasterStrategy(RasterImageProcessor &iProcessor);
	/**	Destructor */
	virtual ~RasterStrategy();
	
	/** Initializes a strategy and the related specialists. 
	*	Call this before execute().*/
	virtual void init() = 0;
	/** Executes the strategy. */
	virtual void execute() = 0;

	protected:
	/** The image processor interfaces */
	RasterImageProcessor *rip;

	/** Implemented with ColorTable interface.
	*	Corrects the color before classifying.
	*	@param v input pixel
	*	@return The classified color of the pixel.
	*/	
	inline colorClass getColor(Vector2<int> v){
		return getColor(v.x,v.y);	
	};
	/** Implemented with ColorTable interface.
	* Corrects the color before classifying.
	*	@param x x-coordinate of the requested pixel
	*	@param y y-coordinate of the requested pixel
	*	@return The classified color of the pixel.*/
	inline colorClass getColor(int x,int y){
		unsigned char cy = rip->image.image[y][0][x];
		unsigned char cu = rip->image.image[y][1][x];
		unsigned char cv = rip->image.image[y][2][x];
		ColorCorrector::correct(x,y,cy,cu,cv);
		return rip->colorTable.getColorClass(cy,cu,cv);	
	};
	/** Implemented with ColorTable interface.
	*		@param x X-coordinate of the requested pixel.
	*		@param y Y-coordinate of the requested pixel.
	*		@param color The color class to match.
	*		@return	True if color equals getColor(x,y).
	*/
	inline bool checkColor(int x,int y,colorClass color){
		return	color == getColor(x,y);
	};
	/** Implemented with ColorTable interface.
	*		@param v The requested pixel.
	*		@param color The color class to match.
	*		@return	True if color equals getColor(x,y).
	*/
	inline bool checkColor(Vector2<int> v,colorClass color){
		return	color == getColor(v.x,v.y);
	};
	/** The theta function.
	*	@param dx first vector of the difference
	*	@param dy second vector of the difference
	*	@return a number between 0 and 360 with the same order as v.angle()*/
	inline double theta(int dx,int dy){
		double t = 0;
		int ax = abs(dx);
		int ay = abs(dy);
		if (dx == 0 && dy == 0) t = 0;
		else t = (double)dy /(double)(ax + ay);
		if (dx < 0) t = 2 - t;
		else if (dy < 0) t = 4 + t;
		return t*90; // multiplication with 90 is not really needed for correct ordering
	}

};

#endif // !defined(AFX_RASTERSTRATEGY_H__E73419DC_85CE_4D73_87C1_DEBCEC21740B__INCLUDED_)

/*
* Change log :
* 
* $Log: RasterStrategy.h,v $
* Revision 1.5  2004/09/06 12:02:26  schmidtb
* commented almost all members, removed warnings in documentation

* did further code clean up
*
* Revision 1.4  2004/09/03 11:32:14  nistico
* References to MSH2004ColorCorrector removed, now uses the unified ColorCorrector
*
* Revision 1.3  2004/09/02 07:59:29  schmidtb
* Added RasterImageProcessor to repository, because we used it for the OpenChallenge.
*
* Revision 1.16  2004/05/27 11:18:48  schmidtb
* new version with further work for open challenge
*
* Revision 1.15  2004/05/25 13:27:34  schmidtb
* modified version of rip for open-challenge
*
* Revision 1.16  2004/05/22 16:01:49  pg_besc
* -modified version of rip for bridge-recognition
*
* Revision 1.15  2004/05/18 10:52:14  pg_besc
* version of the last game at US-Open
*
* Revision 1.14  2004/04/15 19:09:02  pg_besc
* merged code
*
* Revision 1.13  2004/03/17 17:06:09  koh
* added enemyOnlySpecialist;
* added enemySpecialist2;
* added some int and bool variables
*
* Revision 1.12  2004/03/11 20:34:01  schmidtb
* new version of rip
*
* Revision 1.11  2004/03/04 09:54:52  schmidtb
* color correction integrated
*
* Revision 1.10  2004/03/03 12:53:22  schmidtb
* color correction integrated
*
* Revision 1.9  2004/03/01 14:17:26  koh
* added new strategy "RFlexibleStrategy" + new specialist "EnemyOnlySpecialist";
* changed references to "RDefaultStrategy" to references to "RasterStrategy" in RFieldSpecialist
* added Geometry::Line horizon to "RasterStrategy"
*
* Revision 1.8  2004/02/02 13:42:12  schmidtb
* merged sources of RIP. added som functions.
*
* Revision 1.7  2004/01/23 15:44:01  deom
* New specialist :: BoxSpecialist
* Recognizes both landmarks and goals
*
* Revision 1.6  2003/12/15 13:55:33  schmidtb
* Merged and patched new version of RasterImageProcessor.
*
* Revision 1.5  2003/12/02 21:59:02  schmidtb
* New version of RasterImageProcessor
*
* Revision 1.4  2003/12/02 12:45:44  neubach
* new EnemySpecialist added
*
* Revision 1.3  2003/11/20 10:26:56  schmidtb
* Ball Detection added
*
* Revision 1.2  2003/11/13 10:41:29  schmidtb
* renewed RBallSpeciaslist and Strategy
*
* Revision 1.1  2003/11/12 13:13:20  schmidtb
* new RasterImageProcessor added
*
*
*/
