#include "HistogramDistributionLibrary.h"

#define ZERO_MINIMAL_MASS 0.95

//****************************************** SectorHistogramDistribution
SectorHistogramDistribution::SectorHistogramDistribution() {
	reset();
}

void SectorHistogramDistribution::reset() {
	empty = true;
	invalidBitmask = true;
	int i,j,b;
	for(i=0;i<CLUSTERS;i++)
	for(j=0;j<CLUSTERS;j++) {
		for(b=0;b<BINS;b++) 
			freq[i][j][b] = 0;
		sum[i][j]=0;
	}
	normalization = 1;
}

void SectorHistogramDistribution::insertTransition(unsigned char bottom,unsigned char top,int bin) {
	freq[bottom][top][bin]++;
	sum[bottom][top]++;
	empty = false;
	invalidBitmask = true;
}

void SectorHistogramDistribution::updateBitmask() {
	for(int i=0;i<CLUSTERS;i++) {
		zeroBinMask[i] = 0;
		for(int j=0;j<CLUSTERS;j++) {
			if((freq[i][j][0]+1.)/(sum[i][j]+1.)>ZERO_MINIMAL_MASS)
				zeroBinMask[i] |= 1<<j;
		}
	}
	invalidBitmask = false;
}

//****************************************** PanoramaHistogramDistribution
PanoramaHistogramDistribution::PanoramaHistogramDistribution(const PanoramaHistogram &panoramaHistogram,RobotPose &robotPose) 
: panoramaHistogram(panoramaHistogram),robotPose(robotPose) 
{
	reset();
}

void PanoramaHistogramDistribution::reset() {
	for(int s=0;s<SECTORS;s++)
		sector[s].reset();

	sum.reset();
}

void PanoramaHistogramDistribution::insertTransition(int s, unsigned char bottom,unsigned char top,int count) {
	sector[s].insertTransition(bottom,top,count);
	sum.insertTransition(bottom,top,count);
}

void PanoramaHistogramDistribution::appendSectorToWorldModel(int worldSector,int seenSector) {
	const SectorHistogram &seen = panoramaHistogram.sector[seenSector];

	if(seen.empty) return;

	for(int i=0;i<CLUSTERS;i++) 
	for(int j=0;j<CLUSTERS;j++) 
			insertTransition(worldSector,i,j,seen.freq[i][j]);
}

void PanoramaHistogramDistribution::updateBitmask() {
	for(int sector=0;sector<SECTORS;sector++) {
		for(int i=0;i<CLUSTERS;i++) {
			zeroBinMaskSeen[sector][i] = (1<<(CLUSTERS)) -1;
			if(!panoramaHistogram.sector[sector].empty) {
				for(int j=0;j<CLUSTERS;j++) {
					if(panoramaHistogram.sector[sector].freq[i][j]!=0)
						zeroBinMaskSeen[sector][i] -= 1<<j;
				}
			}
		}
	}
}

double PanoramaHistogramDistribution::computeLikelihood(int worldSector,int seenSector) {
	const SectorHistogram &seen = panoramaHistogram.sector[seenSector];
	if(seen.empty) return(0);

	const SectorHistogramDistribution &world = sector[worldSector];
	if(world.empty) return(0);

	double transitionLikelihood;
	double result=1;
	unsigned char bin;
	int i,j,s=0;

	int commonZeroBin;
	for( i=0;i<CLUSTERS;i++) {
		if(OPTIMIZATIONS) {
			commonZeroBin = zeroBinMaskSeen[seenSector][i] & sector[worldSector].zeroBinMask[i];

			if(commonZeroBin == (1<<CLUSTERS)-1) {// zero on both sides
				s+=CLUSTERS;
				continue;
			}
		}

		for( j=0;j<CLUSTERS;j++) {
			if(OPTIMIZATIONS) {
				if((commonZeroBin>>j) & j) {
					s++;
					continue;
				}
			}

			// map to a histogram bin
			bin = seen.freq[i][j];

			transitionLikelihood = ((sector[worldSector].freq[i][j][bin]+1.) / (sector[worldSector].sum[i][j]+1.)) ;

			result *= max(MIN_LIKELIHOOD,transitionLikelihood);
		}
	}

	return(log(result)+log(ZERO_MINIMAL_MASS)*s);
}

void PanoramaHistogramDistribution::saveWorldModel(const char* filename) {
	OutBinaryFile stream(filename);

	stream << (*this); // row row row the boat gently down the stream
}

bool PanoramaHistogramDistribution::loadWorldModel(const char* filename) {
	InBinaryFile stream(filename);

	if(!stream.exists()) return(false);

	stream >> (*this);
	return(true);
}

In& operator>>(In& stream,PanoramaHistogramDistribution& panoramaHistogramDistribution)
{
	stream.read(&panoramaHistogramDistribution.sector,sizeof(SectorHistogramDistribution)*SECTORS);
	stream.read(&panoramaHistogramDistribution.sum,sizeof(SectorHistogramDistribution));
	panoramaHistogramDistribution.updateBitmask();
	return stream;
}

Out& operator<<(Out& stream, const PanoramaHistogramDistribution& panoramaHistogramDistribution)
{
	stream.write(&panoramaHistogramDistribution.sector,sizeof(SectorHistogramDistribution)*SECTORS);
	stream.write(&panoramaHistogramDistribution.sum,sizeof(SectorHistogramDistribution));
	return stream;
}
