#include "PanoramaLocator.h"
#include "Tools/FieldDimensions.h"
#include "Platform/SystemCall.h"
#include "Representations/Perception/SLAMData.h"
#include "Tools/Debugging/Stopwatch.h"
#include "Tools/Debugging/DebugImages.h"

#ifdef _WIN32

#define WRITE_RAWLOG false
#define READ_RAWLOG false

#else

#define WRITE_RAWLOG true
#define READ_RAWLOG false

#endif

#define MIN_SNR_AVERAGE 0.05

#ifndef min
#define min(a,b) (a<=b?a:b)
#endif

#ifndef max
#define max(a,b) (a>=b?a:b)
#endif


PanoramaLocator::PanoramaLocator(const SelfLocatorInterfaces& interfaces)
: SelfLocator(interfaces)
{
	selectedSpot = 0;
	localizationCount = 0;
	learnCount = 0;
	spotCount = 0;
	storedFrames =0;
	remainderOdometryDeltaRotation = 0;
	selectedSpot = 0;
	reference0 = 0;

	// prepare 20deg smoothing kernel and zero buffer
	double mass=0;
	int temp;
	for(temp=0;temp<360;temp++) {
		mass += kernel20[temp] = exp(- 0.5 * sqr(temp-180) / sqr(20));
		bearingAccumulator[temp] = 0;
	}
	for(temp=0;temp<360;temp++) 
		kernel20[temp] /= mass;//normalize sum to 1

	// try loading panorama
	if(loadPanorama("panorama.dat")) {
		TEXTLOG(idText,text,"SUCCESS: loaded panorama model");
/*
		int i;
		for(i=0;i<spotCount;i++) {
				int s;
				for(s=0;s<SECTORS;s++) {
					OUTPUT(idText,text,""<<i<<"\t"<<s<<"\t"<<spot[i]->panoramaHistogramDistribution.sector[s].normalization);

				}
		  
		}*/
	}

	TEXTLOG(idText,text,"number"<<"\t"<<
										"bearing"<<"\t"<<
										"confidence"<<"\t"
										);

	if(READ_RAWLOG) {
		InTextFile console("rawlog.txt");

		if(!console.eof()) {
			TEXTLOG(idText,text,"pos\trot\tconf.\tsize");
			char tag[255];
			console >> tag;
			if(tag[0]!='$') 
				OUTPUT(idText,text,"RAWLOG doesn't start with $-tag!");
			while(!console.eof() && tag[0]=='$')	{
				console >> localizationCount;
				console >> tag;

				if(tag[0]=='!') {
					console >> robotPose.translation.x >> robotPose.translation.y >> robotPose.rotation;
					robotPose.rotation = -robotPose.rotation*PI/180;
					while(robotPose.rotation>PI) robotPose.rotation-=2*PI;
					while(robotPose.rotation<=-PI) robotPose.rotation+=2*PI;
					robotPose.setValidity(1.0);
					console >> tag;
				} else {
					robotPose.translation.x = robotPose.translation.y = robotPose.rotation;
					robotPose.setValidity(0.0);
				}

				if(tag[0]!='-') 
					OUTPUT(idText,text,"RAWLOG section "<< localizationCount<<" contains no data!");

				int frame,lastFrame=-1;
				while(!console.eof() && tag[0]=='-')	{
					console >> frame;
					console >> head[frame];
					console >> offset[frame];
					console >> snr[frame];
					console >> tag;
					if(lastFrame!=-1 && lastFrame+1 != frame) {
						OUTPUT(idText,text,"RAWLOG section "<< localizationCount<<", data missing between frame "<<lastFrame<<" and "<<frame);
					}
					lastFrame = frame;
				}
				storedFrames = frame;

				if(robotPose.getValidity()==1.0) {
					if(false)
					OUTPUT(idText,text,
						"RAWLOG section "<< localizationCount<<
						" starting evaluation. tagged with x="<<robotPose.translation.x<<
						", y="<<robotPose.translation.y<<", "<<robotPose.rotation*180/PI<<" deg");
				}
				evaluateLocalization();
			}
		}
	}
}

void PanoramaLocator::resetSpots() {
	localizationCount = 0;
	learnCount = 0;
	storedFrames =0;
	remainderOdometryDeltaRotation = 0;

	// zero buffer
	for(int temp=0;temp<360;temp++) {
		bearingAccumulator[temp] = 0;
	}

	for(int i=0;i<spotCount;i++)
		delete spot[i];
	spotCount = 0;
}

void PanoramaLocator::appendSpot() {
	snrSpotAverage[spotCount] = 0;
	snrSpotAge[spotCount]=0;
	spot[spotCount++] = 
		new PanoramaModel(
			Pose2D(0,0,0),3000,//refine later
			landmarksPercept,
			robotPose,slamData);
	OUTPUT(idText,text,"adding spot #"<<spotCount)  ;
}

void PanoramaLocator::execute()
{
	if(slamData.currentAction==SLAMData::offline) return;

	switch(slamData.currentAction) {
		case SLAMData::resetSelfLocator: 
			resetSpots();
			slamData.actionCompleted(true);
			localizationCount = 0;
			learnCount = 0;
			storedFrames = 0;
			reference0 = 0;
			OUTPUT(idText,text,"resetting panorama model")  ;
			break;

		case SLAMData::standby: 
			if(storedFrames>0) {
				evaluateLocalization();
				storedFrames = 0;
			}
			slamData.actionIteration=0;
			break;

		case SLAMData::loadPanorama: 
			if(loadPanorama("panorama.dat"))
				slamData.actionCompleted(true);
			else 
				slamData.actionCompleted(false);

			break;

		case SLAMData::recordPanorama: 
			slamData.processingAndBusy();
			while(slamData.learningPoseNumber>=spotCount) appendSpot();

			//STOPWATCH("PanoramaLocator::recordPanorama() ",recordPanorama();)
			recordPanorama();
			localize();
			break;

		case SLAMData::savePanorama: 
			savePanorama("panorama.dat");
			slamData.actionCompleted(true);
			break;

		case SLAMData::localize: 
			slamData.processingAndBusy();
			while(slamData.learningPoseNumber>=spotCount) appendSpot();
			//STOPWATCH("PanoramaLocator::localize() ",localize();)
			localize();
			break;

		case SLAMData::normalize: 
			slamData.processingAndBusy();
			normalization();			
			break;

		default:
			if(slamData.currentAction <= SLAMData::clustersAvailable) {
				// ignore these states; wait for clusters to emerge..

			} else 
				OUTPUT(idText,text,"PanoramaLocator::execute() unknown SLAM action: "<<slamData.getActionName(slamData.currentAction))  ;
			

	}
	//OUTPUT(idText,text,"frame="<<landmarksPercept.frameNumber)  ;

  //robotPose.setFrameNumber(landmarksPercept.frameNumber);
  //robotPose.setPose(odometryData);
	//robotPose.setValidity(1.0);
//	OUTPUT(idText,text,"robotPose="<<robotPose.rotation)  ;

	INIT_DEBUG_IMAGE(imageProcessorScanLines,RGBDebug);
	SEND_DEBUG_IMAGE(imageProcessorScanLines);

	DEBUG_DRAWING_FINISHED(selfLocator); 
	DEBUG_DRAWING_FINISHED(selfLocatorField);  

	lastOdometryData = odometryData;
}

void convolve_gaussian(double sigma,double * in,double &rotation,double &sum) {
	int kernel_size = (int)sigma*3;
	double out[360];
	double kernel[360*10];
	int temp,world;

	for(temp=-kernel_size;temp<=kernel_size;temp++) 
		kernel[temp+kernel_size] = exp(- 0.5 * sqr(temp) / sqr(sigma));

	double kernelSum=0;
	for(temp=-kernel_size;temp<=kernel_size;temp++) 
		kernelSum += fabs(kernel[temp+kernel_size]);

	double scale = kernel[kernel_size]/kernelSum;

	for(world=0;world<360;world++) {
		double sumLeft=0,sumRight=0;
		double weightLeft=0,weightRight=0;
		for(temp=-kernel_size;temp<=0;temp++) {
			sumLeft += in[(world+temp+360*5)%360]*kernel[temp+kernel_size];
			weightLeft += fabs(kernel[temp+kernel_size]);
		}
		for(temp=1;temp<=kernel_size;temp++) {
			sumRight += in[(world+temp+360*5)%360]*kernel[temp+kernel_size];
			weightRight += fabs(kernel[temp+kernel_size]);
		}
		out[world] = (sumLeft + sumRight)/(weightLeft+weightRight);
	}

	//find max and sum
	int bestRotation = 0;
	for(int i=0;i<360;i++) {
		if(out[i]>out[bestRotation])
			bestRotation = i;
	}

	sum = out[bestRotation] / scale;
	rotation = (bestRotation - 180)*2*PI/360;
}
void convolve_band(double width,double * in,double &rotation,double &sum) {
	double out[360];
	int temp,world;

	for(world=0;world<360;world++) {
		double sum=0;
		for(temp=-(int)width/2;temp<=+(int)width/2;temp++) {
			sum += in[(world+temp+360)%360];
		}
		out[world] = sum;
	}

	//find max and sum
	int bestRotation = 0;
	for(int i=0;i<360;i++) {
		if(out[i]>out[bestRotation])
			bestRotation = i;
	}

	sum = out[bestRotation];
	rotation = (bestRotation - 180)*2*PI/360;
}

double dist(double a) {
	while(a<=-PI) a+= 2*PI;
	while(a>PI) a-= 2*PI;
	return(a);
}


void PanoramaLocator::linearRegress(
		double headRotation,
		double headWidth,
		double offsetRotation,
		double offsetWidth,
		double &slope,double &b,double &mass) {
	// linear regression, following http://people.hofstra.edu/faculty/Stefan_Waner/RealWorld/tutorialsf0/frames1_5.html
	double sumXY=0,sumX=0,sumY=0,sumSqrX=0,sumXSqr=0;
	mass=0;
	int usedFrames=0;
	double hNorm,oNorm;
	for(int i=0;i<storedFrames;i++) {
		hNorm = dist(head[i]-headRotation);
		oNorm = dist(offset[i]-offsetRotation);
		if(	
			oNorm<offsetWidth/2 
		&& 
			hNorm<headWidth/2 
		) {
			oNorm *= snr[i];
			hNorm *= snr[i];

			sumXY += hNorm*(oNorm+hNorm);
			sumX += hNorm;
			sumY += (oNorm+hNorm);
			sumSqrX += sqr(hNorm);
			mass += snr[i];
			usedFrames += 1;
		}
	}
	sumXSqr = sqr(sumX);

	slope = (usedFrames * sumXY - sumX*sumY) / ((double)usedFrames*sumSqrX - sumXSqr);
	b = (sumY - slope*sumX) / usedFrames;
}

void PanoramaLocator::evaluateLocalization() {
	double distribution[360];
	
	int i;
	for(i=0;i<360;i++)
		distribution[i] = 0;


	for(i=0;i<storedFrames;i++) {
		distribution[((int)(offset[i]/(2*PI)*360)+180+3600)%360] += snr[i];
	}

	double sum=0;
	for(i=0;i<360;i++)
		sum += distribution[i];

	double rotation,sum1,sum2,sum3;
	convolve_gaussian(5,distribution,rotation,sum1);
	convolve_gaussian(10,distribution,rotation,sum2);
	convolve_gaussian(20,distribution,rotation,sum3);
	double rotation2,sumRange;
	double range=180,diff=90;

	while(diff>1) {
//		convolve_band(range,distribution,rotation2,sumRange);
		convolve_gaussian(range,distribution,rotation2,sumRange);
		if(sumRange/sum>0.8) { 
			range -= diff;
			diff /= 2;
		} else {
			range += diff;
			diff /= 2;
		}
	}
	range = (int(range+0.5))*PI/180;

	double slopeFront,bFront,snrFront;
	double slopeLeft,bLeft,snrLeft;
	double slopeRight,bRight,snrRight;
	if(localizationCount==4)
		bFront =1;
	linearRegress(0,PI*3/4,rotation,2*range,slopeFront,bFront,snrFront);
	linearRegress(PI/3,PI*3/4,rotation,2*range,slopeRight,bRight,snrRight);
	linearRegress(-PI/3,PI*3/4,rotation,2*range,slopeLeft,bLeft,snrLeft);

	double slopeLeftRight = (slopeLeft*snrLeft + (1/slopeRight)*snrRight)/(snrLeft+snrRight);
	if(slopeFront>2 || slopeFront<0.5) slopeFront = 0.0;
	if(slopeLeftRight>2 || slopeLeftRight<0.5) slopeLeftRight = 0.0;

	TEXTLOG(idText,text,localizationCount<<"\t"<<
										(rotation2*180/PI)<<"\t"<<
										(range*180/PI)<<"\t"
										);

	for(i=1;i<spotCount;i++) {
		TEXTLOG(idText,text,"snr-average\t"<<i<<"\t"<<snrSpotAverage[i]);
	}

	/*
	OUTPUT(idText,text,"SNR: "<<sum/storedFrames);
	OUTPUT(idText,text,"\t"<<(int)(sum1/sum*100)<<"% within sigma=5deg");
	OUTPUT(idText,text,"\t"<<(int)(sum2/sum*100)<<"% within sigma=10deg");
	OUTPUT(idText,text,"\t"<<(int)(sum3/sum*100)<<"% within sigma=20deg");
	*/
}

void PanoramaLocator::recordPanorama() {
	WINDEBUG(
		spot[selectedSpot]->match();
		OUTPUT(idText,text,
			" spot "<<selectedSpot<<
			" frame "<<slamData.actionIteration<<
			" head "<<spot[selectedSpot]->currentAlignment.rotationHead*180/PI<<
			" offset "<<spot[selectedSpot]->currentAlignment.rotationOffset*180/PI<<
			" SNR "<<spot[selectedSpot]->currentAlignment.signalToNoiseRatio
			);
	)
	spot[selectedSpot]->learn();
	WINDEBUG(	spot[selectedSpot]->visualizeLikelihoods(RGBDebug);)
	//WINDEBUG(	spot[selectedSpot]->visualizeModel();)
}

void PanoramaLocator::matchSpot() {
	if(slamData.actionIteration<2) {
		slamData.actionIteration = 2;// auto init record
		localizationCount++;
		storedFrames = 0;
	}
	spot[selectedSpot]->match();
//	WINDEBUG(
//		if(slamData.actionIteration % 5 == 0) {
//			OUTPUT(idText,text,
//				"\t"<<slamData.actionIteration<<
//				"\t"<<spot[selectedSpot]->currentAlignment.rotationHead*180/PI<<
//				"\t"<<spot[selectedSpot]->currentAlignment.rotationOffset*180/PI<<
//				"\t"<<spot[selectedSpot]->currentAlignment.signalToNoiseRatio
//				);
//		}
//	)

	if(storedFrames<MAX_FRAMES) {
		offset[storedFrames] = spot[selectedSpot]->currentAlignment.rotationOffset;
		snr[storedFrames] = spot[selectedSpot]->currentAlignment.signalToNoiseRatio;
		head[storedFrames] = spot[selectedSpot]->currentAlignment.rotationHead;
		storedFrames++;
	}

	WINDEBUG(	spot[selectedSpot]->visualizeLikelihoods(RGBDebug);)
}

void PanoramaLocator::updateBearingAccumulatorByOdometry() {
	Pose2D odometryDelta = Pose2D(odometryData)-lastOdometryData;
	double shiftDeg = odometryDelta.rotation+remainderOdometryDeltaRotation;
	int shiftDegInt = (int)(shiftDeg+0.5);
	remainderOdometryDeltaRotation -= shiftDegInt;
	
	double buf[360];
	int i;
	for(i=0;i<360;i++) 
		buf[(i+shiftDegInt)%360] = bearingAccumulator[i];

	for(i=0;i<360;i++) 
		bearingAccumulator[i] = buf[i];
}

void PanoramaLocator::decayBearingAccumulatorByTime(int halftime) {
	double decay = exp(log(0.5)/halftime);

	for(int i=0;i<360;i++) 
		bearingAccumulator[i] *= decay;
}

void PanoramaLocator::updateBearingAccumulatorByAlignment() {
	int offset = ((int)(spot[selectedSpot]->currentAlignment.rotationOffset/(2*PI)*360)+180+3600)%360;
//	double snr = spot[selectedSpot]->currentAlignment.signalToNoiseRatio;

	for(int i=0;i<360;i++) 
		bearingAccumulator[(offset + i)%360] =
		bearingAccumulator[(offset + i)%360] +
			spot[selectedSpot]->currentAlignment.likelihood[(i+180%360)*SECTORS/360];
//			snr * kernel20[i];
}

void PanoramaLocator::findStrongestBearing() {
	double totalMass=0.0;
	int best=0;
	for(int i=0;i<360;i++) {
		totalMass += bearingAccumulator[i];
		if(bearingAccumulator[best]<bearingAccumulator[i])
			best=i;
	}

	estimatedPose.rotation = best*PI/180;
	if(totalMass==0)
		estimatedPose.setValidity(0.0);
	else
		estimatedPose.setValidity(bearingAccumulator[best]/(totalMass*kernel20[180]));

//	OUTPUT(idText,text,"strongestBearing="<<(estimatedPose.rotation*180/PI)<<" validity20="<<estimatedPose.getValidity());
}


void PanoramaLocator::savePanorama(const char* filename) {
	OutBinaryFile stream(filename);

	stream << spotCount;
	for(int i=0;i<spotCount;i++) {
		stream << (*spot[i]);
	}
}

bool PanoramaLocator::loadPanorama(const char* filename) {
	InBinaryFile stream(filename);
	if(!stream.exists()) return(false);

	resetSpots();
	int count,i;
	stream >> count;
	for(i=0;i<count;i++) 
		appendSpot();

	for(i=0;i<spotCount;i++) {
		stream >> (*spot[i]);
		OUTPUT(idText,text,"  spot "<<i<<": x="<<spot[i]->panoramaCenter.translation.x<<
			"mm, y="<<spot[i]->panoramaCenter.translation.y<<
			"mm, phi="<<(spot[i]->panoramaCenter.rotation*180/PI)<<"");
	}
	
	return(true);
}


void PanoramaLocator::localize() {
  robotPose.setFrameNumber(landmarksPercept.frameNumber);
	Pose2D odometryDelta = Pose2D(odometryData)-lastOdometryData;
	int i;

	if(slamData.localizationMode==SLAMData::odometry) {
		robotPose += odometryDelta;
		robotPose.setValidity(1.0);
		for(i=0;i<spotCount;i++) 
			snrSpotAverage[i]=0;
	} 

	if(slamData.localizationMode==SLAMData::trainingSpot) {
		selectedSpot = slamData.learningPoseNumber;
		spot[selectedSpot]->panoramaCenter.translation = slamData.learningPose.translation;
		robotPose.translation = spot[selectedSpot]->panoramaCenter.translation;
//		STOPWATCH("PanoramaLocator::decayBearingAccumulatorByTime(10) ",decayBearingAccumulatorByTime(10);;)
		decayBearingAccumulatorByTime(10);
	} 

	if(slamData.localizationMode==SLAMData::probabilistic) {
	/*
		double sum=0;
		for(i=0;i<spotCount;i++) 
			sum += max(MIN_SNR_AVERAGE,snrSpotAverage[i]);
		double rnd = sum*random();
		for(i=0;i<spotCount;i++) {
			rnd -= max(MIN_SNR_AVERAGE,snrSpotAverage[i]);
			if(rnd<=0) break;
		}

		selectedSpot = i;
		*/
		selectedSpot++;
		if(selectedSpot>=spotCount) selectedSpot=min(1,spotCount-1);
		robotPose.translation = spot[selectedSpot]->panoramaCenter.translation;
		//STOPWATCH("PanoramaLocator::decayBearingAccumulatorByTime(10) ",decayBearingAccumulatorByTime(10);;)
		decayBearingAccumulatorByTime(10);
	}

	if(slamData.bearingMode==SLAMData::trainingBearing) {
		robotPose.rotation = slamData.learningPose.rotation;
		robotPose.setValidity(1.0);
		//STOPWATCH("PanoramaLocator::decayBearingAccumulatorByTime(10) ",decayBearingAccumulatorByTime(10);;)
		decayBearingAccumulatorByTime(10);
	} 
		
	if(slamData.bearingMode>=SLAMData::estimateBearing) {
		//STOPWATCH("PanoramaLocator::matchSpot() ",matchSpot();;)
		matchSpot();
		switch(slamData.bearingMode) {
			case SLAMData::estimateBearing:
		//STOPWATCH("PanoramaLocator::decayBearingAccumulatorByTime(10) ",decayBearingAccumulatorByTime(10);;)
				decayBearingAccumulatorByTime(10);
				break;
			case SLAMData::estimateBearingSlow:
		//STOPWATCH("PanoramaLocator::decayBearingAccumulatorByTime(30) ",decayBearingAccumulatorByTime(50);;)
						decayBearingAccumulatorByTime(50);
				break;
			case SLAMData::estimateBearingNoDecay:
				// do nothing
				break;
			case SLAMData::setReference0:
				reference0 = estimatedPose.rotation;
//		STOPWATCH("PanoramaLocator::decayBearingAccumulatorByTime(30) ",decayBearingAccumulatorByTime(30);;)
				decayBearingAccumulatorByTime(30);
				break;
		}
//		STOPWATCH("PanoramaLocator::updateBearingAccumulatorByOdometry() ",updateBearingAccumulatorByOdometry();;)
//		STOPWATCH("PanoramaLocator::updateBearingAccumulatorByAlignment() ",updateBearingAccumulatorByAlignment();;)
		//STOPWATCH("PanoramaLocator::updateSpotAverageByAlignment(10) ",updateSpotAverageByAlignment(10);;)
		updateBearingAccumulatorByOdometry();
		updateBearingAccumulatorByAlignment();
		updateSpotAverageByAlignment(10);

		//STOPWATCH("PanoramaLocator::findStrongestBearing() ",findStrongestBearing();;)
		findStrongestBearing(); 

		robotPose.rotation = estimatedPose.rotation - reference0;
		robotPose.setValidity(estimatedPose.getValidity());
	}

	if(slamData.localizationMode==SLAMData::probabilistic) {
		WINDEBUG(visualize();)

		robotPose.translation = Vector2<double>(0,0);

		int bestSpot = 0;
		double snrSum = 0.5;
		for(i=1;i<spotCount;i++) {
			if(snrSpotAverage[i]>snrSpotAverage[bestSpot])
				bestSpot = i;
			snrSum +=max(0,snrSpotAverage[i]-0.5);
		}

		for(i=1;i<spotCount;i++) {
			robotPose.translation += spot[i]->panoramaCenter.translation * (max(0,snrSpotAverage[i]-0.5)/snrSum);
		}
	}

	robotPose.rotation = normalize(robotPose.rotation);
}

void PanoramaLocator::normalization() {
	if(slamData.localizationMode==SLAMData::trainingSpot) {
		selectedSpot = slamData.learningPoseNumber;
	} 

	if(slamData.localizationMode==SLAMData::probabilistic) {
		selectedSpot++;
		if(selectedSpot>=spotCount) selectedSpot=min(1,spotCount-1);
	}

	spot[selectedSpot]->match();
	spot[selectedSpot]->normalize();
	WINDEBUG(	spot[selectedSpot]->visualizeLikelihoods(RGBDebug);)
}

void PanoramaLocator::updateSpotAverageByAlignment(int halftime) {
	int offset = ((int)(spot[selectedSpot]->currentAlignment.rotationOffset/(2*PI)*360)+180+3600)%360;
	double snr = spot[selectedSpot]->currentAlignment.signalToNoiseRatio;

	double decay = exp(log(0.5)/halftime);
	snrSpotAverage[selectedSpot] *= decay;
	snrSpotAverage[selectedSpot] += (1-decay)*snr;
	snrSpotAge[selectedSpot] = landmarksPercept.frameNumber;

	/*
	double age = (landmarksPercept.frameNumber-snrSpotAge[selectedSpot])/5.0; // age in image frames
	double decay = exp(age*log(0.5)/halftime);
	snrSpotAverage[selectedSpot] *= decay;
	snrSpotAverage[selectedSpot] += (1-decay)*snr;
	*/
}

void PanoramaLocator::visualize() {
//CIRCLE(id, center_x, center_y, radius, penWidth, penStyle, penColor) 
	for(int s=0;s<spotCount;s++) {
		CIRCLE(selfLocatorField, 
			spot[s]->panoramaCenter.translation.x,
			spot[s]->panoramaCenter.translation.y,
			snrSpotAverage[s]*500,
			1,Drawings::ps_solid,Drawings::blue);
	}
}

