/**
* @file AutoShutter.cpp
* 
* Implementation of class AutoShutter
*
* @author <A href=mailto:jochen@kerdels.de>Jochen Kerdels</A>
*/

#include "AutoShutter.h"
#include "Representations/Perception/CameraParameters.h"
#include "Representations/Perception/CameraInfo.h"
#include "Tools/Location.h"



AutoShutter::AutoShutter(const SpecialVisionInterfaces& interfaces)
: SpecialVision(interfaces)
{
  reset();
}

void AutoShutter::reset()
{
  state = 0;
  for (int i = 0; i < 256; ++i)
    for (int j = 0; j < 9; ++j)
      histograms[i][j] = 0.0;
}

void AutoShutter::execute()
{
  if (state < 9*15)
  {
		if((state % 15)==0)
			setCamera(state/15,false,false);
		if((state % 15)==13)
			makeHistogram(state/15);

    state++;
    specialPercept.type = SpecialPercept::autoShutter;
    specialPercept.shutterSelected = false;
  }
  else
  {
		selectBest(); 
    specialPercept.type = SpecialPercept::autoShutter;
    specialPercept.shutterSelected = true;
    reset();
  }
}

void AutoShutter::makeHistogram(int which)
{
  const double sampleCount = 2500;
  int i;
  for (i = 0; i < sampleCount; ++i)
    ++histograms[image.image[rand()%(cameraResolutionHeight_ERS7*3/2)][0][rand()%(cameraResolutionWidth_ERS7/2)+cameraResolutionWidth_ERS7/4]][which];
  histograms[0][which] /= sampleCount;
  for (i = 1; i < 256; ++i)
    histograms[i][which] = histograms[i-1][which] + (histograms[i][which] / sampleCount);
}

void AutoShutter::setCamera(int minid,bool debug,bool write) {
  CameraParameters tmp;
  switch (minid)
  {
  case  0: {
             tmp.theGain         = CameraParameters::gain_low;
             tmp.theShutterSpeed = CameraParameters::shutter_slow;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: low gain, slow shutter, indoor mode");
             sensors.setCameraParameters(tmp);
           } break;
  case  1: {
             tmp.theGain         = CameraParameters::gain_mid;
             tmp.theShutterSpeed = CameraParameters::shutter_slow;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: mid gain, slow shutter, indoor mode");
             sensors.setCameraParameters(tmp);
           } break;
  case  2: {
             tmp.theGain         = CameraParameters::gain_high;
             tmp.theShutterSpeed = CameraParameters::shutter_slow;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: high gain, slow shutter, indoor mode");
             sensors.setCameraParameters(tmp);
           } break;
  case  3: {
             tmp.theGain         = CameraParameters::gain_low;
             tmp.theShutterSpeed = CameraParameters::shutter_mid;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
             sensors.setCameraParameters(tmp);
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: low gain, mid shutter, indoor mode");
           } break;
  case  4: {
             tmp.theGain         = CameraParameters::gain_mid;
             tmp.theShutterSpeed = CameraParameters::shutter_mid;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
             sensors.setCameraParameters(tmp);
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: mid gain, mid shutter, indoor mode");
           } break;
  case  5: {
             tmp.theGain         = CameraParameters::gain_high;
             tmp.theShutterSpeed = CameraParameters::shutter_mid;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
             sensors.setCameraParameters(tmp);
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: high gain, mid shutter, indoor mode");
           } break;
  case  6: {
             tmp.theGain         = CameraParameters::gain_low;
             tmp.theShutterSpeed = CameraParameters::shutter_fast;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
             sensors.setCameraParameters(tmp);
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: low gain, fast shutter, indoor mode");
           } break;
  case  7: {
             tmp.theGain         = CameraParameters::gain_mid;
             tmp.theShutterSpeed = CameraParameters::shutter_fast;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
             sensors.setCameraParameters(tmp);
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: mid gain, fast shutter, indoor mode");
           } break;
  case  8: {
             tmp.theGain         = CameraParameters::gain_high;
             tmp.theShutterSpeed = CameraParameters::shutter_fast;
             tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;
             sensors.setCameraParameters(tmp);
						 if(debug) TEXTLOG(idText,text,"AutoShutter selected: high gain, fast shutter, indoor mode");
           } break;
  }
	if(write) {
		OutBinaryFile fout(getLocation().getModelFilename("camera.cfg"));
		if (fout.exists()) {
			fout << tmp; 
		} else {
			TEXTLOG(idText,text,"AutoShutter couldn't write camera.cfg");
		}
	}
}

void AutoShutter::selectBest()
{
// we ONLY want to use these settings:
// 2 high gain, slow shutter
// 5 high gain, mid shutter
// 8 high gain, fast shutter
// 7 mid gain, fast shutter
// 6 low gain, fast shutter
// we select the setting with the distance between the 15% and 85% quantiles at least 64, starting with 2

  double sum;
	int quantileLeft,quantileRight,minid=2,i;

	int order[]={2,5,8,7,6};
	int orderCount = sizeof(order)/sizeof(int);

  for (int num=0;num<orderCount;num++) {
		i = order[num];
		sum =0;
		quantileLeft = -1;
		quantileRight = -1;
    for (int j = 0; j < 256; ++j) {
      sum = histograms[j][i];
			if(quantileLeft<0 && sum>=0.15) quantileLeft = j;
			if(quantileRight<0 && sum>=0.85) quantileRight = j;
    }
		switch(i) {
			case 2:
				TEXTLOG(idText,text,"high gain, slow shutter: "<<quantileLeft<<".."<<quantileRight);
				break;
			case 5:
				TEXTLOG(idText,text,"high gain, mid shutter: "<<quantileLeft<<".."<<quantileRight);
				break;
			case 8:
				TEXTLOG(idText,text,"high gain, fast shutter: "<<quantileLeft<<".."<<quantileRight);
				break;
			case 7:
				TEXTLOG(idText,text,"mid gain, fast shutter: "<<quantileLeft<<".."<<quantileRight);
				break;
			case 6:
				TEXTLOG(idText,text,"low gain, fast shutter: "<<quantileLeft<<".."<<quantileRight);
				break;
		}
		//OUTPUT(idText,text,"setting "<<i<<" "<<quantileLeft<<".."<<quantileRight);
		if(quantileRight - quantileLeft > 80) 
			minid = i;

  }

  if (minid >= 0)
		setCamera(minid,true,true);
}

/*
* Change log :
* 
* $Log: AutoShutter.cpp,v $
* Revision 1.2  2004/06/29 15:10:57  kerdels
* Special Vision AutoShutter...
*
* Revision 1.1  2004/06/28 10:54:34  kerdels
* added a class structure for a AutoShutter
*
* Revision 1.1  2004/06/28 10:51:04  kerdels
* added a class structure for a AutoShutter
*
*
*
*/
