/**
* @file VariableLighting.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file VariableLighting.cpp
* 
*/

/*

  How to use:

  1. Decide between which lighting conditions you want to discriminate
  2. Record logfiles and make colortables for them
  3. Determine the confidence range of each colortable (by replaying the logfile or live)
     Activate the VariableLight-challenge-agent (in pause mode it will automatically activate the range learning procedure)
  4. Update (or create) lighting.cfg in the colortable folder
  5. Done!

*/

#include "VariableLighting.h"
#include "Representations/Perception/CameraParameters.h"
#include "Representations/Perception/CameraInfo.h"
#include "Tools/Location.h"
#include "Processes/CMD/Cognition.h"

VariableSetting::VariableSetting() {
}

VariableLighting::VariableLighting(const SpecialVisionInterfaces& interfaces)
: SpecialVision(interfaces)
{
	slidingMedian.reset(INTERPOLATED_FRAMES);

	for(int i=0;i<257;i++) {
		distribution[i]=0;		// initialize median histogram
	}

	currentSetting = 0;
	switchSetting(0);

	numOfSettings = 0;
	countTooBright = 0;
	countTooDark = 0;
	learning = false;

	InTextFile infile(getLocation().getModelFilename("lighting.cfg"));
	if(infile.exists()) {
		while(!infile.eof()) {
			infile >> settings[numOfSettings].name; 
			if(strlen(settings[numOfSettings].name)>1) {
				//OUTPUT(idText, text, "SpecialVision::VariableLighting loading: "<<settings[numOfSettings].name);
				infile >> settings[numOfSettings].rangeLeft;
				infile >> settings[numOfSettings].rangeRight;
				char sbuf[255]; 
				// read gain setting
				infile >> sbuf; 
				settings[numOfSettings].gain = CameraParameters::gain_high;
				if(strcmpi(sbuf,"low")==0) 
					settings[numOfSettings].gain = CameraParameters::gain_low;
				else if(strcmpi(sbuf,"medium")==0) 
					settings[numOfSettings].gain = CameraParameters::gain_mid;
				else if(strcmpi(sbuf,"high")==0) 
					settings[numOfSettings].gain = CameraParameters::gain_high;
				else
					OUTPUT(idText, text, "WARNING: in setting "<<settings[numOfSettings].name<<" gain not recognized: "<<sbuf);
				// read shutter setting
				infile >> sbuf; 
				settings[numOfSettings].shutter = CameraParameters::shutter_mid;
				if(strcmpi(sbuf,"slow")==0) 
					settings[numOfSettings].shutter = CameraParameters::shutter_slow;
				else if(strcmpi(sbuf,"mid")==0) 
					settings[numOfSettings].shutter = CameraParameters::shutter_mid;
				else if(strcmpi(sbuf,"fast")==0) 
					settings[numOfSettings].shutter = CameraParameters::shutter_fast;
				else
					OUTPUT(idText, text, "WARNING: in setting "<<settings[numOfSettings].name<<" shutter speed not recognized: "<<sbuf);
				infile >> sbuf; 

				//OUTPUT(idText, text, "setting "<<settings[numOfSettings].name<<" shutter "<<CameraParameters::getShortShutterSpeedName(settings[numOfSettings].shutter));
				InBinaryFile stream(getLocation().getModelFilename(sbuf));
				settings[numOfSettings].colorTable.setFormat(ColorTable64::CT64);
				if (stream.exists())
				{
					stream >> settings[numOfSettings].colorTable;
				} else {
					OUTPUT(idText, text, "SpecialVision::VariableLighting can't open coltable: "<<getLocation().getModelFilename(sbuf));
				}

				numOfSettings++;
			} else 
				infile >> endl;
		}
		OUTPUT(idText, text, "SpecialVision::VariableLighting loaded "<<numOfSettings <<" settings");
	} else {
		OUTPUT(idText, text, "SpecialVision::VariableLighting lighting.cfg not found!");
	}
}

void VariableLighting::switchSetting(int delta) {
	if(currentSetting+delta<0) {
		countTooDark++;
	} else if(currentSetting+delta>=numOfSettings) {
		countTooBright++;
	} else {
		timeToRecover = INTERPOLATED_FRAMES;
		currentSetting += delta;
		writeColortableNextFrame = true;

        CameraParameters tmp;
//        tmp.theGain         = CameraParameters::gain_high;			// fixed setting
        tmp.theGain         = settings[currentSetting].gain;
        tmp.theShutterSpeed = settings[currentSetting].shutter;

		if(settings[currentSetting].shutter==CameraParameters::shutter_slow) {
//			specialPercept.headSpeedFactor = false;
		} else {
//			specialPercept.headSpeedFactor = true;
		}


        tmp.theWhiteBalance = CameraParameters::wb_indoor_mode;		// fixed setting
        sensors.setCameraParameters(tmp);

		OUTPUT(idText, text, 
			frameNumber
			<<" switching to "<<
			settings[currentSetting].name
			<<" shutter "<<CameraParameters::getShortShutterSpeedName(settings[currentSetting].shutter));
	}
}


void VariableLighting::execute()
{
	bool oldLearning = learning;
	learning = (specialVisionRequest.specialVisionRequest==SpecialVisionRequest::variableLightingLearn);

	if(!oldLearning && learning) {
		// re-init brightness distribution array
		for(int i=0;i<257;i++) 
			distribution[i]=0;
	}

	frameNumber = image.frameNumber;
	if(writeColortableNextFrame) {
		ColorTable64 *currentColorTable = (ColorTable64*)&colorTable;
		ColorTable64 *newColorTable = (ColorTable64*)&settings[currentSetting].colorTable;

		memcpy(&currentColorTable->colorClasses,&newColorTable->colorClasses,64*64*64);

		writeColortableNextFrame = false;
	}


	int w = image.cameraInfo.resolutionWidth;
	int h = image.cameraInfo.resolutionHeight;
	
	int gridSize = 10; // gridSize x gridSize to analyse
	int gridIncX = w / gridSize;
	int gridIncY = h / gridSize;

	// compute histogram of sample grid (Y values)
	int hist[256];

	for(int i=0;i<256;i++) hist[i]=0;

	for (int x=0;x<w;x+=gridIncX) {
		for (int y=0;y<h;y+=gridIncY)	{
			hist[image.image[x][0][y]]++;
		}
	}

	// find median
	int half = gridSize * gridSize / 2;

	int count=0; 
	int median = 0 ;
	while((median<255)&&(count<half)) 
		count += hist[median++];	// add the bin

	slidingMedian.add(median);
	slidingMedian.trigger();

	distribution[median]++;

	if(timeToRecover>0) {
		timeToRecover--;
	} else {
		if(!learning) {	// never switch when learning
			// maybe switch?
			if(slidingMedian.getAverage() < settings[currentSetting].rangeLeft) {
				switchSetting(-1);
			} else
			if(slidingMedian.getAverage() > settings[currentSetting].rangeRight) {
				switchSetting(+1);
			}
		}
	}

	
	if(image.frameNumber-lastDebugFrame>100) {

		if(learning) {
			int sum = 0;
			int weightedSum = 0;

			for(int i=0;i<257;i++) {
				sum += distribution[i];
				weightedSum += i*distribution[i];
			}

			int halfSum = weightedSum /2;					// move to the mass center
			int mid=0;
			count =0;
			while((mid<256) && (count<halfSum))
				count+=mid*distribution[mid++];

			int countLeft = count;							// search left for the border of the NORMALITY range
			int rangeLeft = mid;

			while(
				(rangeLeft>0)
				&&
				(countLeft>
						(weightedSum*((1-NORMALITY)/2) ) ))
				countLeft -= rangeLeft*distribution[rangeLeft--];

			int countRight = count;							// and to the right it goes..
			int rangeRight = mid;

			while(
				(rangeRight<256)
				&&
				(countRight<
						(weightedSum*(1 - ((1-NORMALITY)/2) ))))
				countRight += rangeRight*distribution[rangeRight++];

			OUTPUT(idText, text, "current: "<<slidingMedian.getAverage()<<" ("<<median<<")"<<"  range:"<<rangeLeft<<".."<<rangeRight);
		} else {
			// default statistics output
//			OUTPUT(idText, text, settings[currentSetting].name<<": median: "<<
//				settings[currentSetting].rangeLeft<<"<"<<slidingMedian.getAverage()<<"<"<<settings[currentSetting].rangeRight<<" (tooBright "<<countTooBright<<" tooDark "<<countTooDark<<")");
		}


		lastDebugFrame = image.frameNumber;
	}



	
}


/*
* Change log :
* 
* $Log: VariableLighting.cpp,v $
*
*
*/
