/**
* @file UDPHandler.h
*
* File modified by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/** 
* @file  Platform/Aperios1.3.2/UDPHandler.h
*
* Class for using NetSender/NetReceivers over UDP 
*
* @author <A href=mailto:robocup@m-wachter.de>Michael Wachter</A>
*
*/


#ifndef	_UDPHandler_H_
#define	_UDPHandler_H_


#include "UDPEndpoint.h"
#include "DDPHandler.h"
#include "GameControllerUDPHandler.h"
#include "NetHandler.h"
#include "NetSender.h"
#include "NetReceiver.h"
#include "Tools/Player.h"
#include "NetConfig.h"

class UDPHandler;



/**
*   The UDP-Handler and UDPHandlerEndpoint are used sends team communication data via an UDP-based protocol.
*
*   There are three UDPHandlerEndpoints which are doing the work of sending and receiving the data. 
*   To distinguish between the different teammates the outgoing packages are send to the port number 
*   TEAMPORTBASE + own player number. So the receiving robot knows where the package comes from.
*/

class UDPHandlerEndpoint : public UDPEndpoint , public NetHandler
{
public:
	UDPHandlerEndpoint();
	~UDPHandlerEndpoint();
    virtual void onSendingDone();  /**< sets the sender active again */
  	virtual void onReceive(void* data, int size); /**< receives data and hands it over to the reciever */
	virtual void sendPackage(int SenderNumber, void* package, int size); /**< called by the sender to send the package */
	NetSenderBase* sender; /**< the associated sender */
	NetReceiverBase* receiver; /**< the associated reciver */
	DDPHandler* ddpHandler; /**< the ddpHandler providing the IPs of the teammates */
	UDPHandler* udpHandler; /**< the UDP-Handler which is handling this UDPHandlerEndpoint */
	int lastReceivedSerial; /**< serial of the last received package */
	int sendSerial; /**< serial of the last send package */
};

/*
 * The UDPHandler manages starting up the UDPHandlerEndpoints
 * and associates the senders and receivers to them.
 */

class UDPHandler 
{
  public:
    UDPHandler();
    ~UDPHandler(){};

	/**
	 * Associates senders and receiver to UDPHandlerEndpoints
	 */

    void addSenderAndReceiver(NetSenderBase& sender, NetReceiverBase& receiver);
    void start(); /**< initializes everything */
    void doRegularStuff(); /**< called by the UDPHandlerEndpoints */
	GameControllerUDPHandler gameControllerUDPHandler;
	
  private:
	int numberOfEndpoints; /**< actual number of UPDHandlerEndpoints */
    UDPHandlerEndpoint endPoints[Player::numOfPlayerNumbers-1]; /**< list of UDPHandlerEndpoints */
	DDPHandler ddpHandler; /**< the DDP-Handler */
	
	UDPHandlerEndpoint gameControllerEndPoint;
};

#endif 

/*
 *
 * Changelog :
 *
 * $Log: UDPHandler.h,v $
 * Revision 1.2  2004/09/12 20:10:47  wachter
 * Documentation-fixes
 *
 * Revision 1.1.1.1  2004/05/22 17:23:41  cvsadm
 * created new repository GT2004_WM
 *
 * Revision 1.6  2004/05/14 14:12:08  wachter
 * - Added communication support for 5 robots
 * - rewrote parts of team-communication to be faster and more stable
 *
 * Revision 1.5  2004/01/23 18:28:47  wachter
 * Added detection of lost packets
 *
 * Revision 1.4  2004/01/21 17:33:09  wachter
 * UDP Team-communication now working with packets<1400 bytes.
 * Not activated at the moment.
 *
 * Revision 1.3  2004/01/20 14:21:41  wachter
 * - Added sender-number to NetSender
 * - worked on with Team-Communication
 *
 * Revision 1.2  2004/01/09 15:44:30  wachter
 * Worked on with the Dog-Discovery-Protocol
 *
 * Revision 1.1  2003/12/21 19:27:03  wachter
 * Added classes for Sender/Receiver over TCP and UDP.
 * ( PLEASE DO NOT USE THIS NOW ! )
 *
 *
 */
