/**
* @file PanoramaLibrary.cpp
* 
* 
* @author Jurgen Sturm
*/

#include "Representations/Perception/PanoramaLibrary.h"

namespace Panorama {

	Transition::Transition() {
		firstColor = 255;
		secondColor = 255;
		transitionCount = 0;
	}

	Transition::Transition(unsigned char color1,unsigned char color2) {
		firstColor = color1;
		secondColor = color2;
		transitionCount = 1;
	}

	bool Transition::compatible(Transition &other) {
		return(
			((firstColor == other.firstColor) &&(secondColor == other.secondColor))
			||
			((255 == other.firstColor) &&(255 == other.secondColor))
			||
			((firstColor == 255) &&(secondColor == 255))
			);
	}

	void Transition::merge(Transition &other) {
		transitionCount += other.transitionCount;
		if(firstColor==255) firstColor=other.firstColor;
		if(secondColor==255) secondColor=other.secondColor;
	}

	HistogramSignature::HistogramSignature() {
		empty = true;
		valid = true;
		for(int bin=0;bin<TRANSITION_BINS;bin++)
			for(int cluster=0;cluster<CLUSTERS;cluster++) {
				color[0][bin][cluster]=0;
				color[1][bin][cluster]=0;
			}
	}

	void HistogramSignature::flush() {
		(*this) = HistogramSignature();
	}

	void HistogramSignature::add(Transition &other,double verticalAngle) {
		// compute bin
		int bin = (int)(((verticalAngle / (PI)) + 0.5)*TRANSITION_BINS);// 0..TRANSITION_BINS corresponding to -90..90deg
		// count
		color[0][bin][other.firstColor] += other.transitionCount;
		color[1][bin][other.secondColor] += other.transitionCount;

		empty = false;
	}

	void HistogramSignature::compress() {
		// compress and check validity
		//
		int firstOrSecond,bin,cluster;

		// first compress
		for(firstOrSecond=0;firstOrSecond<2;firstOrSecond++) {
			for(bin=0;bin<TRANSITION_BINS;bin++) {
				for(cluster=0;cluster<CLUSTERS;cluster++) {
					if(color[firstOrSecond][bin][cluster]>0) {
						// start dragging values of past DISTANCE_TRANSTION bins up..
						for(int drag=1;drag<DISTANCE_TRANSTION;drag++) {
							if(bin-drag>=0) {
								color[firstOrSecond][bin][cluster] += color[firstOrSecond][bin-drag][cluster];
								color[firstOrSecond][bin-drag][cluster] = 0;
							}
						}
					}
				}
			}
		}

		// then ensure validity. bigger values kill smaller values. outch.
		for(firstOrSecond=0;firstOrSecond<2;firstOrSecond++) {
			for(bin=0;bin<TRANSITION_BINS;bin++) {
				for(cluster=0;cluster<CLUSTERS;cluster++) {
					if(color[firstOrSecond][bin][cluster]>0) {
						// here we have something, fight against everything in same or lower DISTANCE_TRANSTION bins
						for(int drag=0;(drag<DISTANCE_TRANSTION) && (color[firstOrSecond][bin][cluster]>0);drag++) {
							if(bin-drag>=0) {
								for(int otherCluster=0;(otherCluster<CLUSTERS) && (color[firstOrSecond][bin][cluster]>0);otherCluster++) {
									if(otherCluster != cluster) {
										if(color[firstOrSecond][bin][cluster]>color[firstOrSecond][bin-drag][otherCluster]) {
											// we live, other cluster dies
											color[firstOrSecond][bin-drag][otherCluster] = 0;
										} else {
											// we die
											color[firstOrSecond][bin][cluster] = 0;
											break; // dead man kills nobody
										}
									}
								}								
							}
						}
					}
				}
			}
		}


	}

	Signature::Signature() {
		transitions = 0;
		totalTransitionCount = 0;
		signatureCount = 0;
	}

	Signature::Signature(HistogramSignature &histogram) {
		histogram.compress();

		transitions = 0;
		totalTransitionCount = 0;
		signatureCount = 0;
		// we assume that histogram is valid

		for(int bin=0;bin<TRANSITION_BINS;bin++) {
			for(int cluster1=0;cluster1<CLUSTERS;cluster1++) {
				if(histogram.color[0][bin][cluster1]>0) {	
					// search highest value within DISTANCE_TRANSTION bins for second color
					Transition current;

					current.firstColor = cluster1;
					int transCount=0;
					int best2=0;
					for(int cluster2=0;cluster2<CLUSTERS;cluster2++) {
						for(int drag=-DISTANCE_TRANSTION+1;drag<DISTANCE_TRANSTION;drag++) {
							if((bin+drag>=0) && (bin+drag<TRANSITION_BINS)) {
								if(histogram.color[1][bin-drag][cluster2]>0) {	
									if(transCount<histogram.color[1][bin-drag][cluster2]) {
										// better
										current.secondColor = cluster2;
										transCount = histogram.color[1][bin-drag][cluster2];
										best2=cluster2;
									}
								}
							}
						}
					}

					current.transitionCount = min(transCount,histogram.color[0][bin][cluster1]);// smaller one rules
					if(current.transitionCount>TRANSITION_NOISE) {

						if(0 &&(transitions>0) && (transition[transitions-1].compatible(current)))
							// merge 
							transition[transitions-1].merge(current);
						else
							// or create new
							transition[transitions++] = current;									

						totalTransitionCount += current.transitionCount;
						histogram.color[0][bin][cluster1] = 0;	// take it out of the matrix
						histogram.color[1][bin][best2] = 0;			// and this one too
						signatureCount = 1;// houston, we have something..
						if(transitions==TRANSITIONS) return;
					}
				}
			}
		}
	}


	bool Signature::compatible(Signature &other) {
		// not same amount of transitions
		if(transitions!=other.transitions) 
			return(false);

		// now check transitions..
		for(int i=0;i<transitions;i++) 
			if(!transition[i].compatible(other.transition[i])) return(false);

		// obviously we are made of the same wood
		return(true);
	}

	void Signature::merge(Signature &other) {
		totalTransitionCount += other.totalTransitionCount;
		signatureCount += other.signatureCount;
		for(int i=0;i<transitions;i++) 
			transition[i].merge(other.transition[i]);

	}


	Sector::Sector() {
		flush();
	}

	void Sector::flush() {
		empty = true;
		signatures = 0;
		totalSignatureCount = 0;
	}

	void Sector::appendOrMerge(Signature &other) {
		empty = false;

		for(int s=0;s<signatures;s++) {
			if(signature[s].compatible(other)) {
				signature[s].merge(other);
				return;
			}
		}

		if(signatures==SIGNATURES) {	//full?
			// sort signatures
			sortSignaturesOnFrequency();
			signature[SIGNATURES-1] = other; // then overwrite the last, should filter out (some) noise..
		} else {
			signature[signatures++] = other;
		}
	}

	void Sector::sortSignaturesOnFrequency() {
		Signature temp;
		for(int i=0;i<signatures;i++)
			for(int j=0;j<signatures-1;j++)
				if(signature[i].signatureCount < signature[i+1].signatureCount) {
					// swap i and i+1
					temp = signature[i];
					signature[i] = signature[i+1];
					signature[i+1] = temp;
				}
	}

	void HistogramPanorama::flush() {
		for(int i=0;i<SECTORS;i++)
			histogram[i].flush();
	}

	PanoramaView::PanoramaView() {
		filledSectors = 0;
	}

	PanoramaView::PanoramaView(HistogramPanorama &histogramPanorama) {
		filledSectors = 0;

		for(int i=0;i<SECTORS;i++) {
			if(histogramPanorama.histogram[i].empty) {
				// empty
			} else {
				// create sector with a single signature
				Signature sig(histogramPanorama.histogram[i]);
				sector[i].appendOrMerge(sig);
				filledSectors++;
			}
		}
	}

	void PanoramaView::flush() {
		filledSectors = 0;

		for(int i=0;i<SECTORS;i++)
			sector[i].flush();
	}

}

In& operator>>(In& stream,Panorama::PanoramaView& panoramaView)
{
	panoramaView.flush();

	stream >> panoramaView.filledSectors;

	int sector;
	for(int i = 0;i<panoramaView.filledSectors;i++) {
		stream >> sector;

		stream.read(&panoramaView.sector[sector],sizeof(Panorama::Sector));
	}
	return stream;
}

Out& operator<<(Out& stream, const Panorama::PanoramaView& panoramaView)
{
	stream << panoramaView.filledSectors;

	for(int sector = 0;sector<SECTORS;sector++) {
		if(!panoramaView.sector[sector].empty) {
			stream << sector;
			stream.write(&panoramaView.sector[sector],sizeof(Panorama::Sector));
		}
	}
	return stream;
}




SectorHistogram::SectorHistogram() {
	reset();
}

inline void SectorHistogram::reset() {
	empty = true;
	horizon = 0;
	sum = 0;
}

inline void SectorHistogram::initialize() {
	int i,j;
	for(i=0;i<CLUSTERS;i++) 
	for(j=0;j<CLUSTERS;j++)
		freq[i][j] = 0;
	sum = 0;
	empty = false;
}

inline void SectorHistogram::insertTransition(unsigned char bottom,unsigned char top) {
	if(empty) initialize();
	freq[bottom][top]++;
	sum++;
}

PanoramaHistogram::PanoramaHistogram() {
	reset();
}

void PanoramaHistogram::reset() {
  sum.reset();
	for(int s=0;s<SECTORS;s++)
		sector[s].reset();
}

void PanoramaHistogram::insertTransition(int s,unsigned char bottom,unsigned char top) {
	sector[s].insertTransition(bottom,top);
	sum.insertTransition(bottom,top);	// TODO: this can be done more optimal
}


In& operator>>(In& stream,PanoramaHistogram& panoramaHistogram)
{
	panoramaHistogram.reset();

	int filledSectors;

	stream >> filledSectors;
	int sector;
	for(int i = 0;i<filledSectors;i++) {
		stream >> sector;

		stream.read(&panoramaHistogram.sector[sector],sizeof(SectorHistogram));
	}
	return stream;
}

Out& operator<<(Out& stream, const PanoramaHistogram& panoramaHistogram)
{
	int filledSectors = 0;
	for(int i = 0;i<SECTORS;i++) 
		if(!panoramaHistogram.sector[i].empty) filledSectors++;
		
	stream << filledSectors;

	for(int sector = 0;sector<SECTORS;sector++) {
		if(!panoramaHistogram.sector[sector].empty) {
			stream << sector;
			stream.write(&panoramaHistogram.sector[sector],sizeof(SectorHistogram));
		}
	}
	return stream;
}
/*
*
*/
