/**
* @file PanoramaLibrary.h
* 
* Definition of class PanoramaLibrary
*
* @author Jurgen Sturm
*/


#ifndef __PanoramaLibrary_h_
#define __PanoramaLibrary_h_

#include <stdlib.h>
#include <string.h>
#include "Modules/ImageProcessor/PanoramaImageProcessor/EMGaussianMixtureLibrary.h"
#include "Representations/Cognition/RobotPose.h"
#include "Representations/Perception/PanoramaParameters.h"
// signatures to store per segment
#define SIGNATURES 1
// maximum number of color transitions to store in one signature
#define TRANSITIONS 5
// theoretically -90..+90 degrees, stored in RAW_TRANSITIONS different histogrammic bins
// for 150 pix i guess that 50 is a good choice, but, 150pix correspond to approx 50 deg, therefore
// for 180 deg i start with 180 bins
#define TRANSITION_BINS 180
// distance between two different transitions in bins
// histogram signatures that don't comply with this restriction are regarded as noise and discarded
#define DISTANCE_TRANSTION 3
// every transition with count <= TRANSITION_NOISE_CUTOFF is considered to be noise
#define TRANSITION_NOISE 2


#ifndef PI
#ifdef M_PI
#define PI M_PI
#else
#define PI 3.14159265358979323846
#endif
#endif


#ifndef min
#define min(a,b) (a<=b?a:b)
#endif

#ifndef max
#define max(a,b) (a>=b?a:b)
#endif

namespace Panorama {

	class Transition {
	public:
		unsigned char firstColor,secondColor;
		int transitionCount;

		Transition();
		Transition(unsigned char color1,unsigned char color2);

		Transition& operator=(const Transition& other) 
		{
			memcpy((void *)this, (const void *)&other, sizeof(Transition));
			return *this;
		}

		bool compatible(Transition &other);
		void merge(Transition &other);
	};

	class HistogramSignature {
	public:
		bool empty;
		bool valid;
		int color[2][TRANSITION_BINS][MAX_CLUSTERS];

		HistogramSignature();

		HistogramSignature& operator=(const HistogramSignature& other) 
		{
			memcpy((void *)this, (const void *)&other, sizeof(HistogramSignature));
			return *this;
		}

		void flush();
		void add(Transition &other,double verticalAngle);
		void compress();
	};

	class Signature {
	public:
		Transition transition[TRANSITIONS];
		int transitions;
		int totalTransitionCount;
		int signatureCount;

		Signature();

		Signature(HistogramSignature &histogram);

		Signature& operator=(const Signature& other) 
		{
			memcpy((void *)this, (const void *)&other, sizeof(Signature));
			return *this;
		}

		bool compatible(Signature &other);
		void merge(Signature &other);
	};

	class Sector {
	public:
		bool empty;
		Signature signature[SIGNATURES];
		int signatures;
		int totalSignatureCount;

		Sector();
		void flush();
		void appendOrMerge(Signature &other);		
		void sortSignaturesOnFrequency();
	};

	class HistogramPanorama {
	public:
		HistogramSignature histogram[MAX_SECTORS];

		void flush();
	};

	class PanoramaView {
	public:
		Sector sector[MAX_SECTORS];
		int filledSectors;

		PanoramaView();
		PanoramaView(HistogramPanorama &histogramPanorama);
		void flush();
	};
}

class SectorHistogram {
	public:
		bool empty;
		int freq[MAX_CLUSTERS][MAX_CLUSTERS];
		int sum;						// transitions seen in total
		int horizon;

		SectorHistogram();
		void reset();
		void initialize();
		void insertTransition(unsigned char bottom,unsigned char top);
};

class PanoramaHistogram {
	public:
		SectorHistogram sector[MAX_SECTORS];
		SectorHistogram sum;

		PanoramaHistogram();
		void reset();
		void insertTransition(int sector,unsigned char bottom,unsigned char top);

};



In& operator>>(In& stream,PanoramaHistogram& panoramaHistogram);
Out& operator<<(Out& stream, const PanoramaHistogram& panoramaHistogram);

In& operator>>(In& stream,Panorama::PanoramaView& panoramaView);
Out& operator<<(Out& stream, const Panorama::PanoramaView& panoramaView);

#endif// __PanoramaLibrary_h_

/*
*
*/
