/**
* @file SLAMData.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file SLAMData.cpp
*
* contains the implementation of the class SLAMData
*/

#include "SLAMData.h"
#include "Tools/Debugging/Debugging.h"

#ifndef PI
#ifdef M_PI
#define PI M_PI
#else
#define PI 3.14159265358979323846
#endif
#endif

SLAMData::SLAMData():
	currentAction(resetImageProcessor)
{
	actionIteration = 0;
	currentAction = lastAction = lastSlamModeOut = slamModeOut = SLAMData::standby;
	learningPoseX = learningPoseY = learningPosePhi = 0;
	localizationMode = SLAMData::probabilistic;
	bearingMode = SLAMData::estimateBearing;
}

void SLAMData::requestAction(SLAMAction action) {
//	OUTPUT(idText,text,"SLAMData.currentAction="<<getActionName(action))  ;
	currentAction = action;
	lastAction = currentAction;
	actionIteration = 0;

}

void SLAMData::updateFromXablsOutputSymbols() {
	// Xabsl action request?
	if(lastSlamModeOut != slamModeOut) {
		requestAction(SLAMAction(slamModeOut));
		lastSlamModeOut = slamModeOut;
	}

	// update learning pose
	learningPose.translation.x = learningPoseX * 500; // 100cm grid seems reasonable
	learningPose.translation.y = learningPoseY * 500; // 100cm grid seems reasonable
	learningPose.rotation = learningPosePhi * PI/180;
	while(learningPose.rotation>PI) learningPose.rotation-=2*PI;
	while(learningPose.rotation<=-PI) learningPose.rotation+=2*PI;
}

void SLAMData::processingAndBusy() {
	if(lastAction==currentAction) 
		actionIteration++;
	else 
		requestAction((SLAMAction)currentAction);
}

void SLAMData::actionCompleted(bool success) {
	switch(currentAction) {
		case resetImageProcessor:
			requestAction(resetSelfLocator);
			break;

		case resetSelfLocator:
			requestAction(standby);
			break;

		case loadClusters:
			if(success) 
				requestAction(loadPanorama);
			else
				requestAction(recordColors);
			break;

		case recordColors:
			requestAction(clusterColors);
			break;

		case clusterColors:
			requestAction(saveClusters);
			break;

		case saveClusters:
			requestAction(standby);
			break;

		case loadPanorama:
			if(success) 
				requestAction(localize);
			else
				requestAction(recordPanorama);
			break;

		case recordPanorama:
			requestAction(standby);
			break;

		case savePanorama:
			requestAction(localize);
			break;

		case localize:
			requestAction(standby);
			break;

		case normalize:
			requestAction(standby);
			break;

	}
}

In& operator>>(In& stream,SLAMData& SLAMData)
{
  stream.read(&SLAMData,sizeof(SLAMData));
  return stream;
}

Out& operator<<(Out& stream, const SLAMData& SLAMData)
{
  stream.write(&SLAMData,sizeof(SLAMData));
  return stream;
}
  
/*
 * Change log :
 * 
 * $Log: TaskVisionData.cpp,v $
 * Revision 1.1  2004/06/23 12:17:15  floris
 * met taskVisionData
 *
 * Revision 1.3  2004/06/08 15:39:16  andrew
 * Renamed GT2003 to DT2004
 *
 * Revision 1.2  2004/02/27 11:31:16  wcaarls
 * Import of Utrecht changes as of 26-02-2004.
 *
 * Revision 1.1.1.1  2004/01/15 12:30:36  bottens
 *
 *
 * Revision 1.1.1.1  2003/07/02 09:40:22  cvsadm
 * created new repository for the competitions in Padova from the 
 * tamara CVS (Tuesday 2:00 pm)
 *
 * removed unused solutions
 *
 * Revision 1.1  2002/09/10 15:26:39  cvsadm
 * Created new project DT2004 (M.L.)
 * - Cleaned up the /Src/DataTypes directory
 * - Removed Challenge Code
 * - Removed processing of incoming audio data
 * - Renamed AcousticMessage to SoundRequest
 *
 * Revision 1.1.1.1  2002/05/10 12:40:13  cvsadm
 * Moved GT2002 Project from ute to tamara.
 *
 * Revision 1.6  2002/02/13 22:43:02  roefer
 * First working versions of DefaultLandmarksPerceptor and MonteCarloSelfLocator
 *
 * Revision 1.5  2002/02/05 03:36:12  loetzsch
 * replaced direct member access by
 * inline const VALUE& get...() const   and
 * inline void set...(const Value&) methods.
 *
 * Revision 1.4  2002/01/13 12:16:16  AndySHB
 * no message
 *
 * Revision 1.3  2001/12/10 17:47:06  risler
 * change log added
 *
 */
