/**
* @file StatisticsDlgBar.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file StatisticsDlgBar.h
*
* Declaration of class CStatisticsDlgBar.
*
* @author Jurgen Sturm, Amsterdam
*/
#ifndef __StatisticsDlgBar_h_
#define __StatisticsDlgBar_h_

#define M_PI   3.14159265358979323846f
#define KIDNAP_DETECTION_FRAMES 200

#include "StdAfx.h"
#include "RobotControlDialogBar.h"
#include "Representations/Motion/OdometryData.h"
#include "Representations/Perception/LandmarksPercept.h"
#include "Representations/Perception/BallPercept.h"
#include "Representations/Perception/PlayersPercept.h"
#include "Representations/Perception/ObstaclesPercept.h"
#include "Representations/Perception/PSDPercept.h"
#include "Representations/Perception/LinesPercept.h"
#include "Representations/Perception/BodyPercept.h"
#include "Representations/Perception/CameraMatrix.h"
#include "Representations/Perception/SpecialPercept.h"
#include "Representations/Perception/EdgesPercept.h"
#include "Representations/Perception/ColorTable64.h"
#include "Representations/Perception/CollisionPercept.h"
#include "Representations/Cognition/RobotPose.h"
#include "Representations/Cognition/BallModel.h"
#include "Representations/Cognition/PlayerPoseCollection.h"
#include "Representations/Cognition/ObstaclesModel.h"
#include "Representations/Cognition/RobotState.h"
#include "Representations/RoboCup/GameControlData.h"
#include "Representations/Motion/MotionRequest.h"
#include "Tools/FieldDimensions.h"

/**
* A docking window for the statistics panel
*
* @author Jurgen Sturm, Amsterdam
*/
#define MAX_FRAMES 250
#define ox 45
#define oy 45
#define ow 22
#define oh 24
#define ocolumns 10

#define CAMERA_PIXEL_BORDER 10

void log(CString s);

// order is important.. idb_statistics
enum PerceptType { goalYellow,flagPinkYellow,flagYellowPink,flagPinkBlue,flagBluePink,goalBlue,maxPerceptType};
enum Visibility { visibleYes, visiblePartially, visibleNo, visibleUnknown };

const int predefinedPositions[11][3] = 
{	
	{0,0,0 },			// center
	{xPosOwnGoalpost,0,0 },			// own goal
	{xPosOwnPenaltyArea, yPosRightPenaltyArea, 45}, // yellow goal right corner
	{xPosOwnPenaltyArea, yPosLeftPenaltyArea, 315}, // yellow goal left corner
	{-centerCircleRadius,0,0 },	// red striker initial position
	{0,yPosLeftGroundline,270 },	// left re-entrance point
	{+centerCircleRadius,0,180 },	// blue striker initial position
	{0,yPosRightGroundline,90 },	// right re-entrance point
	{xPosOpponentPenaltyArea, yPosRightPenaltyArea, 135}, // blue goal right corner
	{xPosOpponentPenaltyArea, yPosLeftPenaltyArea, 225}, // blue goal left corner
	{xPosOpponentGoalpost,0,180 },	// opponent goal
};

class TrueData {
public:
	bool validCamera;
	bool validRobotPosition;
	CameraMatrix cameraMatrix;		
	CameraInfo cameraInfo;
	Pose2D trueRobotPosition;
	bool changed;
	bool kidnapped;
	bool validError;

	TrueData();
	void setCameraData(CameraMatrix &matrix,CameraInfo &info);
	void setTrueRobotPosition(Pose2D &trueRobotPosition);
	void setKidnapped();
	void setRecovered();
};

class LandmarkStatistics {
public:
	int TruePositives;
	int TrueNegatives;
	int FalsePositives;
	int FalseNegatives;
	int TotalCount;
	int framesSinceLastLandmark;

	LandmarkStatistics();
};

class LocatorStatistics {
public:
	int WithinRange;
	int TotalCount;
	double SumErrorDist, SumErrorPhi;

	LocatorStatistics();
};


class LandmarkPercept {
public:
	bool changed;
	bool seenPerceptValid;
	bool trueAbsolutePositionValid;
	Pose2D trueAbsolute;
	Pose2D trueRelative;
	Pose2D seenRelative;
	Visibility seenVisibility;
	Visibility trueVisibility;
	LandmarkStatistics statistics;
	PerceptType type;

	LandmarkPercept();
	void setSeenLandmark(Pose2D &relativePosition);
	void setLandmarkNotSeen();
	void setLandmarkTruePosition(PerceptType atype,Pose2D &truePosition);
	void updateVisibility(TrueData &data);
	void updateStatistics(LandmarkPercept &previousPercept);
	bool LandmarkPercept::checkVisibility(
					Vector3<int> &relativePositionInt, int testInsideOutside,
					CameraMatrix &cameraMatrix, CameraInfo &cameraInfo);
	Visibility LandmarkPercept::checkVisibility(
					Pose2D &relativePosition, 
					Vector2<double> lowerLeft, Vector2<double> upperRight, 
					CameraMatrix &cameraMatrix, CameraInfo &cameraInfo);
};

class LocatorPercept {
public:
	bool changed;
	bool estimatedPerceptValid;
	Pose2D estimatedPosition;
	bool withinRange;
	bool withinRangeValid;
	double errDist,errPhi;
	bool validError;
	LocatorStatistics statistics;

	LocatorPercept();

	void setEstimatedPosition(Pose2D &position);
	void updatePrecision(TrueData &data,double distBound);
	void updateStatistics(LocatorPercept &previousPercept);
};

class SingleFrame {
public:
	bool changed;
	bool isEmpty;
	unsigned long frame;
	TrueData trueData;
	LandmarkPercept landmark[maxPerceptType];
	LocatorPercept locator;
	int framesSinceLastLandmark;

	SingleFrame();

	void spreadTrueData(SingleFrame &previousFrame);
	void updateStatistics(SingleFrame &previousFrame);
	void evaluate(double distBound);
	void removeChangedFlag();
	bool hasBeenChanged();

	void setSeenLandmark(PerceptType type,Pose2D &relativePosition);
	void setLandmarkNotSeen(PerceptType type);
	void setLandmarkTruePosition(PerceptType type, Player player);
	void setLocatorPosition(Pose2D &estimatedRobotPosition);
	void setCameraData(CameraMatrix &matrix,CameraInfo &info);
	void setTrueRobotPosition(Pose2D &trueRobotPosition);
	void setKidnapped();
	void setRecovered();

};

class MainBuffer {
public:
	SingleFrame frame[MAX_FRAMES];
	unsigned long latestFrame;
	int latestIndex;

	bool kidnapMode;

	MainBuffer();

	SingleFrame * getSingleFrame(unsigned long frame,bool forceExistance=true);
	SingleFrame * getLatest();

	void spreadTrueData();
	void updateStatistics();
	void evaluate(double distBound);
	void removeChangedFlag();

	void setSeenLandmark(unsigned long frame, PerceptType type,Pose2D &relativePosition);
	void setLandmarkNotSeen(unsigned long frame, PerceptType type);
	void setLandmarkTruePosition(unsigned long frame, PerceptType type, Player player);
	void setLocatorPosition(unsigned long frame, Pose2D &estimatedRobotPosition);
	void setCameraData(unsigned long frame, CameraMatrix &matrix,CameraInfo &info);
	void setTrueRobotPosition(unsigned long frame, Pose2D &trueRobotPosition);
	void setKidnapped(unsigned long frame);

};


class CStatisticsDlgBar : public CRobotControlDialogBar
{
protected:
  MainBuffer buffer;

  PerceptType selectedType;
  unsigned long selectedFrame;
  unsigned long framePaintedAt[ocolumns];
  CFont fSmall,fError,fNormal,fGiant,fBig;
  CBrush brushOrange,brushLime,brushGreen,brushBlue,brushRed,brushFade[6],brushGrey;

  
public:
  CStatisticsDlgBar();
  ~CStatisticsDlgBar();

  //{{AFX_DATA(CStatisticsDlgBar)
  enum { IDD = IDD_DIALOG_BAR_STATISTICS};
  CStatic  m_generalFrames;
  CStatic  m_generalSeconds;
  CButton m_generalReset;
  CButton m_generalTimer;
  CEdit  m_positionTrueX;
  CEdit  m_positionTrueY;
  CEdit  m_positionTruePhi;
  CEdit  m_positionWithinDist;
  CStatic m_History;
  CStatic m_History_Estimated;
  CStatic m_History_True;
  CStatic m_History_EstimatedRelative;
  CStatic m_History_TrueRelative;
  CComboBox m_positionSource;
  CComboBox m_positionPredefined;
  //}}AFX_DATA
  
  //{{AFX_VIRTUAL(CStatisticsDlgBar)
protected:
  virtual void DoDataExchange(CDataExchange* pDX);  
  virtual BOOL OnInitDialog();
  //}}AFX_VIRTUAL
public:  
  virtual bool handleMessage(InMessage& message);
  virtual bool handleCommand(UINT command);
  virtual void updateUI(CCmdUI* pCmdUI);
  virtual void OnConfigurationLoad(CString sectionName);
  virtual void OnConfigurationSave(CString sectionName);

  void OnLButtonDown(UINT nFlags, CPoint point);
  void handleMessagePercepts(InMessage& message);
  void handleMessageWorldState(InMessage& message);
  void handleMessageOracledWorldState(InMessage& message);
  void handleMessageManualWorldState(unsigned long frameNumber);

  void paintBackground(CPaintDC &dc);
  void paintSelectedFrame(CPaintDC &dc);
  void paintPercepts(CPaintDC &dc);
  void paintStatistics(CPaintDC &dc);
  void showSummary();


  void setStaticInt(CStatic &control, int i);
  void setStaticDouble(CStatic &control, double i);

  double getDouble(CEdit &control);
  CString getString(CEdit &control);

protected:  
  int captureCount;
  unsigned long predefinedPositionsFound[11];
  unsigned long startingFrameAtPosition[11];
  bool kidnapMode;

  int activePosition;	// -2  disabled

public:
  //{{AFX_MSG(CStatisticsDlgBar)
	afx_msg void OnResetButton();
	afx_msg void OnTimerButton();
  	afx_msg void OnPaint();
	afx_msg void OnReferenceSourceChange();
	afx_msg void OnPredefinedPositionChange();
//}}AFX_MSG
  DECLARE_MESSAGE_MAP();
};

#endif //__StatisticsDlgBar_h_

/*
* Change Log:
*
* $Log$
*
*/
